<?php
$layElementTypes = ["project","thumbnail","text","embed","html","shortcode","hr","vl","thumbnailgrid","elementgrid","carousel","img","html5video","video","stack","productsgrid","addtocartbutton","textfooter","projectindex","socialmediaicons",'news','marquee'];

class LayCol{

    public $col_classes = 'col';
    public $col_id = '';
    public $type = '';
    public $inline_css = '';
    public $yvel = '';
    public $offset_data_attrs = '';
    public $data_attrs = '';

    public function __construct($col, $inside100vhRow = false){
        // CSS
        $hasInlineCSS = false;
        $css = '';
        $spaceabove = array_key_exists( 'spaceabove', $col ) ? $col['spaceabove'] : 0;
        $spacebelow = array_key_exists( 'spacebelow', $col ) ? $col['spacebelow'] : 0;
        $offsetx = array_key_exists('offsetx', $col) ? $col['offsetx'] : 0;
        $offsety = array_key_exists('offsety', $col) ? $col['offsety'] : 0;
        $absolute_position = array_key_exists('absolute_position', $col) ? $col['absolute_position'] : false;

        $absolute_position_class_exists = false;
        if( array_key_exists('classes', $col) && strpos($col['classes'], 'absolute-position') !== false ) {
            $absolute_position_class_exists = true;
        }

        $place_at_end_of_col = array_key_exists('placeAtEndOfCol', $col) ? $col['placeAtEndOfCol'] : false;

        $spaceaboveMu = array_key_exists('spaceaboveMu', $col ) ? $col['spaceaboveMu'] : LayFrontend_Options::$spaces_mu;
        $spacebelowMu = array_key_exists('spacebelowMu', $col ) ? $col['spacebelowMu'] : LayFrontend_Options::$spaces_mu;
        $offsetxMu = array_key_exists('offsetxMu', $col ) ? $col['offsetxMu'] : LayFrontend_Options::$offsets_mu;
        if( $offsetxMu == '%' ) {
            $offsetxMu = 'vw';
        }
        $offsetyMu = array_key_exists('offsetyMu', $col ) ? $col['offsetyMu'] : LayFrontend_Options::$offsets_mu;

        if( $offsetx == 0 && $offsety == 0 ){
            $this->col_classes .= ' no-offset';
        }else{
            $this->col_classes .= ' has-offset';
        }

        if(array_key_exists('opacity', $col)){
            $opacity = $col['opacity'];
            $css .= 'opacity:'.Lay_LayoutFunctions::replaceCommaWithDot($opacity).';';
            $hasInlineCSS = true;
        }

        if( $spaceabove != 0 ) {
            // need vw so padding will be okay inside a stack element
            if($spaceaboveMu == '%') {
                $spaceaboveMu = 'vw';
            }
            $css .= 'padding-top: '.$spaceabove.$spaceaboveMu.';';
            $hasInlineCSS = true;
        }
        if( $spacebelow != 0 ) {
            // need vw so padding will be okay inside a stack element
            if($spacebelowMu == '%') {
                $spacebelowMu = 'vw';
            }
            $css .= 'padding-bottom: '.$spacebelow.$spacebelowMu.';';
            $hasInlineCSS = true;
        }

        // Handle spaceleft and spaceright
        $spaceleft = array_key_exists('spaceleft', $col) ? $col['spaceleft'] : 0;
        $spaceright = array_key_exists('spaceright', $col) ? $col['spaceright'] : 0;
        $spaceleftMu = array_key_exists('spaceleftMu', $col) ? $col['spaceleftMu'] : LayFrontend_Options::$spaces_mu;
        $spacerightMu = array_key_exists('spacerightMu', $col) ? $col['spacerightMu'] : LayFrontend_Options::$spaces_mu;

        if($spaceleft != 0) {
            if($spaceleftMu == '%') {
                $spaceleftMu = 'vw';
            }
            $css .= 'padding-left: '.$spaceleft.$spaceleftMu.';';
            $hasInlineCSS = true;
        }
        if($spaceright != 0) {
            if($spacerightMu == '%') {
                $spacerightMu = 'vw';
            }
            $css .= 'padding-right: '.$spaceright.$spacerightMu.';';
            $hasInlineCSS = true;
        }

        // Handle margin
        if(array_key_exists('margin', $col)) {
            $margin = $col['margin'];
            $topMu = $margin['topMu'] == '%' ? 'vw' : $margin['topMu'];
            $bottomMu = $margin['bottomMu'] == '%' ? 'vw' : $margin['bottomMu'];
            $css .= 'margin-top: '.$margin['top'].$topMu.';';
            $css .= 'margin-bottom: '.$margin['bottom'].$bottomMu.';';
            $hasInlineCSS = true;
        }

        // Handle border radius
        if(array_key_exists('borderRadius', $col)) {
            $br = $col['borderRadius'];
            $css .= 'border-radius: '.$br['topLeft'].$br['topLeftMu'].' '.$br['topRight'].$br['topRightMu'].' '.$br['bottomRight'].$br['bottomRightMu'].' '.$br['bottomLeft'].$br['bottomLeftMu'].';';
            $css .= 'overflow: hidden;';
            $hasInlineCSS = true;
        }

        // Handle border
        if(array_key_exists('border', $col) && $col['border']['enableBorder']) {
            $border = $col['border'];
            $borderColor = $border['borderColor'];
            $borderOpacity = $border['borderOpacity'] / 100;
            $borderWidth = $border['borderWidth'];
            
            // Convert hex to rgba for opacity
            $r = hexdec(substr($borderColor, 1, 2));
            $g = hexdec(substr($borderColor, 3, 2));
            $b = hexdec(substr($borderColor, 5, 2));
            
            $css .= 'border-width: '.$borderWidth['top'].'px '.$borderWidth['right'].'px '.$borderWidth['bottom'].'px '.$borderWidth['left'].'px;';
            $css .= 'border-color: rgba('.$r.','.$g.','.$b.','.$borderOpacity.');';
            $css .= 'border-style: solid;';
            $hasInlineCSS = true;
        }

        // Handle background color
        if(array_key_exists('bgColor', $col) && $col['bgColor']['enableBgColor']) {
            $bgColor = $col['bgColor'];
            $bgColorHex = $bgColor['bgColor'];
            $bgOpacity = $bgColor['bgOpacity'] / 100;
            
            // Convert hex to rgba for opacity
            $r = hexdec(substr($bgColorHex, 1, 2));
            $g = hexdec(substr($bgColorHex, 3, 2));
            $b = hexdec(substr($bgColorHex, 5, 2));
            
            $css .= 'background-color: rgba('.$r.','.$g.','.$b.','.$bgOpacity.');';
            $hasInlineCSS = true;
        }

        // Handle shadow
        if(array_key_exists('shadow', $col) && $col['shadow']['enableShadow']) {
            $shadow = $col['shadow'];
            $shadowColor = $shadow['color'];
            $shadowOpacity = $shadow['opacity'] / 100;
            
            // Convert hex to rgba for opacity
            $r = hexdec(substr($shadowColor, 1, 2));
            $g = hexdec(substr($shadowColor, 3, 2));
            $b = hexdec(substr($shadowColor, 5, 2));
            
            $inset = $shadow['inset'] ? 'inset ' : '';
            $css .= 'box-shadow: '.$inset.$shadow['offsetX'].'px '.$shadow['offsetY'].'px '.$shadow['blur'].'px '.$shadow['spread'].'px rgba('.$r.','.$g.','.$b.','.$shadowOpacity.');';
            $hasInlineCSS = true;
        }

        if( $offsetx != 0 || $offsety != 0 ) {
            $this->offset_data_attrs .= ' data-offsetx="'.$offsetx.'" data-offsetxmu='.$offsetxMu.' data-offsety="'.$offsety.'" data-offsetymu='.$offsetyMu.'';
            $y_offset_markup = $offsety.$offsetyMu;

            // need to use translate3d so on safari when fs slider is active, z-index will work properly
            $css .= 'transform:translate3d('.$offsetx.$offsetxMu.', '.$y_offset_markup.', 0);';
            $hasInlineCSS = true;
        }

        // TYPE
        $this->type = $col['type'];
        // for backwards compatibility
        $isProjectLink = array_key_exists( 'postid', $col ) ? true : false;
        if(($this->type == 'img' && $isProjectLink) || $this->type == 'postThumbnail'){
            $this->type = 'project';
        }
        
        //  ID
        $this->col_id = array_key_exists('html_id', $col) ? 'id="'.$col['html_id'].'"' : '';

        // instead of only in placefree mode, we now use a zIndex always, for when horizontalGrid is active or normal mode 
        if( array_key_exists('zIndex', $col) ) {
            $hasInlineCSS = true;
            $css .= 'z-index:'.((int)$col['zIndex']+2).';';
        }

        // CLASSES
        if( array_key_exists('placeFreely', $col) ) {
            $hasInlineCSS = true;
            $this->col_classes .= ' place-freely';
            $css .= 'position:absolute;left:'.$col['left'].'%;top:calc((100vw - var(--scrollbarWidth)) * '.($col['top'] / 100).');width:'.$col['width'].'%;';
        } else {
            $this->col_classes .= ' place-normal';
            if( $absolute_position_class_exists ) {
                // doing ' push-'.$col['push'] should work too, but
                // pushes are calculated wrong in older lay theme versions for absolute position elements
                // because these elements were not absolute position before
                // so we just do this for backwards compatibility

                // dont remove this we need it for position absolute!
                $this->col_classes .= ' push-'.$col['col'];
            } else {
                $this->col_classes .= ' push-'.$col['push'];
            }
            $this->col_classes .= ' span-'.$col['colspan'];

            $this->col_classes .= ' colstart-'.$col['col'];
            $this->col_classes .= ' colend-'.($col['col'] + $col['colspan']);
            $this->col_classes .= $place_at_end_of_col ? ' place-at-end-of-col' : '';
    
            if( array_key_exists('frameOverflow', $col) ) {
                switch( $col['frameOverflow'] ){
                    case 'left':
                        $this->col_classes .= ' frame-overflow-left';
                        break;
                    case 'both':
                        $this->col_classes .= ' frame-overflow-both';
                        break;
                    case 'right':
                        $this->col_classes .= ' frame-overflow-right';
                        break;
                    case '':
                        $this->col_classes .= ' no-frame-overflow';
                    break;
                    default:
                        $this->col_classes .= ' no-frame-overflow';
                    break;
                }
            } else {
                $this->col_classes .= ' no-frame-overflow';
            }

            if( array_key_exists('horizontalIndex', $col) && array_key_exists('horizontalGrid', $col) ) {
                $this->col_classes .= ' uses-horizontal-grid';
                if( array_key_exists('stickytop', $col) ) {
                    // horizontal grid items still need align top when using stickytop so sticky works
                    $this->col_classes .= ' align-'.$col['align'];
                }
            } else {
                $this->col_classes .= ' align-'.$col['align'];
            }

        }



        $this->col_classes .= array_key_exists('classes', $col) ? ' '.$col['classes'] : '';


        if( array_key_exists('yvel', $col) && $col['yvel'] != 1 && LayFrontend_Options::$simple_parallax ){
            $this->col_classes .= ' has-parallax';
        }else{
            $this->col_classes .= ' no-parallax';
        }

        $this->col_classes .= ' type-'.$this->type;
        $this->col_classes .= ' id-'.$col['relid'];


        $lightboxoff = array_key_exists('lightboxoff', $col) && $col['lightboxoff'] == true && $this->type == 'img' ? 'lightboxoff' : '';

        $this->col_classes .= ' '.$lightboxoff;

        if( $this->type == 'carousel' ) {
            if( array_key_exists('carousel_options', $col) ){
                if( array_key_exists('fixedHeight', $col['carousel_options']) ){
                    if( $col['carousel_options']['fixedHeight'] == '100vh' ) {
                        $this->col_classes .= ' contains_100vh-carousel';
                    }else{
                        $this->col_classes .= ' no_100vh-carousel';
                    }
                }else{
                    $this->col_classes .= ' no_100vh-carousel';
                }
            }else{
                $this->col_classes .= ' no_100vh-carousel';
            }
        }


        // YVEL
        if( array_key_exists( 'yvel', $col ) && $col['yvel'] != 0 ){
            $this->yvel = 'data-yvel="'.$col['yvel'].'"';
        }

        $stickytop = array_key_exists( 'stickytop', $col ) ? $col['stickytop'] : false;
        // $this->data_attrs = $stickytop;
        if( $stickytop !== false && $stickytop !== null && $stickytop !== 'null' && $stickytop !== '' ){
            $this->col_classes .= ' lay-sticky';

            $hasInlineCSS = true;
            $css .= 'position:sticky;top:'.$stickytop.'px;';

            // $this->data_attrs .= ' data-sticky-top="'.$stickytop.'"';
            // $this->data_attrs .= ' data-sticky-for="'.( array_key_exists( 'stickyfor', $col ) ? $col['stickyfor'] : 1 ).'"';
        } else {
            $this->col_classes .= ' no-sticky';
        }

        if( array_key_exists( 'fs_maxdim', $col ) ) {
            $this->data_attrs .= ' data-fullscreenslider-maxdim="'.$col['fs_maxdim'].'"';
        }
        
        if( $hasInlineCSS ) {
            $this->inline_css = 'style="'.$css.'"';
        }

        if( $this->type == 'thumbnailgrid' || $this->type == 'elementgrid' || $this->type == 'productsgrid' ) {
            if( array_key_exists('config', $col) ) {
                if( $col['config']['layoutType'] == 'masonry' ) {
                    $this->col_classes .= ' contains-masonry-layout';
                }
            }
        }
        
        global $layElementTypes;
        if( !in_array($this->type, $layElementTypes) ) {
            $this->col_classes .= ' cmb2-element';
        }

    }

    public function getOpeningMarkup(){
        return '<div class="'.$this->col_classes.'" '.$this->col_id.' '.$this->data_attrs.' data-type="'.$this->type.'" '.$this->inline_css.' '.$this->yvel.' '.$this->offset_data_attrs.'>';
    }

    public function getClosingMarkup(){
        return '</div>';
    }

}