<?php

class LayElementGrid{

    public $el;
    public $config;
    public $elements;
    public $device;

    public function __construct($el, $device = 'desktop'){
        
        $this->el = $el;
        $this->device = $device;
    }

    public function getMarkup(){
        $this->config = $this->el['config'];
        $randomorder = array_key_exists('randomOrder', $this->config) ? $this->config['randomOrder'] : false;
        $this->elements = $this->config['elements'];
        if($randomorder){
            shuffle($this->elements);
        }
        $markup = '';
        $device = $this->device;

        $sameheight = $this->config['layoutType'] == 'same-height';
        $height = false;
        if($sameheight){
            $height = true;
        }

        $lightboxoff_all_images = array_key_exists('lightboxoff', $this->el) ? $this->el['lightboxoff'] : false;

        foreach( $this->elements as $el ) {
            $caption = '';
            switch( $el['type'] ) {
                case 'img':
                case 'html5video':
                    $el['captionMarkup'] = LayElFunctions::getCaptionMarkup($this->el);
            }
            $element = false;
            $sizeDesktop = array_key_exists('desktop', $el) ? $el['desktop']['size'] : 1;
            $sizeTablet = array_key_exists('tablet', $el) ? $el['tablet']['size'] : 1;
            $sizePhone = array_key_exists('phone', $el) ? $el['phone']['size'] : 1;

            switch( $el['type'] ) {
                case "postThumbnail":
                    $element = new LayThumbnail($el, false, $height);
                break;
                case "text":
                    $element = new LayText($el, $device);
                break;
                case "img":
                    if(!array_key_exists('lightboxoff', $el)) {
                        $el['lightboxoff'] = $lightboxoff_all_images;
                    }
                    $element = new LayImg($el, $height);
                break;
                case "html5video":
                    $element = new LayHTML5Video($el, $height);
                break;
                case "video":
                    $element = new LayVideo($el, $height);
                break;
                case "carousel":
                    if($sameheight){
                        // todo: carousels in sameheight elementgrid should be able to have different heights for desktop, tablet, phone
                        // here, we only set the desktop height :D
                        $h = $this->config['desktop']['elementHeight'];
                        $el['carousel_options']['fixedHeight'] = $h.'px';
                        $el['carousel_options']['size'] = 'fixedHeight';
                        // also we set this class to prevent the carousel js from setting widths with js as this breaks the carousel width :D
                        $el['carousel_options']['extraclass'] = 'in-same-height-elementgrid';
                        $element = new LayCarousel($el);

                    } else {
                        $element = new LayCarousel($el);
                    }
                break;
            }
            if( $element ) {
                $markup .= 
                '<div class="element-wrap" data-size-desktop="'.$sizeDesktop.'" data-size-tablet="'.$sizeTablet.'" data-size-phone="'.$sizePhone.'">'
                    .$element->getMarkup().
                '</div>';
            }
        }

        unset( $this->config['elements'] );

        $caption = LayElFunctions::getCaptionMarkup($this->el);
        // why do we have a wrapping div here :D
        return 
        '<div>
            <div class="elements-collection-region '.$this->config['layoutType'].'" data-config="'.htmlspecialchars(json_encode($this->config), ENT_QUOTES, 'UTF-8').'">
                <div class="element-collection">'
                    .$markup.
                '</div>
            </div>
            '.$caption.'
        </div>';
    }

}