<?php
require get_template_directory().'/frontend/assets/php/layout_functions.php';
require get_template_directory().'/frontend/assets/php/footer/footer.php';
require get_template_directory().'/frontend/assets/php/row/row_functions.php';
require get_template_directory().'/frontend/assets/php/row/row.php';
require get_template_directory().'/frontend/assets/php/col/col.php';
require get_template_directory().'/frontend/assets/php/elements/el_functions.php';
require get_template_directory().'/frontend/assets/php/elements/thumbnailgrid_functions.php';
require get_template_directory().'/frontend/assets/php/elements/el.php';
require get_template_directory().'/frontend/assets/php/cover/cover.php';
require get_template_directory().'/frontend/assets/php/cover/cover_functions.php';

// todo: add lay-col-needs-leftframe-margin
class Lay_Layout{

    public function __construct(){
        add_action( 'wp_ajax_get_laytheme_layout', array( $this, 'getLayoutAjax' ) );
        add_action( 'wp_ajax_nopriv_get_laytheme_layout', array( $this, 'getLayoutAjax' ) );
    }

    public static function getLayoutAjax(){
        $id = $_POST['id'];
        $type = $_POST['type'];
        $password = $_POST['password'];
        $is_project_overlay = $_POST['is_project_overlay'];

        $layout = Lay_Layout::getLayout($id, $type, $password, true, true, $is_project_overlay);

        $overlays = Lay_Overlays::getOverlays($layout['markup']);
        echo json_encode(array('layout' => $layout, 'overlays' => $overlays));
        wp_die();
    }

    public static function getLayoutInit(){
        $data = Frontend::$current_type_id_slug_catid;
        $id = $data["id"];
        $type = $data["type"];
        $slug = $data["slug"];
        $layout = Lay_Layout::getLayout($id, $type, false);
        return $layout['markup'];
    }

    // I need layoutObj, type and obj for the newpageshown event
    // getLayout returns an array of "markup", "layoutObj", "type", "obj". These are needed for the newpageshown javascript event
    // It also returns a "notification" array member, used for password protection status, so my javascript can shake the password input on wrongpassword

    // todo: split this function? getLayout

    // when $getFullLayout == false, then cover and footer are not returned
    // this is for the news full content layout
    public static function getLayout($id, $type, $password = '', $getFullLayout = true, $getImageHover = true, $is_project_overlay = 'false') {

        if($is_project_overlay == '') {
            $is_project_overlay = 'false';
        }


        if( ($type == 'project' || $type == 'page' || $type == 'lay_news' || $type == 'product') && post_password_required( $id ) ) {
            return array('password' => 'protected');
        }

        // test
        $desktop_json = '';
        $cpl_json = '';
        $desktop_footer_json = '';
        $cpl_footer_json = '';

        // for newpageshown event:
        $layoutObj;
        $obj = array( 
            'id' => '',
            'type' => '',
            'slug' => '',
            'catid' => '',
            'footerid' => '',
            'title' => ''
        );

        // cover
        $cover_on_desktop = LayCoverFunctions::hasCover( $type, $id );
        $desktop_cover_is_100vh_set_by_user = false;
        $cover_on_phone = $cover_on_desktop && LayFrontend_Options::$cover_disable_for_phone == false ? true : false;

        // disable cover for desktop and phone when magnetic slides is active, because cover feature is incompatible with magnetic slides feature
        $magnetic_slides_active = false;
        if( class_exists('LayThemeMagneticSlides') && method_exists('LayThemeMagneticSlides', 'isActiveHere') && $getFullLayout == true ){
            $magnetic_slides_active = LayThemeMagneticSlides::isActiveHere($id, $type);
            if( $magnetic_slides_active == true ) {
                $cover_on_desktop = false;
            }
        }
        $magnetic_slides_active_on_phone = false;
        if( class_exists('LayThemeMagneticSlides') && method_exists('LayThemeMagneticSlides', 'activeOnPhone') && $getFullLayout == true ){
            if( LayThemeMagneticSlides::activeOnPhone() == true && $magnetic_slides_active ) {
                $magnetic_slides_active_on_phone = true;
            }
            if( $magnetic_slides_active_on_phone == true ){
                $cover_on_phone = false;
            }
        }

        $scrollslider_active_and_using_packery = false;
        if( class_exists('LayThemeScrollSlider') && method_exists('LayThemeScrollSlider', 'isActiveHere') && $getFullLayout == true ){
            if( LayThemeScrollSlider::isActiveHere($id, $type) == true && get_option( 'ss_element_layout', 'custom' ) == 'custom') {
                $scrollslider_active_and_using_packery = true;
            }
        }

        $getOpeningAndClosingRowMarkup = true;

        if( $scrollslider_active_and_using_packery == true ) {
            $cover_on_desktop = false;
            $cover_on_phone = false;
            $getOpeningAndClosingRowMarkup = false;
        }

        if( $getFullLayout == false ) {
            $cover_on_desktop = false;
            $cover_on_phone = false;
        }

        switch($type){
            case "archive":
                $queried_obj = get_queried_object();
                // "shop" page is an archive
                $desktop_json = get_post_meta( $id, '_gridder_json', true );
                $cpl_json = get_post_meta( $id, '_phone_gridder_json', true );
                $obj['id'] = $id;
                // https://developer.wordpress.org/reference/functions/get_queried_object/
                // todo: should i use $queried_obj->slug; here instead? 
                $obj['slug'] = $queried_obj->name;
                $obj['type'] = 'archive';
                $obj['catid'] = '';
                $obj['title'] = $queried_obj->name;
            break;
            case "category":
                $desktop_json = get_option($id."_category_gridder_json", '');
                $cpl_json = get_option( $id.'_phone_category_gridder_json', '' );
                $term = get_term($id);
                $obj['id'] = $term->term_id;
                $obj['slug'] = $term->slug;
                $obj['type'] = 'category';
                $obj['catid'] = array($term->term_id);
                $obj['title'] = $term->name;


            break;
            case "project":
                $desktop_json = get_post_meta( $id, '_gridder_json', true );
                $cpl_json = get_post_meta( $id, '_phone_gridder_json', true );
                $post = get_post($id);
                $obj['id'] = $post->ID;
                $obj['slug'] = $post->post_name;
                $obj['type'] = 'project';
                $obj['catid'] = wp_get_post_categories($post->ID, array('fields'=>'ids'));
                $obj['title'] = $post->post_title;

            break;
            case "product":
                if(LayFrontend_Options::$gridder_active_for_products) {
                    $desktop_json = get_post_meta( $id, '_gridder_json', true );
                    $cpl_json = get_post_meta( $id, '_phone_gridder_json', true );
                    $post = get_post($id);
                    $obj['id'] = $post->ID;
                    $obj['slug'] = $post->post_name;
                    $obj['type'] = 'product';
                    $obj['catid'] = wp_get_post_categories($post->ID, array('fields'=>'ids'));
                    $obj['title'] = $post->post_title;
                }
            break;
            case "page":
                $desktop_json = get_post_meta( $id, '_gridder_json', true );
                $cpl_json = get_post_meta( $id, '_phone_gridder_json', true );
                $page = get_post($id);
                $obj['id'] = $page->ID;
                $obj['slug'] = $page->post_name;
                $obj['type'] = 'page';
                $obj['catid'] = '';
                $obj['title'] = $page->post_title;
            break;
            case "news":
            case "lay_news":
                $desktop_json = get_post_meta( $id, '_gridder_json', true );
                $cpl_json = get_post_meta( $id, '_phone_gridder_json', true );
                $lay_news = get_post($id);
                $obj['id'] = $lay_news->ID;
                $obj['slug'] = $lay_news->post_name;
                $obj['type'] = 'lay_news';
                $obj['catid'] = '';
                $obj['title'] = $lay_news->post_title;
            break;
        }

        // no content, like when someone didnt create a page yet but that page is shown on frontend
        // if( $desktop_json == '' ) {
        //     return array(
        //         'markup' => '<div class="lay-content lay-empty"></div>',
        //         'type' => $type,
        //         'layoutObj' => '',
        //         'obj' => '',
        //         'notification' => 'success'
        //     );
        // }

        $obj['footerid'] = Lay_Footer::getFooterId($obj['id'], $obj['type']);
        $footer_json_array = Lay_Footer::getFooterJSON($id, $type);

        if( is_array( $footer_json_array ) ) {
            $desktop_footer_json = array_key_exists('desktop', $footer_json_array) ? $footer_json_array['desktop'] : '';
            $cpl_footer_json = array_key_exists('phone', $footer_json_array) ? $footer_json_array['phone'] : '';
        }

        // decode JSON
        $desktop_arr = json_decode($desktop_json, true);
        $cpl_arr = json_decode($cpl_json, true);
        $footer_desktop_arr = json_decode($desktop_footer_json, true);
        $cpl_footer_arr = json_decode($cpl_footer_json, true);

        // for newpageshown event:
        $layoutObj = $desktop_arr;

        // prepare Rows
        $desktop_rows = false;
        $cpl_rows = false;
        $desktop_footer_rows = false;
        $cpl_footer_rows = false;
        $desktop_cover_row = false;
        $cpl_cover_rows = false;

        // markups
        $desktop_freely_placed_els = '';
        $desktop_cover_markup = false;
        $desktop_cover_freely_placed_els = false;
        $desktop_markup = false;
        $cpl_markup = false;
        $cpl_freely_placed_els = '';
        $cpl_cover_freely_placed_els = false;
        $cpl_cover_markup = false;
        $desktop_footer_markup = false;
        $footer_desktop_freely_placed_els = false;
        
        $cpl_footer_markup = false;
        $cpl_row_col_el_markup = false;
        $cpl_footer_freely_placed_els = false;

        // DESKTOP
        if( is_array($desktop_arr) ){
            $desktop_freely_placed_els = Lay_Layout::getFreelyPlacedCols($desktop_arr, $obj['id'], 'desktop', $cover_on_desktop ? 'outside-cover': 'all');
            $desktop_rows = Lay_Layout::prepareRows($desktop_arr, $cover_on_desktop);
        }
        // split up rows into cover row and desktop rows
        if( $cover_on_desktop && is_array( $desktop_rows ) ) {
            // https://www.php.net/manual/en/function.array-shift.php
            $desktop_cover_row = array_shift( $desktop_rows );
            $desktop_cover_is_100vh_set_by_user = Lay_LayoutFunctions::getRowIs100vh($desktop_cover_row);
            $desktop_cover_row['row100vh'] = true;
            $desktop_cover_row['rowcustomheight'] = '';
            $desktop_cover_row = array($desktop_cover_row);
            $desktop_cover_markup = Lay_Layout::getRowColElMarkup($desktop_cover_row, $obj['id'], 'desktop', $getImageHover, $getOpeningAndClosingRowMarkup);
            $desktop_cover_freely_placed_els = Lay_Layout::getFreelyPlacedCols($desktop_arr, $obj['id'], 'desktop', 'in-cover');
        }
        if( is_array($desktop_arr) ){
            $desktop_markup = Lay_Layout::getRowColElMarkup($desktop_rows, $obj['id'], 'desktop', $getImageHover, $getOpeningAndClosingRowMarkup);
        }

        // CPL
        if( is_array($cpl_arr) ){
            $cpl_freely_placed_els = Lay_Layout::getFreelyPlacedCols($cpl_arr, $obj['id'], 'phone', $cover_on_phone ? 'outside-cover': 'all');
            $cpl_rows = Lay_Layout::prepareRows($cpl_arr, $cover_on_phone);
        }
        // split up rows into cover row and phone rows
        if( $cover_on_phone && is_array($cpl_arr) ){
            // https://www.php.net/manual/en/function.array-shift.php
            $cpl_cover_rows = array_shift( $cpl_rows );
            $cpl_cover_rows['row100vh'] = true;
            $cpl_cover_rows['rowcustomheight'] = '';
            $cpl_cover_rows = array($cpl_cover_rows);
            $cpl_cover_markup = Lay_Layout::getRowColElMarkup($cpl_cover_rows, $obj['id'], 'phone', $getImageHover, $getOpeningAndClosingRowMarkup);
            $cpl_cover_freely_placed_els = Lay_Layout::getFreelyPlacedCols($cpl_arr, $obj['id'], 'phone', 'in-cover');
        }
        if( is_array($cpl_arr) ){
            $cpl_row_col_el_markup = Lay_Layout::getRowColElMarkup($cpl_rows, $obj['id'], 'phone', $getImageHover, $getOpeningAndClosingRowMarkup);
        }

        // FOOTER
        if( is_array($footer_desktop_arr) && $getFullLayout == true ){
            $desktop_footer_rows = Lay_Layout::prepareRows($footer_desktop_arr, false);
            $desktop_footer_markup = Lay_Layout::getRowColElMarkup($desktop_footer_rows, $obj['id'], 'desktop', $getImageHover, $getOpeningAndClosingRowMarkup);
            $footer_desktop_freely_placed_els = Lay_Layout::getFreelyPlacedCols($footer_desktop_arr, $obj['id'], 'desktop');
        }
        if( is_array($cpl_footer_arr) && $getFullLayout == true ){
            $cpl_footer_rows = Lay_Layout::prepareRows($cpl_footer_arr, false);
            $cpl_footer_markup = Lay_Layout::getRowColElMarkup($cpl_footer_rows, $obj['id'], 'phone', $getImageHover, $getOpeningAndClosingRowMarkup);
            $cpl_footer_freely_placed_els = Lay_Layout::getFreelyPlacedCols($cpl_footer_arr, $obj['id'], 'phone');
        }

        // Custom Phone Layout
        $cpl_exists_html_class = 'nocustomphonegrid';
        $cover_cpl_class = '';
        if( LayFrontend_Options::$phone_layout_active && is_array($cpl_arr) ) {
            // Cover CPL
            $cover_cpl_class = 'cpl-nocover';
            $cpl_cover_rowgutter_css = '';
            $cover_cpl_css = '';
            // todo: && !isFullscreenSliderActive()
            // if( cover_controller.hasCover(type, obj.id) && !isFullscreenSliderActive() ){
            if( $cover_on_phone ) {
                $cover_cpl_class = 'cpl-hascover';
                $cover_cpl_css = Lay_LayoutFunctions::getGridCSS($cpl_arr, '.cover-region-phone', true);
                $cover_cpl_css .= Lay_LayoutFunctions::getHorizontalGridCSS($cpl_arr, '.cover-region-phone', true);
                // set main grid padding-top to first rowgutter
                $cpl_cover_rowgutter_css = Lay_LayoutFunctions::getFirstRowGutterAsPaddingTopForCoverCSS($cpl_arr, '#custom-phone-grid.id-'.$id, true);
                // need .cover-inner so the grid's background color can overwrite the global background color set in customizer
                $cpl_cover_markup = 
                '<div class="cover-region cover-region-phone _100vh id-'.$id.'"">
                    <div class="cover-inner _100vh">
                        '.$cpl_cover_markup.'
                        '.$cpl_cover_freely_placed_els.'
                    </div>
                    '.Lay_LayoutFunctions::getCoverDownArrowMarkup(array('phoneOnly' => false)).'
                </div>';
            }

            $cpl_exists_html_class = 'hascustomphonegrid';
            $cpl_css = Lay_LayoutFunctions::getGridCSS($cpl_arr, '#custom-phone-grid.id-'.$id, true);
            $cpl_css .= Lay_LayoutFunctions::getHorizontalGridCSS($cpl_arr, '#custom-phone-grid.id-'.$id, true);

            $cpl_frame_css = Lay_LayoutFunctions::getGridFrameCSS($cpl_arr, '#custom-phone-grid.id-'.$id, '#custom-phone-grid.id-'.$id, true);
            if( $cover_on_phone ) {
                $cpl_frame_css = Lay_LayoutFunctions::getGridFrameCSS($cpl_arr, '.cover-region-phone.id-'.$id.' .cover-inner', '#custom-phone-grid.id-'.$id, true);
            }
            $cpl_rows_css = Lay_LayoutFunctions::getRowsMarginBottomCSS($cpl_arr, $cpl_rows, '#custom-phone-grid.id-'.$id, true);
            $cpl_background_color_css = Lay_LayoutFunctions::getBackgroundCSS($cpl_arr, '#custom-phone-grid.id-'.$id.', .cover-region-phone.id-'.$id.' .cover-inner');
            
            $cpl_background_color_projectoverlay = '';
            if( $is_project_overlay == 'true' ) {
                // set project overlay cover bg color
                $bgColor = $cpl_arr['bgColor'];
                if( $bgColor == '' || $bgColor == null ) {
                    $bgColor = ProjectsOverlaysOptions::$bg_color;
                }
                if( $cover_on_phone ) {
                    $cpl_background_color_projectoverlay = '<style>
                    #lay-project-overlay .cover-region-phone.id-'.$id.',
                    #lay-project-overlay .cover-region-phone.id-'.$id.' .cover-inner,
                    #lay-project-overlay #custom-phone-grid.id-'.$id.'{background-color:'.$bgColor.';}</style>';
                } else {
                    $cpl_background_color_projectoverlay = '<style>@media (max-width: '.(LayFrontend_Options::$breakpoint).'px){ #lay-project-overlay{background-color:'.$bgColor.';} }</style>';
                }
            }

            $cpl_background_image_css = Lay_LayoutFunctions::getBackgroundImageCSS($cpl_arr, '#custom-phone-grid.id-'.$id.', .cover-region-phone.id-'.$id.' .cover-inner');     

            $cpl_empty_class = Lay_Layout::isEmptyLayout($cpl_rows) && $cpl_freely_placed_els == '' && $cpl_background_image_css == '' ? 'lay-empty' : 'lay-not-empty';

            $cpl_markup = 
                '<!-- Start CPL Layout -->'
                .$cpl_cover_rowgutter_css
                .$cover_cpl_css
                .$cpl_cover_markup;
                if( $cover_on_phone ) {
                    // need cover content wrap, so i can set global bg color and bg image to it
                    // the inner div: .grid can now be set to the grid-specific background-color, overwriting .cover-content's background
                    // i cannot just set the global background color or background image to an outer div like .lay-content, because then ->
                    // when cover would be active and i would scroll down, the transparent div that is on top of the cover would not hide it beneath
                    $cpl_markup .= '<div class="cover-content cover-content-phone">';
                }
                // need .grid-inner wrapper so I can have a fullscreen slider that is horizontal
                $cpl_markup .= 
                '<div id="custom-phone-grid" class="grid '.$cpl_empty_class.' id-'.$id.' '.($is_project_overlay == 'true' ? 'project-overlay-grid' : '').'">
                    <div class="grid-inner">
                        '.$cpl_frame_css.'
                        '.$cpl_rows_css.'
                        '.$cpl_css.'
                        '.$cpl_background_color_css.'
                        '.$cpl_background_color_projectoverlay.'
                        '.$cpl_background_image_css.'
                        '.$cpl_freely_placed_els.'
                        '.$cpl_row_col_el_markup.'
                    </div>
                </div>';
                if( $cover_on_phone ) {
                    $cpl_markup .= '</div>';
                }
                $cpl_markup .= 
                '<!-- End CPL Layout -->';
        }

        // Footer Custom Phone Layout
        $footer_cpl_markup = '';
        $footer_cpl_exists_html_class = 'footer-nocustomphonegrid';
        if( LayFrontend_Options::$phone_layout_active && is_array($cpl_footer_arr) ) {
            $footer_cpl_exists_html_class = 'footer-hascustomphonegrid';
            $footer_cpl_css = Lay_LayoutFunctions::getGridCSS($cpl_footer_arr, '#footer-custom-phone-grid', true);
            $footer_cpl_css .= Lay_LayoutFunctions::getHorizontalGridCSS($cpl_footer_arr, '#footer-custom-phone-grid', true);

            $footer_cpl_frame_css = Lay_LayoutFunctions::getGridFrameCSS($cpl_footer_arr, '#footer-custom-phone-grid', '#footer-custom-phone-grid', true);
            $footer_cpl_rows_css = Lay_LayoutFunctions::getRowsMarginBottomCSS($cpl_footer_arr, $cpl_footer_rows, '#footer-custom-phone-grid', true);
            $footer_cpl_background_color_css = Lay_LayoutFunctions::getBackgroundCSS($cpl_footer_arr, '#footer-custom-phone-grid');
            $footer_cpl_background_image_css = Lay_LayoutFunctions::getBackgroundImageCSS($cpl_footer_arr, '#footer-custom-phone-grid');     

            $footer_cpl_markup = 
            '<div id="footer-custom-phone-grid" class="grid">
                '.$footer_cpl_frame_css.'
                '.$footer_cpl_rows_css.'
                '.$footer_cpl_css.'
                '.$footer_cpl_background_color_css.'
                '.$footer_cpl_background_image_css.'
                '.$cpl_footer_markup.'
                '.$cpl_footer_freely_placed_els.'
            </div>';
        }

        // Desktop CSS
        $desktop_css = '';
        $desktop_frame_css = '';
        $desktop_rows_css = '';
        $background_color_css = '';
        $background_color_projectoverlay = '';
        $background_image_css = '';
        
        if( is_array($desktop_arr) ) {
            $desktop_css = Lay_LayoutFunctions::getGridCSS($desktop_arr, '#grid.id-'.$id, false);
            $desktop_css .= Lay_LayoutFunctions::getHorizontalGridCSS($desktop_arr, '#grid.id-'.$id, false);
            $desktop_frame_css = Lay_LayoutFunctions::getGridFrameCSS($desktop_arr, '#grid.id-'.$id, '#grid.id-'.$id, false);
            if( $cover_on_desktop ){
                $desktop_frame_css = Lay_LayoutFunctions::getGridFrameCSS($desktop_arr, '.cover-region-desktop .cover-inner', '#grid.id-'.$id, false);
            }
            $desktop_rows_css = Lay_LayoutFunctions::getRowsMarginBottomCSS($desktop_arr, $desktop_rows, '#grid.id-'.$id, false);
            // need to set background color for .cover-inner too because it is not inside #grid div
            $background_color_css = Lay_LayoutFunctions::getBackgroundCSS($desktop_arr, '#grid.id-'.$id.', .cover-region-desktop.id-'.$id.' .cover-inner');
            
            if( $is_project_overlay == 'true' ) {
                // set project overlay cover bg color
                $bgColor = $desktop_arr['bgColor'];
                if( $bgColor == '' || $bgColor == null ) {
                    $bgColor = ProjectsOverlaysOptions::$bg_color;
                }
                if( $cover_on_desktop ){
                    $background_color_projectoverlay = '<style>
                    #lay-project-overlay .cover-region-desktop.id-'.$id.',
                    #lay-project-overlay .cover-region-desktop.id-'.$id.' .cover-inner,
                    #lay-project-overlay #grid.id-'.$id.'{background-color:'.$bgColor.';}
                    </style>';
                } else {
                    if( LayFrontend_Options::$phone_layout_active && is_array($cpl_arr) ) {
                        // cpl exists, so use media query:
                        $background_color_projectoverlay = '<style>@media (min-width: '.(LayFrontend_Options::$breakpoint + 1).'px){ #lay-project-overlay{background-color:'.$bgColor.';} }</style>';
                    } else {
                        $background_color_projectoverlay = '<style>#lay-project-overlay{background-color:'.$bgColor.';}</style>';
                    }
                }
            }

            $background_image_css = Lay_LayoutFunctions::getBackgroundImageCSS($desktop_arr, '#grid.id-'.$id.', .cover-region-desktop.id-'.$id.' .cover-inner');     
        }
        
        // Is there a footer?

        // Footer Desktop CSS
        $desktop_footer_css = '';
        $desktop_frame_footer_css = '';
        $desktop_rows_footer_css = '';
        $background_color_footer_css = '';

        if( is_array($footer_desktop_arr) ){
            $desktop_footer_css = Lay_LayoutFunctions::getGridCSS($footer_desktop_arr, '#footer', false);
            $desktop_footer_css .= Lay_LayoutFunctions::getHorizontalGridCSS($footer_desktop_arr, '#footer', false);
            $desktop_frame_footer_css = Lay_LayoutFunctions::getGridFrameCSS($footer_desktop_arr, '#footer', '#footer', false);
            $desktop_rows_footer_css = Lay_LayoutFunctions::getRowsMarginBottomCSS($footer_desktop_arr, $desktop_footer_rows, '#footer', false);
            $background_color_footer_css = Lay_LayoutFunctions::getBackgroundCSS($footer_desktop_arr, '#footer');    
        }

        // Cover
        // todo: cover disabled for phone
        $cover_desktop_class = 'nocover';
        // todo: && !isFullscreenSliderActive()
        // if( cover_controller.hasCover(type, obj.id) && !isFullscreenSliderActive() ){
        $desktop_cover_rowgutter_css = '';
        $cover_desktop_css = '';
        if( $cover_on_desktop ){
            $cover_desktop_class = 'hascover';
            $cover_desktop_css = Lay_LayoutFunctions::getGridCSS($desktop_arr, '.cover-region', false);
            $cover_desktop_css .= Lay_LayoutFunctions::getHorizontalGridCSS($desktop_arr, '.cover-region', false);
            // set main grid padding-top to first rowgutter
            $desktop_cover_rowgutter_css = Lay_LayoutFunctions::getFirstRowGutterAsPaddingTopForCoverCSS($desktop_arr, '#grid.id-'.$id);
            
            // need .cover-inner so the grid's background color can overwrite the global background color set in customizer

            // ok so when we have a cover on desktop, but cover is disabled for phone, 
            // i dont want the cover row to be 100vh on APL if the user hasn't set it to be 100vh explicitly for phone
            // APL = automatically generated phone layout
            $_100vh_set_by_user_class = $desktop_cover_is_100vh_set_by_user ? '_100vh-set-by-user' : '_100vh-not-set-by-user';
            $desktop_cover_markup = 
            '<div class="cover-region cover-region-desktop _100vh '.$_100vh_set_by_user_class.' id-'.$id.'"">
                <div class="cover-inner _100vh">
                    '.$desktop_cover_markup.'
                    '.$desktop_cover_freely_placed_els.'
                </div>
                '.Lay_LayoutFunctions::getCoverDownArrowMarkup(array('phoneOnly' => false)).'
            </div>';
        }

        // this is not for cpl but just so i can disable cover via css on phone version of the page
        $cover_disabled_on_phone_class = LayFrontend_Options::$cover_disable_for_phone == true ? 'cover-disabled-on-phone' : 'cover-enabled-on-phone';

        if( $getFullLayout == false ) {
            $cover_disabled_on_phone_class = 'cover-disabled-on-phone';
        }

        // $hover_image_markup = '';
        // if( is_plugin_active( 'laytheme-imagehover/laytheme-imagehover.php' ) && $getImageHover ) {
        //     switch(LayFrontend_Options::$imagehover_imageposition){
        //         case 'centerfixed':
        //         case 'leftbackground':
        //         case 'rightbackground':
        //         case 'fullsize':
        //             $all_rows = array($desktop_rows, $cpl_rows, $desktop_footer_rows, $cpl_footer_rows, $desktop_cover_row, $cpl_cover_rows);
        //             $array_to_merge_to = array();
        //             LayRowFunctions::mergeAllRows($all_rows, $array_to_merge_to);
        //             $hover_image_markup = '<div class="lay-imagehover-region">'.LayThemeImagehover::getMarkup( $array_to_merge_to, $desktop_markup.$desktop_freely_placed_els.$desktop_footer_markup.$footer_desktop_freely_placed_els ).'</div>';
        //     }
        // }

        // todo: add lay-empty class to cover row too, and cpl cover row and to footer!
        $desktop_empty_class = Lay_Layout::isEmptyLayout($desktop_rows) && $desktop_freely_placed_els == '' && $background_image_css == '' ? 'lay-empty' : 'lay-not-empty';

        // create a new filter where we can append things to $markup. add $obj
        // to the filter so we can use it in the filter function

        $markupFilter = apply_filters( 'laytheme_filter_content', $obj );

        $markup =
        '<div class="lay-content '.$cpl_exists_html_class.' '.$footer_cpl_exists_html_class.' '.$cover_desktop_class.' '.$cover_cpl_class.' '.$cover_disabled_on_phone_class.'">
            <!-- Start Desktop Layout -->
            '.$desktop_cover_rowgutter_css.'
            '.$cover_desktop_css.'
            '.$desktop_cover_markup;
            if( $cover_on_desktop == true ) {
                // need cover content wrap, so i can set global bg color and bg image to it
                // the inner div: .grid can now be set to the grid-specific background-color, overwriting .cover-content's background
                // i cannot just set the global background color or background image to an outer div like .lay-content, because then ->
                // when cover would be active and i would scroll down, the transparent div that is on top of the cover would not hide it beneath
                $markup .= '<div class="cover-content cover-content-desktop">';
            }
            $markup = apply_filters( 'lay_before_desktop_grid', $markup, $obj );
            // need .grid-inner wrapper so I can have a fullscreen slider that is horizontal
            $markup .= 
            '<div id="grid" class="grid '.$desktop_empty_class.' id-'.$id.' '.($is_project_overlay == 'true' ? 'project-overlay-grid' : '').'">';
            $markup = apply_filters( 'lay_after_desktop_grid_opening_tag', $markup, $obj );
            $markup .=
                '<div class="grid-inner">
                '.$desktop_frame_css.'
                '.$desktop_rows_css.'
                '.$desktop_css.'
                '.$background_color_css.'
                '.$background_color_projectoverlay.'
                '.$background_image_css.'
                '.$desktop_freely_placed_els.'
                '.$desktop_markup.'
                </div>';
            $markup = apply_filters( 'lay_after_desktop_grid_closing_tag', $markup, $obj );
            $markup .=
            '</div>';
            if( $cover_on_desktop == true ) {
                // closing .cover-content
                $markup .= '</div>';
            }
            $markup .= 
            '<!-- End Desktop Layout -->';
            $markup = apply_filters( 'lay_after_desktop_grid', $markup, $obj );
            $markup .=
            $cpl_markup;
            if( $getFullLayout == true ) {
                $markup .= 
                '<div id="footer-region">';
                if( is_array($footer_desktop_arr) ){
                    // need "grid" class here to make expandrow work! 
                    /*
                    var $row = jQuery(this)
                    .closest('.grid')
				    .find('.row-id-' + expandArr[i])
                    */
                    $markup .=
                    '<div id="footer" class="footer grid">
                        '.$desktop_frame_footer_css.'
                        '.$desktop_rows_footer_css.'
                        '.$desktop_footer_css.'
                        '.$background_color_footer_css.'
                        '.$footer_desktop_freely_placed_els.'
                        '.$desktop_footer_markup.'
                    </div>';
                }
                $markup .=
                    $footer_cpl_markup.
                '</div>';
            }
            if(is_string($markupFilter)) {
                $markup .=  $markupFilter;
            }
        $markup .='</div>';

        // apply content hook for plugins:
        if( get_option( 'misc_options_enable_content_filter', "" ) == "on" ) {
            $markup = apply_filters( 'the_content', $markup );
        }

        return array(
            'markup' => $markup,
            'type' => $type,
            'layoutObj' => $layoutObj,
            'obj' => $obj,
            'notification' => 'success'
        );
    }

    // a layout is empty if there are no elements inside of it
    // todo: layout should not be empty if a row is empty but has a background image or background video
    public static function isEmptyLayout($layout_array){
        if( is_array($layout_array) && count($layout_array) > 0 ){
            $isempty = true;
            for ($i=0; $i < count($layout_array); $i++) { 
                if( array_key_exists('columns', $layout_array[$i]) ) {
                    if( !empty($layout_array[$i]['columns']) ) {
                        $isempty = false;
                        break;
                    }
                }
                if( array_key_exists('webglslideshow', $layout_array[$i]) ) {
                    if( !empty($layout_array[$i]['webglslideshow']) ) {
                        $isempty = false;
                        break;
                    }
                }
                if( array_key_exists('bgimage', $layout_array[$i]) ) {
                    if( !empty($layout_array[$i]['bgimage']) ) {
                        $isempty = false;
                        break;
                    }
                }
                if( array_key_exists('bgvideo', $layout_array[$i]) ) {
                    if( !empty($layout_array[$i]['bgvideo']) ) {
                        $isempty = false;
                        break;
                    }
                }
            }

            return $isempty;
        }
        return true;
    }

    public static function getFreelyPlacedCols($grid_arr, $postid, $device, $specifier = 'all') {
        // $specifier can be "all" or "in-cover" or "outside-cover"
        // this is for splitting up markup into cover and markup below
        $freelyPlacedEls = array();
        $cols = $grid_arr['cont'];
        $markup = '';
        for( $i=0; $i<count($cols); $i++ ){
            $col = $cols[$i];
            if ( array_key_exists('placeFreely', $col) ){
                if( $specifier == 'in-cover' ) {
                    if($col['row'] > 0) {
                        continue;
                    }
                } else if( $specifier == 'outside-cover' ) {
                    if($col['row'] == 0) {
                        continue;
                    }
                }
                $layCol = new LayCol($col, false);
                $layEl = new LayEl($col, false, $postid, $device);
                $elMarkup = $layEl->getMarkup();
                if( $elMarkup != '' ) {
                    $markup .= $layCol->getOpeningMarkup();
                    $markup .= $elMarkup;
                    $markup .= $layCol->getClosingMarkup();;
                }
            }
        }
        return $markup;
    }

    public static function prepareRows($grid_arr, $cover_active) {
        if( $grid_arr == null ) {
            return false;
        }

        $cols = $grid_arr['cont'];
        $rowAttrs = array_key_exists( 'rowAttrs', $grid_arr ) ? $grid_arr['rowAttrs'] : false;
        $rowAmt = count($grid_arr['rowGutters']) + 1;
        $rows = array();
        $cover = null;

        // create row objects in an array, possibly with row attributes
        for( $i = 0; $i < $rowAmt; $i++ ){
            $rows []= array();
            if( is_array($rowAttrs) ) {
                if( array_key_exists($i, $rowAttrs) ) {
                    $rows[$i] = $rowAttrs[$i];
                }
            }
            // add row gutter
            if( array_key_exists($i, $grid_arr['rowGutters']) ){
                $rows[$i]['rowGutter'] = $grid_arr['rowGutters'][$i];
            }
            $rows[$i]['columns'] = array();
        }

        if( $cover_active == true ) {
            $rows[0]['row100vh'] = 1;
            $rows[0]['rowcustomheight'] = '';
        }

        // loop through all columns and put them into their according row's .columns
        if( $cols ) {

            $reorder_cols = false;

            for( $i=0; $i<count($cols); $i++ ){
                if ( is_array($cols[$i]) && !array_key_exists('placeFreely', $cols[$i]) ){
                    $rowIx = $cols[$i]['row'];
                    $type = $cols[$i]['type'];
                    if( is_array($rows[$rowIx]) && array_key_exists('row100vh', $rows[$rowIx]) && $rows[$rowIx]['row100vh'] == 1 ||
                    is_array($rows[$rowIx]) && array_key_exists('rowcustomheight', $rows[$rowIx]) && $rows[$rowIx]['rowcustomheight'] != '' || 
                    $type == 'vl' ) {
                        // if( $type != 'thumbnailgrid' && $type != 'elementgrid' ) {
                            if( array_key_exists('classes', $cols[$i]) ) {
                                $cols[$i]['classes'] .= ' absolute-position';
                                $reorder_cols = true;
                            } else {
                                $cols[$i]['classes'] = 'absolute-position';
                                $reorder_cols = true;
                            }
                        // }
                    }
                    if( !is_array($rows[$rowIx]) ) {
                        $rows[$rowIx] = array();
                    }
                    $rows[$rowIx]['columns'] []= $cols[$i];
                }
            }

            // reorder cols
            // the columns that are absolute positioned should always come after cols that are not
            // this way, the "first-child" class will be used correctly so the frame-left value is correct
            // this is for the rare case when an absolutely positioned element has col-0 for example and an image too

            if( $reorder_cols == true ) {
                for( $i=0; $i<count($rows); $i++ ) {
                    $cols = $rows[$i]['columns'];

                    $cols_new = array();
                    $cols_start = array();
                    $cols_end = array();
                    for( $x=0; $x<count($cols); $x++ ) {
                        if( array_key_exists('classes', $cols[$x]) && strpos($cols[$x]['classes'], 'absolute-position') !== false ){
                            $cols_end []= $cols[$x];
                        } else {
                            $cols_start []= $cols[$x];
                        }
                    }
                    $cols_new = array_merge( $cols_start, $cols_end );

                    $rows[$i]['columns'] = $cols_new;
                }
            }
            // #reorder cols

            // set classes
            for( $i=0; $i<count($rows); $i++ ) {
                $cols = $rows[$i]['columns'];
                for( $x=0; $x<count($cols); $x++ ) {
                    if( $x == 0 ) {
                        if( array_key_exists('classes', $cols[$x]) ) {
                            $rows[$i]['columns'][$x]['classes'] .= ' first-child';
                        } else {
                            $rows[$i]['columns'][$x]['classes'] = 'first-child';
                        }
                    } else {
                        if( array_key_exists('classes', $cols[$x]) ) {
                            $rows[$i]['columns'][$x]['classes'] .= ' not-first-child';
                        } else {
                            $rows[$i]['columns'][$x]['classes'] = 'not-first-child';
                        }
                    }

                }
            }

            // add horizontal grid info
            if( array_key_exists('horizontalGrid', $grid_arr) && $grid_arr['horizontalGrid']['show'] == true ) {
                for( $i=0; $i<count($rows); $i++ ) {
                    $cols = $rows[$i]['columns'];
                    for( $x=0; $x<count($cols); $x++ ) {
                        $rows[$i]['columns'][$x]['horizontalGrid'] = $grid_arr['horizontalGrid'];
                    }
                }
            }


        }

        return $rows;
    }

    public static function getRowColElMarkup($rows, $postid, $device, $getImageHover, $getOpeningAndClosingRowMarkup = true) {
        $markup = '';
        for( $i = 0; $i < count($rows); $i++ ){
            $row = $rows[$i];
            if( $i == 0 ){
                if( array_key_exists('classes', $row) ){
                    $row['classes'] .= ' first-row';
                }else{
                    $row['classes'] = 'first-row';
                }
                
            }
            if($getOpeningAndClosingRowMarkup){
                $layRow = new LayRow($row, $i);
                $layRowOpeningMarkup = $layRow->getOpeningMarkup();
                $markup .= $layRowOpeningMarkup;
            }

            // add imagehover markup for row background imagehover images
            if( is_plugin_active( 'laytheme-imagehover/laytheme-imagehover.php' ) && class_exists('LayThemeImagehover') && $getImageHover ) {
                if($getOpeningAndClosingRowMarkup){
                    $imagesArrayForRows = LayThemeImagehover::getImageFromMarkup2( $layRowOpeningMarkup );
                    
                    if(is_array($imagesArrayForRows)){
                        foreach ($imagesArrayForRows as $image) {
                            $markup .= '<div class="lay-imagehover-on-element" data-transition="'.LayFrontend_Options::$imagehover_transition.'" data-imageposition='.$image['position'].' data-hoverimageposition='.$image['position'].' data-hoverimageshowon="'.$image['showon'].'" data-hoverimagepositionforscroll='.$image['positionforscroll'].' data-hoverimagepreactivated="'.$image['preactivated'].'">'.$image['media'].'</div>';
                        }
                    }
                }
            }

            // when row is 100vh, the first column inside that is not a text and not a html element needs class lay-col-needs-leftframe-margin
            $isRowWhereElsArePositionAbsolute = false;
            if( array_key_exists('row100vh', $row) && $row['row100vh'] == true || array_key_exists('rowcustomheight', $row) && $row['rowcustomheight'] == true ){
                $isRowWhereElsArePositionAbsolute = true;
            }
            if( $isRowWhereElsArePositionAbsolute == true ) {
                for( $i3=0; $i3<count($row['columns']); $i3++ ) {
                    $col = $row['columns'][$i3];
                    if( $col['type'] != 'text' && $col['type'] != 'html' ) {
                        if( array_key_exists('classes', $col) ){
                            $row['columns'][$i3]['classes'] .= ' lay-col-needs-leftframe-margin';
                        } else {
                            $row['columns'][$i3]['classes'] = 'lay-col-needs-leftframe-margin';
                        }
                    break;
                    }
                }
            }

            // 
            for( $i2=0; $i2<count($row['columns']); $i2++ ){
                $col = $row['columns'][$i2];
                $layCol = new LayCol($col, $isRowWhereElsArePositionAbsolute);
                $layEl = new LayEl($col, false, $postid, $device);
                $elMarkup = $layEl->getMarkup();
                if( $elMarkup != '' ) {
                    $markup .= $layCol->getOpeningMarkup();
                    $markup .= $elMarkup;
                    $markup .= $layCol->getClosingMarkup();
                    // add imagehover markup
                    if( is_plugin_active( 'laytheme-imagehover/laytheme-imagehover.php' ) && $getImageHover && method_exists('LayThemeImagehover', 'getImageFromMarkup2') ) {
                        // todo: does it work for stack element?
                        $imagesArray2 = LayThemeImagehover::getImageFromMarkup2( $elMarkup );
                        if(is_array($imagesArray2)){
                            foreach ($imagesArray2 as $image) {

                                $isScroll = false;
                                if($image['showon'] == 'onscrollonly' || $image['showon'] == 'onscrollandhover' || $image['showon'] == 'onscrollfortouchdevices_onhoverfordesktop' && class_exists('LTUtility') && LTUtility::$isTouchDevice) {
                                    $isScroll = true;
                                }

                                // clicking on imagehover image so its a link should only work if it is an image that is shown on scroll
                                // otherwise it would just flicker around because the linked hoverimage is not pointer events "none"

                                if( property_exists('LayThemeImagehover', 'clickable') && LayThemeImagehover::$clickable == "on" && $isScroll && array_key_exists('href', $image) && $image['href'] != '') {
                                    // 'href' => $href, 'dataId' => $dataId, 'dataType' => $dataType, 'dataTitle' => $dataTitle, 'dataCatid' => $dataCatid, 
                                    $linkAttrs = '';
                                    if( array_key_exists('href', $image) ) {
                                        $linkAttrs .= ' href="'.$image['href'].'"';
                                    }
                                    if( array_key_exists('dataId', $image) ) {
                                        $linkAttrs .= ' data-id="'.$image['dataId'].'"';
                                    }
                                    if( array_key_exists('dataType', $image) ) {
                                        $linkAttrs .= ' data-type="'.$image['dataType'].'"';
                                    }
                                    if( array_key_exists('dataTitle', $image) ) {
                                        $linkAttrs .= ' data-title="'.$image['dataTitle'].'"';
                                    }
                                    if( array_key_exists('dataCatid', $image) ) {
                                        $linkAttrs .= ' data-catid="'.$image['dataCatid'].'"';
                                    }

                                    $markup .= '<a class="lay-imagehover-on-element" '.$linkAttrs.' data-transition="'.LayFrontend_Options::$imagehover_transition.'" data-imageposition='.$image['position'].' data-hoverimageposition='.$image['position'].' data-hoverimageshowon="'.$image['showon'].'" data-hoverimagepositionforscroll='.$image['positionforscroll'].' data-hoverimagepreactivated="'.$image['preactivated'].'">'.$image['media'].'</a>';
                                }else{
                                    $markup .= '<div class="lay-imagehover-on-element" data-transition="'.LayFrontend_Options::$imagehover_transition.'" data-imageposition='.$image['position'].' data-hoverimageposition='.$image['position'].' data-hoverimageshowon="'.$image['showon'].'" data-hoverimagepositionforscroll='.$image['positionforscroll'].' data-hoverimagepreactivated="'.$image['preactivated'].'">'.$image['media'].'</div>';
                                }
                            }
                        }

                    }
                }
            }
            if($getOpeningAndClosingRowMarkup){
                $markup .= $layRow->getClosingMarkup();
            }
        }
        return $markup;
    }

}

new Lay_Layout();