<?php

// news:

/*

- list of news, sorted by date
- "next news", "previous news" buttons
- some kind of thumbnailgrid of news

*/

class LayNews{

    public function __construct(){
        $val = get_option('misc_options_activate_news_feature', '');
		if( $val == "on" && !MiscOptions::$locked ){
			add_action('init', array($this, 'lay_register_news_post_type'));
		}
        add_action('wp_ajax_get_news_feed_via_ajax', array($this, 'get_news_feed_via_ajax'));
        add_action('wp_ajax_nopriv_get_news_feed_via_ajax', array($this, 'get_news_feed_via_ajax'));

        // make excerpt wysiwyg
        add_action( 'add_meta_boxes', array($this, 'lay_ape_replace_metabox') );

        // external link
        add_action( 'add_meta_boxes', array($this, 'lay_add_external_link_metabox') );
        add_action( 'save_post_lay_news', array($this, 'lay_save_external_link_and_disable_link'), 10, 2 );

        // disable link metabox
        add_action( 'add_meta_boxes', array($this, 'lay_add_disable_link_metabox') );
    }

    public static function lay_save_external_link_and_disable_link($post_id, $post){
        $post_id = (int)$post_id;
		/*
		 * We need to verify this came from the our screen and with proper authorization,
		 * because save_post can be triggered at other times.
		 */

		// Check if our nonce is set.
		if ( ! isset( $_POST['external_link_is_active_nonce'] ) )
			return $post_id;

		$nonce = $_POST['external_link_is_active_nonce'];

		// Verify that the nonce is valid.
		if ( ! wp_verify_nonce( $nonce, 'external_link_is_active' ) )
			return $post_id;

		// If this is an autosave, our form has not been submitted,
                //     so we don't want to do anything.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) 
			return $post_id;

		// Check the user's permissions.
		if ( 'lay_news' == $_POST['post_type'] ) {
			if ( ! current_user_can( 'edit_post', $post_id ) )
				return $post_id;
		}

		/* OK, its safe for us to save the data now. */
        $external_link_is_active = isset( $_POST['external_link_is_active'] ) ? $_POST['external_link_is_active'] : '';
        $external_link = isset( $_POST['external_link'] ) ? $_POST['external_link'] : '';
        $disable_link_is_active = isset( $_POST['disable_link_is_active'] ) ? $_POST['disable_link_is_active'] : '';
        
		// Update the setting
		update_post_meta($post_id, 'external_link_is_active', $external_link_is_active);
        update_post_meta($post_id, 'external_link', $external_link);
        update_post_meta($post_id, 'disable_link_is_active', $disable_link_is_active);
        
    }

    public static function lay_add_disable_link_metabox(){
        add_meta_box(
            'lay_disable_link',
            'Disable Link',
            'LayNews::lay_disable_link_render_metabox',
            array('lay_news'),
            'side',
            'default'
        );
    }

    public static function lay_add_external_link_metabox(){
        add_meta_box(
            'lay_external_link',
            'External Link',
            'LayNews::lay_external_link_render_metabox',
            array('lay_news'),
            'side',
            'default'
        );
    }

    /**
     * Replace the default 'postexcerpt' meta box.
     */
    public static function lay_ape_replace_metabox() {
        $post_types = get_post_types_by_support( 'excerpt' );

        // Remove the default postexcerpt meta box.
        remove_meta_box( 'postexcerpt', $post_types, 'normal' );

        // Register our new meta box.
        add_meta_box(
            'postexcerpt',
            _x( 'Excerpt', 'meta box heading', 'advanced-post-excerpt' ),
            'LayNews::lay_ape_render_metabox',
            $post_types,
            'normal',
            'high',
            array(
                '__back_compat_meta_box' => false,
            )
        );
    }

    /**
     * Replace the meta box contents.
     *
     * @param WP_Post $post The current post object.
     */
    public static function lay_ape_render_metabox( $post ) {
        wp_editor( html_entity_decode( $post->post_excerpt ), "excerpt", Gridder::$tinymceSettings );
    }

    public static function lay_disable_link_render_metabox($post) {
        // Add an nonce field so we can check for it later.
        wp_nonce_field( 'disable_link_is_active', 'disable_link_is_active_nonce' );

        $isActive = get_post_meta( $post->ID, 'disable_link_is_active', false );
        $checked = is_array($isActive) && count($isActive) > 0 && $isActive[0] == 'on' ? "checked" : "";
        // Display the form, using the current value.
        echo '<div>
            <input type="checkbox" id="disable_link_is_active" name="disable_link_is_active" value="on" '.$checked.'/>
            <label for="disable_link_is_active">Disable Link for Thumbnail</label>
            <div style="margin-top:10px;">This is for thumbnails in: +More &rarr; +News Element</div>
        </div>';
    }

    public static function lay_external_link_render_metabox($post) {
        // Add an nonce field so we can check for it later.
        wp_nonce_field( 'external_link_is_active', 'external_link_is_active_nonce' );

        $isActive = get_post_meta( $post->ID, 'external_link_is_active', false );
        $checked = is_array($isActive) && count($isActive) > 0 && $isActive[0] == 'on' ? "checked" : "";

        $externalLink = get_post_meta( $post->ID, 'external_link', '' );
        $val = is_array($externalLink) && count($externalLink) > 0 ? $externalLink[0] : "";
        // Display the form, using the current value.
        echo '<div>
            <input type="checkbox" id="external_link_is_active" name="external_link_is_active" value="on" '.$checked.'/>
            <label for="external_link_is_active">Activate Link for Thumbnail</label>
        </div>';

        echo '<div style="margin-top:10px;">
            <label for="external_link">External Link:</label><br/>
            <input placeholder="https://laytheme.com" type="url" id="external_link" name="external_link" value="'.$val.'"/><br/>
            <i>(needs to start with https://)</i>
            <div style="margin-top:10px;">This is for thumbnails in: +More &rarr; +News Element</div>
        </div>';
    }

    public static function get_news_feed_via_ajax(){
        $config = $_POST['config'];
        $target = $_POST['target'];
        $markup = LayNews::get_news_feed($config, $target);
        echo $markup;
		die();
    }

    // Comparison function 
    public static function date_compare($element1, $element2) {
        return strtotime($element2->post_date) - strtotime($element1->post_date);
    }

    public static function get_news_feed($config, $target){
        $layout = 'list';
        if( array_key_exists('layout', $config) ) {
            $layout = $config['layout'];
        }
        $markup = '';

        $amount = -1;

        $date_format = 'd.m.Y';
        if( array_key_exists('date_format', $config) ) {
            $date_format = $config['date_format'];
        }
        $args = array(
            'posts_per_page' => $amount,
            'orderby' => 'post_date',
            'order' => 'DESC',
            'post_type' => 'lay_news',
            // 'fields' => 'ids',
            'post_status' => 'publish',
        );
        // show only of that category
        if(array_key_exists('cat_id', $config) && $config['cat_id'] != -1){
            $args ['tax_query'] = array(             
                array(
                   'taxonomy' => 'lay_news_category',
                   'field' => 'term_id',
                   'terms' => $config['cat_id']
               ),
            );
        }
        remove_all_filters('posts_orderby');
        
        $query = new WP_Query( $args );

        // as of recently, wordpress sometimes doesn't sort this correctly, so i just sort it myself:
        if ( $query->have_posts() ) {
            // Sort the array  
            usort($query->posts, 'LayNews::date_compare');
        }

        switch( $layout ) {
            case 'list':
                $markup = LayNews::get_news_feed_list_layout($config, $target, $query, $date_format);
            break;
            case 'columns':
                $markup = LayNews::get_news_feed_columns_layout($config, $target, $query, $date_format);
            break;
            case 'full':
                $markup = LayNews::get_news_feed_fullcontent_layout($config, $target, $query, $date_format);
            break;
        }

        return $markup;
	}

    public static function get_news_feed_fullcontent_layout($config, $target, $query, $date_format){
        $array = array();

        $ix = 0;
        $amount = -1;
        if( array_key_exists('amount', $config) ) {
            if($config['amount'] > 0) {
                $amount = (int)$config['amount'];
            }
        }
        $loadmore = '';
        if(array_key_exists('showMoreButton', $config) && $config['showMoreButton'] == 'true'){
            $buttonAlignment = array_key_exists('moreButtonAlignment', $config) ? $config['moreButtonAlignment'] : 'center';
            $spacetop = array_key_exists('moreButtonSpaceTop', $config) ? $config['moreButtonSpaceTop'] : 10;
            $style = array_key_exists('showMoreButtonStyle', $config) ? $config['showMoreButtonStyle'] : 'laybutton1';
            $text = array_key_exists('moreButtonText', $config) ? $config['moreButtonText'] : 'Load More';
            
            $loadmore = '<div class="lay-news-load-more-wrap" style="justify-content:'.$buttonAlignment.';margin-top:'.$spacetop.'px;">
            <div class="lay-news-load-more '.$style.'">'.$text.'</div>
            </div>';
        }

        $array = array();
        if ( $query->have_posts() ) {
            foreach ($query->posts as $post){
                if($amount == -1 || $ix < $amount || $loadmore != ''){
                    $ix++;
                    $post_id = $post->ID;
                    $display = 'show-news';
                    if($loadmore != '' && $ix > $amount){
                        $display = 'hide-news';
                    }
                    $array []= array('id' => $post_id, 'title' => get_the_title($post_id), 'date' => get_the_date($date_format, $post_id), 'display' => $display );
                }
            }
        }

        $css_prefix = $target == 'modal' ? '.lay-input-modal ' : '';

        $lineMarkup = '';

        if( $config['line'] === 'true' || $config['line'] === true  ) {
            $lineMarkup = 
            $css_prefix.'.lay-news-element-'.$config['uniqueId'].'>div{
                border-top: 1px solid '.$config['lineColor'].';
                margin-bottom: '.$config['space_below_line'].'px;
                padding-top: '.$config['space_above_line'].'px;
            }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].'>div:first-child{
                border-top: none;
            }';
        } else {
            $lineMarkup = 
            $css_prefix.'.lay-news-element-'.$config['uniqueId'].'>div{
                margin-bottom: '.$config['space_below_line'].'px;
            }';
        }
        $markup =
        '<style>
            '.$lineMarkup.'
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-separator{ padding-left: '.$config['separator_space'].'px!important; padding-right: '.$config['separator_space'].'px!important; }
        </style>';

        // show title, show separator, show date
        $show_title = true;
        if( array_key_exists('show_title', $config) ) {
            if($config['show_title'] === 'false' || $config['show_title'] === false) {
                $show_title = false;
            }
        }

        $separator = '<div class="lay-news-element-separator _'.$config['separator_textformat'].'_no_spaces">'.$config['separator_text'].'</div>';
        $show_separator = true;
        if( $config['show_separator'] === 'false' || $config['show_separator'] === false ) {
            $show_separator = false;
            $separator = '';
        }

        $show_date = true;
        if( $config['show_date'] === 'false' || $config['show_date'] === false ) {
            $show_date = false;
        }

        $show_date_first = false;
        if( array_key_exists('show_date_first', $config) && ( $config['show_date_first'] === 'true' || $config['show_date_first'] === true ) ) {
            $show_date_first = true;
        }
        // # show title, show separator, show date

        $getImagehoverMarkup = $target == 'frontend' ? true : false;
        // hello
        $markup .= 
        '<div class="lay-news-element-wrap">
            <div class="lay-news-element lay-news-element-layout-fullcontent lay-news-element-'.$config['uniqueId'].'" data-amount="'.$amount.'">';
        foreach ($array as $news){
            $id = $news['id'];
            $layout = Lay_Layout::getLayout($id, 'lay_news', '', false, $getImagehoverMarkup);
            $date = $show_date ? '<div class="lay-news-element-date _'.$config['date_textformat'].'_no_spaces">'.$news['date'].'</div>' : '';

            $_title = $show_title ? __($news['title']) : '';

            $t = '';
            if( $show_date_first ) {
                $t = $date.$separator.'<div class="lay-news-element-title _'.$config['title_textformat'].'_no_spaces">'.$_title.'</div>';
            } else {
                $t = '<div class="lay-news-element-title _'.$config['title_textformat'].'_no_spaces">'.$_title.'</div>'.$separator.$date;
            }

            $title =
            '<div class="lay-news-element-title-wrap">
                '.$t.'
            </div>';
            if( !$show_title && !$show_separator && !$show_date ) {
                $title = '';
            }
            if(array_key_exists('markup', $layout)) {
                $markup .= '<div class="'.$news['display'].'">'.$title.$layout['markup'].'</div>';
            }
        }

        return $markup.'</div>'.$loadmore.'
        </div>';
    }

    public static function get_news_feed_columns_layout($config, $target, $query, $date_format){
        $imageSpaceBottom = array_key_exists('imageSpaceBottom', $config) ? $config['imageSpaceBottom'] : 10;

        $alignment = array_key_exists('columnsLayoutType', $config) ? $config['columnsLayoutType'] : 'top-aligned';

        $colCountDesktop = array_key_exists('colCount', $config['desktop']) ? $config['desktop']['colCount'] : 4;
        $colCountTablet = array_key_exists('colCount', $config['tablet']) ? $config['tablet']['colCount'] : 3;
        $colCountPhone = array_key_exists('colCount', $config['phone']) ? $config['phone']['colCount'] : 2;

        $rowGutterDesktop = array_key_exists('rowGutter', $config['desktop']) ? $config['desktop']['rowGutter'] : 20;
        $rowGutterTablet = array_key_exists('rowGutter', $config['tablet']) ? $config['tablet']['rowGutter'] : 20;
        $rowGutterPhone = array_key_exists('rowGutter', $config['phone']) ? $config['phone']['rowGutter'] : 20;

        $rowGutterMuDesktop = array_key_exists('rowGutterMu', $config['desktop']) ? $config['desktop']['rowGutterMu'] : 'px';
        $rowGutterMuTablet = array_key_exists('rowGutterMu', $config['tablet']) ? $config['tablet']['rowGutterMu'] : 'px';
        $rowGutterMuPhone = array_key_exists('rowGutterMu', $config['phone']) ? $config['phone']['rowGutterMu'] : 'px';

        $colGutterDesktop = array_key_exists('colGutter', $config['desktop']) ? $config['desktop']['colGutter'] : 20;
        $colGutterTablet = array_key_exists('colGutter', $config['tablet']) ? $config['tablet']['colGutter'] : 20;
        $colGutterPhone = array_key_exists('colGutter', $config['phone']) ? $config['phone']['colGutter'] : 20;

        $colGutterMuDesktop = array_key_exists('colGutterMu', $config['desktop']) ? $config['desktop']['colGutterMu'] : 'px';
        $colGutterMuTablet = array_key_exists('colGutterMu', $config['tablet']) ? $config['tablet']['colGutterMu'] : 'px';
        $colGutterMuPhone = array_key_exists('colGutterMu', $config['phone']) ? $config['phone']['colGutterMu'] : 'px';

        $read_more_underline = array_key_exists('read_more_underline', $config) ? $config['read_more_underline'] : true;
        if( $read_more_underline === 'false' || $read_more_underline === '' ) {
            $read_more_underline = false;
        }
        else if( $read_more_underline === 'true' || $read_more_underline === '1' ) {
            $read_more_underline = true;
        }

        $loadmore = '';
        if(array_key_exists('showMoreButton', $config) && $config['showMoreButton'] == 'true') {
            $alignmentButton = array_key_exists('moreButtonAlignment', $config) ? $config['moreButtonAlignment'] : 'center';
            $spacetop = array_key_exists('moreButtonSpaceTop', $config) ? $config['moreButtonSpaceTop'] : 10;
            $style = array_key_exists('showMoreButtonStyle', $config) ? $config['showMoreButtonStyle'] : 'laybutton1';
            $text = array_key_exists('moreButtonText', $config) ? $config['moreButtonText'] : 'Load More';
            
            $loadmore = '<div class="lay-news-load-more-wrap" style="justify-content:'.$alignmentButton.';margin-top:'.$spacetop.'px;">
            <div class="lay-news-load-more '.$style.'">'.$text.'</div>
            </div>';
        }

        $ix = 0;
        $amount = -1;
        if( array_key_exists('amount', $config) ) {
            if($config['amount'] > 0) {
                $amount = (int)$config['amount'];
            }
        }

        $array = array();
        if ( $query->have_posts() ) {
            foreach ($query->posts as $post){
                if($amount == -1 || $ix < $amount || $loadmore != ''){
                    $ix++;
                    $post_id = $post->ID;
                    $display = 'show-news';
                    if($loadmore != '' && $ix > $amount){
                        $display = 'hide-news';
                    }
                    $array []= array('id' => $post_id, 'title' => get_the_title($post_id), 'date' => get_the_date($date_format, $post_id), 'display' => $display );
                }
            }
        }

        $phone_breakpoint = get_option('lay_breakpoint', 600);
        $phone_breakpoint = (int)$phone_breakpoint;

        $tablet_breakpoint = get_option( 'lay_tablet_breakpoint', 1024 );
        $tablet_breakpoint = (int)$tablet_breakpoint;


        $media_queries = '';
        $preview_desktop = $target == 'modal' ? '.lay-input-modal .preview-desktop ' : '';
        $preview_tablet = $target == 'modal' ? '.lay-input-modal .preview-tablet ' : '';
        $preview_phone = $target == 'modal' ? '.lay-input-modal .preview-phone ' : '';

        if( $target == 'gridder' ) {
            $preview_desktop = '.show-desktop-version ';
            $preview_tablet = '.show-tablet-version ';
            $preview_phone = '.show-phone-version ';
        }

        if( $target == 'frontend' ) {
            $media_queries .= '@media (min-width: '.($tablet_breakpoint+1).'px){';
        }
        $media_queries .=
            $preview_desktop.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row{
                width: calc( 100% / '.$colCountDesktop.' - '.($colCountDesktop-1).' * '.$colGutterDesktop.$colGutterMuDesktop.' / '.$colCountDesktop.' );
                margin-right: '.$colGutterDesktop.$colGutterMuDesktop.';
                margin-bottom: '.$rowGutterDesktop.$rowGutterMuDesktop.';
            }
            '.$preview_desktop.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row:nth-child('.$colCountDesktop.'n){ margin-right: 0; }';
        if( $target == 'frontend' ) {
            $media_queries .= '}';
        }

        if( $target == 'frontend' ) {
            $media_queries .= '@media (max-width: '.($tablet_breakpoint).'px) and (min-width: '.($phone_breakpoint+1).'px){';
        }
        $media_queries .=
            $preview_tablet.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row{
                width: calc( 100% / '.$colCountTablet.' - '.($colCountTablet-1).' * '.$colGutterTablet.$colGutterMuTablet.' / '.$colCountTablet.' );
                margin-right: '.$colGutterTablet.$colGutterMuTablet.';
                margin-bottom: '.$rowGutterTablet.$rowGutterMuTablet.';
            }
            '.$preview_tablet.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row:nth-child('.$colCountTablet.'n){ margin-right: 0; }';
        if( $target == 'frontend' ) {
            $media_queries .= '}';
        }

        if( $target == 'frontend' ) {
            $media_queries .= '@media (max-width: '.($phone_breakpoint).'px){';
        }
        $media_queries .=
            $preview_phone.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row{
                width: calc( 100% / '.$colCountPhone.' - '.($colCountPhone-1).' * '.$colGutterPhone.$colGutterMuPhone.' / '.$colCountPhone.' );
                margin-right: '.$colGutterPhone.$colGutterMuPhone.';
                margin-bottom: '.$rowGutterPhone.$rowGutterMuPhone.';
            }
            '.$preview_phone.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row:nth-child('.$colCountPhone.'n){ margin-right: 0; }';
        if( $target == 'frontend' ) {
            $media_queries .= '}';
        }

        $css_prefix = $target == 'modal' ? '.lay-input-modal ' : '';

        $markup =
        '<style>
            '.$media_queries.'
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-image { padding-bottom:'.$imageSpaceBottom.'px; display: block; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row .lay-news-image{
                margin-right: 0;
            }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row .lay-news-element-title-outer-wrap>div{ display: inline-block; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row .lay-news-element-excerpt{ display: block; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-excerpt{ padding-top: '.$config['excerpt_spacetop'].'px!important; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-read-more{ padding-top: '.$config['readmore_spacetop'].'px!important; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-separator{ padding-left: '.$config['separator_space'].'px!important; padding-right: '.$config['separator_space'].'px!important; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-read-more{ display: inline-block; } 
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row{ border-bottom: none; } 
        </style>';

        // show title, show separator, show date
        $show_title = true;
        if( array_key_exists('show_title', $config) ) {
            if($config['show_title'] === 'false' || $config['show_title'] === false) {
                $show_title = false;
            }
        }

        $separator = '<div class="lay-news-element-separator _'.$config['separator_textformat'].'_no_spaces">'.$config['separator_text'].'</div>';
        $show_separator = true;
        if( $config['show_separator'] === 'false' || $config['show_separator'] === false ) {
            $show_separator = false;
            $separator = '';
        }

        $show_date = true;
        if( $config['show_date'] === 'false' || $config['show_date'] === false ) {
            $show_date = false;
        }

        $show_date_first = false;
        if( array_key_exists('show_date_first', $config) && ( $config['show_date_first'] === 'true' || $config['show_date_first'] === true ) ) {
            $show_date_first = true;
        }
        // # show title, show separator, show date

        $markup .= 
        '<div class="lay-news-element-wrap">
        <div class="lay-news-element lay-news-element-layout-columns lay-news-element-alignment-'.$alignment.' lay-news-element-'.$config['uniqueId'].'" data-amount="'.$amount.'">';
            foreach ($array as $news) {

                $href = 'href="'.get_permalink($news['id']).'" data-id="'.$news['id'].'" data-type="news" data-title="'.__($news['title']).'"';
                $isActive = get_post_meta( $news['id'], 'external_link_is_active', false );
                $checked = is_array($isActive) && count($isActive) > 0 && $isActive[0] == 'on' ? "on" : "";
                $externalLink = get_post_meta($news['id'], 'external_link', '');
                $link = is_array($externalLink) && count($externalLink) > 0 ? $externalLink[0] : '';
                $href = $checked == 'on' ? 'href="'.$link.'" target="_blank"' : $href; 

                $disable_link_is_active_metabox_value = get_post_meta( $news['id'], 'disable_link_is_active', false );
                $disable_link_is_active = is_array($disable_link_is_active_metabox_value) && count($disable_link_is_active_metabox_value) > 0 && $disable_link_is_active_metabox_value[0] == 'on' ? "on" : "";
                $anchorTag = $disable_link_is_active == 'on' ? 'div' : 'a';
                if($disable_link_is_active == 'on'){
                    $href = '';
                }

                $separator = '<div class="lay-news-element-separator _'.$config['separator_textformat'].'_no_spaces">'.$config['separator_text'].'</div>';
                if( $show_separator == false ) {
                    $separator = '';
                }
                $date = '<div class="lay-news-element-date _'.$config['date_textformat'].'_no_spaces">'.$news['date'].'</div>';
                if( $show_date == false ) {
                    $date = '';
                }
                $excerpt_content = get_the_excerpt($news['id']);
                $excerpt = '<div class="lay-news-element-excerpt lay-link-parent _'.$config['excerpt_textformat'].'_no_spaces">'.$excerpt_content.'</div>';
                if( $config['show_excerpt'] === 'false' || $config['show_excerpt'] === false || trim($excerpt_content) == '' ) {
                    // if there is no excerpt we need a br, so show more doesnt land on same line
                    $excerpt = '<br/>';
                }
                $readmore = '';
                if( $config['show_read_more'] === 'true' ||  $config['show_read_more'] === true ) {
                    $readmore = '<a '.$href.' class="lay-news-element-read-more '.($read_more_underline ? 'link-in-text' : '').' _'.$config['readmore_textformat'].'_no_spaces">'.$config['read_more_text'].'</a>';
                }
                if($disable_link_is_active == 'on'){
                    $readmore = '';
                }

                $attid = get_post_thumbnail_id($news['id']);
                $img = LayElFunctions::getLazyImgByImageId($attid);
                if( $target != 'frontend' ) {
                    $img = wp_get_attachment_image($attid, array(200, 200));
                }
                $src_ = wp_get_attachment_image_src($attid);
                $pb = 0;
                if( is_array($src_) ) {
                    $pb = $src_[2] / $src_[1] * 100;
                }

                if( $img != '' ) {
                    $img = '<div class="ph" style="padding-bottom:'.Lay_LayoutFunctions::replaceCommaWithDot($pb).'%;">'.$img.'</div>';
                }

                // https://qtranslatexteam.wordpress.com/faq/#HowToTranslateCustomFields
                $_title = $show_title ? '<div class="lay-news-element-title _'.$config['title_textformat'].'_no_spaces">'.__($news['title']).'</div>' : '';
                
                $t = '';
                if( $show_date_first ) {
                    $t = $date.$separator.$_title;
                } else {
                    $t = $_title.$separator.$date;
                }
                
                $markup .= 
                '<div class="lay-news-element-row '.$news['display'].'">
                    <'.$anchorTag.' class="lay-news-image" '.$href.' >
                        '.$img.'
                    </'.$anchorTag.'>
                    <div>
                        <'.$anchorTag.' class="lay-news-element-title-outer-wrap" '.$href.'>'.$t.'</'.$anchorTag.'>
                        '.__($excerpt).'
                        '.$readmore.'
                    </div>
                </div>';
            }

        $markup .= 
        '</div>';
        $markup .= 
        $loadmore.'</div>';
        return $markup;

    }

    public static function get_news_feed_list_layout($config, $target, $query, $date_format){

        $ix = 0;
        $amount = -1;
        if( array_key_exists('amount', $config) ) {
            if($config['amount'] > 0) {
                $amount = (int)$config['amount'];
            }
        }

        $loadmore = '';
        if(array_key_exists('showMoreButton', $config) && $config['showMoreButton'] == 'true'){
            $alignmentButton = array_key_exists('moreButtonAlignment', $config) ? $config['moreButtonAlignment'] : 'center';
            $spacetop = array_key_exists('moreButtonSpaceTop', $config) ? $config['moreButtonSpaceTop'] : 10;
            $style = array_key_exists('showMoreButtonStyle', $config) ? $config['showMoreButtonStyle'] : 'laybutton1';
            $text = array_key_exists('moreButtonText', $config) ? $config['moreButtonText'] : 'Load More';
            
            $loadmore = '<div class="lay-news-load-more-wrap" style="justify-content:'.$alignmentButton.';margin-top:'.$spacetop.'px;">
            <div class="lay-news-load-more '.$style.'">'.$text.'</div>
            </div>';
        }

        $array = array();
        if ( $query->have_posts() ) {
            foreach ($query->posts as $post){
                if($amount == -1 || $ix < $amount || $loadmore != ''){
                    $ix++;
                    $post_id = $post->ID;
                    $display = 'show-news';
                    if($loadmore != '' && $ix > $amount){
                        $display = 'hide-news';
                    }
                    $array []= array('id' => $post_id, 'title' => get_the_title($post_id), 'date' => get_the_date($date_format, $post_id), 'display' => $display );
                }
            }
        }

        $phone_breakpoint = get_option('lay_breakpoint', 600);
        $phone_breakpoint = (int)$phone_breakpoint;

        $tablet_breakpoint = get_option( 'lay_tablet_breakpoint', 1024 );
        $tablet_breakpoint = (int)$tablet_breakpoint;

        $css_prefix = $target == 'modal' ? '.lay-input-modal ' : '';

        $lineMarkup = '';
        if( $config['line'] === 'true' || $config['line'] === true  ) {
            $lineMarkup = 
            $css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row{
                border-top: 1px solid '.$config['lineColor'].';
                padding-top: '.$config['space_below_line'].'px;
                margin-bottom: '.$config['space_above_line'].'px;
            }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row:first-child{
                border-top: none;
            }';
        } else {
            $lineMarkup = 
            $css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row{
                margin-bottom: '.$config['space_below_line'].'px;
            }';
        }

        $desktop_width = array_key_exists('desktop', $config) ? $config['desktop']['image_width'] : 180;
        $tablet_width = array_key_exists('tablet', $config) ? $config['tablet']['image_width'] : 150;
        $phone_width = array_key_exists('phone', $config) ? $config['phone']['image_width'] : 100;

        $read_more_underline = array_key_exists('read_more_underline', $config) ? $config['read_more_underline'] : true;
        if( $read_more_underline === 'false' || $read_more_underline === '' ) {
            $read_more_underline = false;
        }
        else if( $read_more_underline === 'true' || $read_more_underline === '1' ) {
            $read_more_underline = true;
        }

        $media_queries = '';
        $preview_desktop = ($target == 'modal') ? '.lay-input-modal .preview-desktop ' : '';
        $preview_tablet = ($target == 'modal') ? '.lay-input-modal .preview-tablet ' : '';
        $preview_phone = ($target == 'modal') ? '.lay-input-modal .preview-phone ' : '';
        
        if( $target == 'gridder' ) {
            $preview_desktop = '.show-desktop-version ';
            $preview_tablet = '.show-tablet-version ';
            $preview_phone = '.show-phone-version ';
        }
        // media queries
        // desktop
        if( $target == 'frontend' ) {
            $media_queries .= '@media (min-width: '.($tablet_breakpoint+1).'px){';
        }
        $media_queries .=
            $preview_desktop.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row .ph{
                width: '.$desktop_width.'px;
            }';
        if( $target == 'frontend' ) {
            $media_queries .= '}';
        }
        // tablet
        if( $target == 'frontend' ) {
            $media_queries .= '@media (max-width: '.($tablet_breakpoint).'px) and (min-width: '.($phone_breakpoint+1).'px){';
        }
        $media_queries .=
            $preview_tablet.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row .ph{
                width: '.$tablet_width.'px;
            }';
        if( $target == 'frontend' ) {
            $media_queries .= '}';
        }
        // phone
        if( $target == 'frontend' ) {
            $media_queries .= '@media (max-width: '.($phone_breakpoint).'px){';
        }
        $media_queries .=
            $preview_phone.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row .ph{
                width: '.$phone_width.'px;
            }';
        if( $target == 'frontend' ) {
            $media_queries .= '}';
        }
        // #end media queries

        // show title, show separator, show date
        $show_title = true;
        if( array_key_exists('show_title', $config) ) {
            if($config['show_title'] === 'false' || $config['show_title'] === false) {
                $show_title = false;
            }
        }

        $separator = '<div class="lay-news-element-separator _'.$config['separator_textformat'].'_no_spaces">'.$config['separator_text'].'</div>';
        $show_separator = true;
        if( $config['show_separator'] === 'false' || $config['show_separator'] === false ) {
            $show_separator = false;
            $separator = '';
        }

        $show_date = true;
        if( $config['show_date'] === 'false' || $config['show_date'] === false ) {
            $show_date = false;
        }
        $show_date_first = false;
        if( array_key_exists('show_date_first', $config) && ( $config['show_date_first'] === 'true' || $config['show_date_first'] === true ) ) {
            $show_date_first = true;
        }
        // # show title, show separator, show date

        $image_spaceright = array_key_exists('image_spaceright', $config) ? $config['image_spaceright'] : 10;
        $markup =
        '<style>
            '.$media_queries.'
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row .lay-news-image{
                margin-right: '.$image_spaceright.'px;
                display: block;
            }
            '.$lineMarkup.'
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row{ width: 100%; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row .lay-news-element-title-outer-wrap>div{ display: inline-block; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-row .lay-news-element-excerpt{ display: block; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-excerpt{ padding-top: '.$config['excerpt_spacetop'].'px!important; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-read-more{ padding-top: '.$config['readmore_spacetop'].'px!important; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-separator{ padding-left: '.$config['separator_space'].'px!important; padding-right: '.$config['separator_space'].'px!important; }
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-element-read-more{ display: inline-block; } 
            '.$css_prefix.'.lay-news-element-'.$config['uniqueId'].' .lay-news-image { margin-bottom:0px; }
        </style>
        <div class="lay-news-element-wrap">
        <div class="lay-news-element lay-news-element-layout-list lay-news-element-'.$config['uniqueId'].'" data-amount="'.$amount.'">';
            foreach ($array as $news) {

                $href = 'href="'.get_permalink($news['id']).'" data-id="'.$news['id'].'" data-type="news" data-title="'.__($news['title']).'"';
                $isActive = get_post_meta( $news['id'], 'external_link_is_active', false );
                $checked = is_array($isActive) && count($isActive) > 0 && $isActive[0] == 'on' ? "on" : "";
                $externalLink = get_post_meta($news['id'], 'external_link', '');
                $link = is_array($externalLink) && count($externalLink) > 0 ? $externalLink[0] : '';
                $href = $checked == 'on' ? 'href="'.$link.'" target="_blank"' : $href; 

                $disable_link_is_active_metabox_value = get_post_meta( $news['id'], 'disable_link_is_active', false );
                $disable_link_is_active = is_array($disable_link_is_active_metabox_value) && count($disable_link_is_active_metabox_value) > 0 && $disable_link_is_active_metabox_value[0] == 'on' ? "on" : "";
                $anchorTag = $disable_link_is_active == 'on' ? 'div' : 'a';
                if($disable_link_is_active == 'on'){
                    $href = '';
                }

                $date = '<div class="lay-news-element-date _'.$config['date_textformat'].'_no_spaces">'.$news['date'].'</div>';
                if( $show_date == false ) {
                    $date = '';
                }
                $excerpt_content = get_the_excerpt($news['id']);
                $excerpt = '<div class="lay-news-element-excerpt lay-link-parent _'.$config['excerpt_textformat'].'_no_spaces">'.$excerpt_content.'</div>';
                if( $config['show_excerpt'] === 'false' || $config['show_excerpt'] === false || trim($excerpt_content) == '' ) {
                    // if there is no excerpt we need a br, so show more doesnt land on same line
                    $excerpt = '<br/>';
                }
                $readmore = '';
                if( $config['show_read_more'] === 'true' ||  $config['show_read_more'] === true ) {
                    $readmore = '<a '.$href.' class="lay-news-element-read-more '.($read_more_underline ? 'link-in-text' : '').' _'.$config['readmore_textformat'].'_no_spaces">'.$config['read_more_text'].'</a>';
                }
                if($disable_link_is_active == 'on'){
                    $readmore = '';
                }

                $attid = get_post_thumbnail_id($news['id']);
                $img = LayElFunctions::getLazyImgByImageId($attid);
                if( $target != 'frontend' ) {
                    $img = wp_get_attachment_image($attid, array(200, 200));
                }
                $src_ = wp_get_attachment_image_src($attid);
                $pb = 0;
                if( is_array($src_) ) {
                    $pb = $src_[2] / $src_[1] * 100;
                }

                if( $img != '' ) {
                    $img = '<div class="ph" style="padding-bottom:'.Lay_LayoutFunctions::replaceCommaWithDot($pb).'%;">'.$img.'</div>';
                }

                $_title = $show_title ? '<div class="lay-news-element-title _'.$config['title_textformat'].'_no_spaces">'.__($news['title']).'</div>' : '';
                
                $t = '';
                if( $show_date_first ) {
                    $t = $date.$separator.$_title;
                } else {
                    $t = $_title.$separator.$date;
                }
                // https://qtranslatexteam.wordpress.com/faq/#HowToTranslateCustomFields
                $markup .= 
                '<div class="lay-news-element-row '.$news['display'].'">
                    <'.$anchorTag.' class="lay-news-image" '.$href.'>
                        '.$img.'
                    </'.$anchorTag.'>
                    <div>
                        <'.$anchorTag.' class="lay-news-element-title-outer-wrap" '.$href.'>'.$t.'</'.$anchorTag.'>
                        '.__($excerpt).'
                        '.$readmore.'
                    </div>
                </div>';
            }

        $markup .= 
        '</div>'.$loadmore;
        $markup .= 
        '</div>';
        return $markup;
    }

    public static function lay_register_news_post_type(){
        $name = get_option( 'misc_options_news_feature_name', 'News' );
        $slug = get_option( 'misc_options_news_feature_slug', 'news' );


        register_post_type('lay_news',
            array(
                'show_in_menu' => true,
                'menu_position' => 5,
                'labels' => array(
                    'name' => $name
                ),
                'rewrite' => array( 
                    'slug' => $slug, // use this slug instead of post type name
                    'with_front' => FALSE // if you have a permalink base such as /news/ then setting this to false ensures your custom post type permalink structure will be /products/ instead of /blog/products/
                ),
                'public'      => true,
                'has_archive' => true,
                'menu_icon' => 'dashicons-welcome-write-blog',
                'supports' => array( 'title', 'excerpt', 'thumbnail', 'author' )
            )
        );

        register_taxonomy('lay_news_category', 'lay_news', array(
            'labels' => array(
                'name' => 'News Category'
            ),
            'hierarchical' => true,
            'public'      => true,
            'show_admin_column' => true
        ));
    }

}
new LayNews();