<?php
// http://codex.wordpress.org/Settings_API#Examples
class MiscOptions{

	public static $phoneLayoutActive;
	public static $phone_breakpoint;
	public static $tablet_breakpoint;
	public static $image_alt_tag;
    public static $locked;
    public static $unlock_notice;
    public static $disabled_input;

	public function __construct(){
		add_action( 'admin_menu', array($this, 'misc_options_setup_menu'), 10 );
		add_action( 'admin_init', array($this, 'misc_options_settings_api_init') );
		add_action( 'admin_enqueue_scripts', array($this, 'enqueue_scripts_for_misc') );
		add_action( 'wp_head', array($this, 'misc_options_css') );
		add_action( 'admin_head', array($this, 'misc_options_css_for_admin'));

        MiscOptions::$locked = true;
        $key = get_option('lay_license_key', '');
        if( $key != "" ) {
            MiscOptions::$locked = false;
        }

        MiscOptions::$disabled_input = MiscOptions::$locked == true ? 'disabled="disabled"' : '';
        MiscOptions::$unlock_notice = '';
        if( MiscOptions::$locked == true ) {
            MiscOptions::$unlock_notice = '<a class="lay-buy-license-key" href="https://laytheme.com/buy-now.html" target="_blank"><span class="dashicons dashicons-lock"></span><span class="dashicons dashicons-unlock"></span>'.__('Buy License Key to Unlock', 'lay').'</a>';
        }

		MiscOptions::$phoneLayoutActive = get_option('misc_options_extra_gridder_for_phone', 'on');
		if(MiscOptions::$phoneLayoutActive == "on" && !MiscOptions::$locked){
			MiscOptions::$phoneLayoutActive = true;
		}else{
			MiscOptions::$phoneLayoutActive = false;
		}

		MiscOptions::$phone_breakpoint = get_option('lay_breakpoint', 600);
		MiscOptions::$phone_breakpoint = (int)MiscOptions::$phone_breakpoint;
		MiscOptions::$tablet_breakpoint = get_option( 'misc_options_textformats_tablet_breakpoint', 1024 );
		MiscOptions::$tablet_breakpoint = (int)MiscOptions::$tablet_breakpoint;
		MiscOptions::$image_alt_tag = strip_tags(get_option( 'misc_options_image_alt_tag', '' ));

	}

	public static function misc_options_css_for_admin(){
		$screen = get_current_screen();
		if ( LTUtility::is_admin_page_with_a_gridder($screen) ) {
			MiscOptions::horizontal_lines_css();
            MiscOptions::vertical_lines_css();
		}
	}

	public static function misc_options_css(){
		$placeholder_color = get_option( 'misc_options_image_placeholder_color', '' );
		if($placeholder_color != ""){
			// todo: should i also add this to images in elementgrid for example?
            echo
			'<!-- image placeholder background color -->
			<style>
				.col.type-img .ph,
                .col.type-elementgrid .ph{
				    background-color: '.$placeholder_color.';
				}
                
			</style>';
		}
		MiscOptions::horizontal_lines_css();
        MiscOptions::vertical_lines_css();
	}

	public static function horizontal_lines_css(){
		// horizontal lines
		$hr_height = get_option( 'misc_options_hr_height', '1' );
		$hr_color = get_option( 'misc_options_hr_color', '#000000' );
		echo
		'<!-- horizontal lines -->
		<style>
			.lay-hr{
				height:'.$hr_height.'px;
				background-color:'.$hr_color.';
			}
		</style>';
	}

    public static function vertical_lines_css(){
		// vertical lines
		$vl_width = get_option( 'misc_options_vl_width', '1' );
		$vl_color = get_option( 'misc_options_vl_color', '#000000' );
		echo
		'<!-- vertical lines -->
		<style>
            .element.type-vl{
                width:'.$vl_width.'px!important;
            }
			.lay-vl{
				width:'.$vl_width.'px;
				background-color:'.$vl_color.';
			}
		</style>';
	}

	public function misc_options_setup_menu(){
		// http://wordpress.stackexchange.com/questions/66498/add-menu-page-with-different-name-for-first-submenu-item
		add_menu_page( 'Lay Options', __('Lay Options', 'lay'), 'manage_options', 'manage-layoptions', '', '', 999 );

        add_submenu_page( 'manage-layoptions', 'Lay Options', __('Lay Options', 'lay'), 'manage_options', 'manage-layoptions', array($this, 'misc_options_markup') );
	}

	public function misc_options_markup(){
		require_once( Setup::$dir.'/options/misc_options_markup.php' );
	}



	public function enqueue_scripts_for_misc($hook){
        if( strpos($hook, 'manage-layoptions') !== false ){
			wp_enqueue_media();

			wp_enqueue_style( 'wp-color-picker' );
	        wp_enqueue_script( 'misc_options-colorpicker_controller', Setup::$uri.'/options/assets/js/misc_colorpicker_controller.js', array( 'wp-color-picker' ), Setup::$ver, true );

			wp_enqueue_script( 'misc_options-image_upload', Setup::$uri.'/options/assets/js/image_upload.js', array(), Setup::$ver );
			wp_enqueue_script( 'misc_options-settings_showhide', Setup::$uri.'/options/assets/js/misc_options_showhide_settings.js', array(), Setup::$ver );
		
            wp_enqueue_script( 'misc_options-colorpalette', Setup::$uri.'/options/assets/js/palette_chooser.min.js', array(), Setup::$ver, true );

        }
	}

	public function misc_options_settings_api_init(){
	 	add_settings_section(
			'extra_features_section',
			__('Extra Features', 'lay'),
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-extra-features"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
		);

        add_settings_section(
			'scroll_transition_section',
			__('Scroll Transitions', 'lay'),
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-scroll-transition"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
		);


        add_settings_section(
			'transition_section',
			__('Transitions when Navigating', 'lay'),
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-transition"></div>',
                'after_section'  => '<p>'.__('Transitions will not work if: WooCommerce is enabled, Shortcode is on a page or footer, "Disable Ajax / Compatibility Mode" is active<br/>or if you created a link by coding it yourself and it doesn\'t contain the data-* attributes.', 'lay').'</p>',
                'section_class'  => '',
            )
		);

	 	add_settings_section(
			'textformats_settings_section',
			__('Textformats Settings', 'lay'),
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-textformats"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
		);

		add_settings_section(
			'lazyloading_section',
			__('Lazy Loading', 'lay'),
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-lazyloading"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
		);

 	 	add_settings_section(
 			'images_section',
 			__('Images', 'lay'),
 			'',
 			'manage-miscoptions',
             array(
                 'before_section' => '<div id="lay-images"></div>',
                 'after_section'  => '',
                 'section_class'  => '',
             )
 		);

	 	add_settings_section(
			'appearance_section',
			__('Appearance', 'lay'),
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-appearance"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
		);



		// yoast
		$yoast_message = '';
		if ( is_plugin_active('wordpress-seo/wp-seo.php') ) {
			$yoast_message = 
            '<div class="lay-infobox lay-cyan-border">
                <p>
                    <strong>'.__('Yoast SEO Plugin Warning', 'lay').'</strong><br>
                    '.__('Because you have Yoast activated, none of these Meta Tags will be used. Instead, Yoast\'s Meta Tags will be used.', 'lay').'
                </p>
                <p>
                    '.__('For Yoast\'s metatags to work on your Frontpage you need to do this:', 'lay').'<br/>
                    '.__('Set your Frontpage in "Settings" → "Reading" → "Your homepage displays".', 'lay').'<br/>
                    '.__('Set "Your homepage displays" to "A static page." Set the "Homepage" to your Frontpage and "Posts page" to "- Select -".', 'lay').'<br/>
                    '.__('This does not actually set Lay Theme\'s Frontpage, but that setting lets Yoast know which page to get the meta tags from.', 'lay').'
                </p>
            </div>';
		}
	 	add_settings_section(
			'meta_section',
			__('Meta Tags', 'lay').' '.$yoast_message,
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-meta"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
		);

	 	add_settings_section(
			'hr_section',
			__('Horizontal Lines', 'lay'),
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-horizontal-lines"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
		);

        add_settings_section(
            'vl_section',
            __('Vertical Lines', 'lay'),
            '',
            'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-vertical-lines"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
        );

	 	add_settings_section(
			'other_section',
			__('Other', 'lay'),
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-other"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
		);

        add_settings_section(
			'colorpalette_section',
			__('Color Palette', 'lay'),
			'',
			'manage-miscoptions',
            array(
                'before_section' => '<div id="lay-colorpalette"></div>',
                'after_section'  => '',
                'section_class'  => '',
            )
		);

		// extra features
		add_settings_field(
			'misc_options_menu_amount',
			__('Menu Amount', 'lay'),
			array($this, 'misc_setting_menu_amount'),
			'manage-miscoptions',
			'extra_features_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_menu_amount' );

 	 	add_settings_field(
 			'misc_options_extra_gridder_for_phone',
 			__('Activate Custom Phone Layouts', 'lay'),
 			array($this, 'misc_setting_extra_gridder_for_phone'),
 			'manage-miscoptions',
 			'extra_features_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_extra_gridder_for_phone' );

          add_settings_field(
            'misc_options_gridder_sidebar',
            __('Show Sidebar for extended Settings in Gridder', 'lay'),
            array($this, 'misc_setting_gridder_sidebar'),
            'manage-miscoptions',
            'extra_features_section'
        );
         register_setting( 'manage-miscoptions', 'misc_options_gridder_sidebar' );

        add_settings_field(
            'misc_options_activate_projects_overlay_feature',
            __('Activate Projects Overlays Feature', 'lay'),
            array($this, 'misc_setting_activate_projects_overlay_feature'),
            'manage-miscoptions',
            'extra_features_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_activate_projects_overlay_feature' );

        add_settings_field(
            'misc_options_activate_overlay_feature',
            __('Activate Page Overlays / Desktop Burger Menu Feature', 'lay'),
            array($this, 'misc_setting_activate_overlay_feature'),
            'manage-miscoptions',
            'extra_features_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_activate_overlay_feature' );

		$news_feature = true;
		if( $news_feature == true ) {
			add_settings_field(
				'misc_options_activate_news_feature',
				__('Activate News Feature', 'lay'),
				array($this, 'misc_setting_activate_news_feature'),
				'manage-miscoptions',
				'extra_features_section'
			);
			register_setting( 'manage-miscoptions', 'misc_options_activate_news_feature' );
	
			add_settings_field(
				'misc_options_news_feature_name',
				__('News Feature Name', 'lay'),
				array($this, 'misc_setting_news_feature_name'),
				'manage-miscoptions',
				'extra_features_section'
			);
			register_setting( 'manage-miscoptions', 'misc_options_news_feature_name' );
	
			add_settings_field(
				'misc_options_news_feature_slug',
				__('News Feature Slug', 'lay'),
				array($this, 'misc_setting_news_feature_slug'),
				'manage-miscoptions',
				'extra_features_section'
			);
			register_setting( 'manage-miscoptions', 'misc_options_news_feature_slug' );	
		}



        add_settings_field(
            'misc_options_cover',
            __('Activate Cover Feature', 'lay'),
            array($this, 'misc_setting_cover'),
            'manage-miscoptions',
            'extra_features_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_cover' );

        add_settings_field(
            'misc_options_activate_project_description',
            __('Activate Project Descriptions for Thumbnails', 'lay'),
            array($this, 'misc_setting_activate_project_description'),
            'manage-miscoptions',
            'extra_features_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_activate_project_description' );

        add_settings_field(
            'misc_options_intro',
            __('Activate Intro Feature', 'lay'),
            array($this, 'misc_setting_intro'),
            'manage-miscoptions',
            'extra_features_section'
        );
         register_setting( 'manage-miscoptions', 'misc_options_intro' );

        add_settings_field(
            'misc_options_simple_parallax',
            __('Activate Simple Parallax', 'lay'),
            array($this, 'misc_setting_simple_parallax'),
            'manage-miscoptions',
            'extra_features_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_simple_parallax' );
        
        add_settings_field(
            'misc_options_thumbnail_mouseover_image',
            __('Activate Mouseover Image for Thumbnails', 'lay'),
            array($this, 'misc_setting_thumbnail_mouseover_image'),
            'manage-miscoptions',
            'extra_features_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_thumbnail_mouseover_image' );

        add_settings_field(
            'misc_options_thumbnail_video',
            __('Activate Video for Thumbnails', 'lay'),
            array($this, 'misc_setting_thumbnail_video'),
            'manage-miscoptions',
            'extra_features_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_thumbnail_video' );

        add_settings_field(
            'misc_options_thumbnail_video_and_mouseover_image_behavior',
            __('Behavior when both Mouseover Image and Video are used for Thumbnails', 'lay'),
            array($this, 'misc_setting_thumbnail_video_and_mouseover_image_behavior'),
            'manage-miscoptions',
            'extra_features_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_thumbnail_video_and_mouseover_image_behavior' );

        add_settings_field(
            'misc_options_show_project_arrows',
            __('Show arrows in projects for navigation (Project Arrows)', 'lay'),
            array($this, 'misc_setting_show_project_arrows'),
            'manage-miscoptions',
            'extra_features_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_show_project_arrows' );

        if( is_plugin_active('woocommerce/woocommerce.php') ){
            add_settings_field(
                'misc_options_activate_gridder_for_products',
                __('Activate Gridder for WooCommerce Products', 'lay'),
                array($this, 'misc_setting_activate_gridder_for_products'),
                'manage-miscoptions',
                'extra_features_section'
            );
            register_setting( 'manage-miscoptions', 'misc_options_activate_gridder_for_products' );
        }

 	 	// textformats settings section
 	 	add_settings_field(
 			'misc_options_textformats_for_tablet',
 			__('Add "Tablet" settings to Textformats', 'lay'),
 			array($this, 'textformats_for_tablet_setting'),
 			'manage-miscoptions',
 			'textformats_settings_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_textformats_for_tablet' );

 	 	add_settings_field(
 			'misc_options_textformats_tablet_breakpoint',
 			__('Tablet Breakpoint for Textformats', 'lay'),
 			array($this, 'textformats_tablet_breakpoint_setting'),
 			'manage-miscoptions',
 			'textformats_settings_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_textformats_tablet_breakpoint' );

        add_settings_field(
            'misc_options_textformats_lineheight_in_px',
            __('Line Height can be px', 'lay'),
            array($this, 'textformats_lineheight_in_px_setting'),
            'manage-miscoptions',
            'textformats_settings_section'
        );
         register_setting( 'manage-miscoptions', 'misc_options_textformats_lineheight_in_px' );

		// lazy loading section
		add_settings_field(
			'misc_options_image_loading',
			__('Use Lazy Loading for Images', 'lay'),
			array($this, 'misc_setting_image_loading'),
			'manage-miscoptions',
			'lazyloading_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_image_loading' );

        add_settings_field(
			'misc_options_image_loading_blur',
			__('Use Blur for Image Lazy Loading', 'lay'),
			array($this, 'misc_setting_image_loading_blur'),
			'manage-miscoptions',
			'lazyloading_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_image_loading_blur' );

        add_settings_field(
			'misc_options_videos_lazyload',
			__('Use Lazy Loading for HTML5 Videos', 'lay'),
			array($this, 'misc_setting_videos_lazyload'),
			'manage-miscoptions',
			'lazyloading_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_videos_lazyload' );

        add_settings_field(
            'misc_options_enable_video_lazyloading_for_safari',
            __('Use HTML5 Video Lazyloading for Safari', 'lay'),
            array($this, 'misc_setting_enable_video_lazyloading_for_safari'),
            'manage-miscoptions',
            'lazyloading_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_enable_video_lazyloading_for_safari' );

        add_settings_field(
			'misc_options_enable_video_lazyloading_for_firefox',
			__('Use HTML5 Video Lazyloading for Firefox', 'lay'),
			array($this, 'misc_setting_enable_video_lazyloading_for_firefox'),
			'manage-miscoptions',
			'lazyloading_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_enable_video_lazyloading_for_firefox' );

 	 	// images section
 	 	add_settings_field(
 			'misc_options_image_placeholder_color',
 			__('Image Placeholder Background Color', 'lay'),
 			array($this, 'misc_setting_image_placeholder_color'),
 			'manage-miscoptions',
 			'images_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_image_placeholder_color' );

 	 	add_settings_field(
 			'misc_options_image_quality',
 			__('Image Quality (.jpeg, )', 'lay'),
 			array($this, 'misc_setting_image_quality'),
 			'manage-miscoptions',
 			'images_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_image_quality' );

 	 	add_settings_field(
 			'misc_options_showoriginalimages',
 			__('Never show resized versions of your images', 'lay'),
 			array($this, 'misc_setting_showoriginalimages'),
 			'manage-miscoptions',
 			'images_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_showoriginalimages' );


        add_settings_field(
            'misc_options_showoriginalimages_for_phone',
            __('Never show resized versions of your images for Phone', 'lay'),
            array($this, 'misc_setting_showoriginalimages_for_phone'),
            'manage-miscoptions',
            'images_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_showoriginalimages_for_phone' );


		add_settings_field(
			'misc_options_image_alt_tag',
			__('Add to Image Alt Tag', 'lay'),
			array($this, 'misc_setting_image_alt_tag'),
			'manage-miscoptions',
			'images_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_image_alt_tag' );

        add_settings_field(
			'misc_options_image_alt_tag_to_use',
			__('Alt Text to use', 'lay'),
			array($this, 'misc_setting_image_alt_tag_to_use'),
			'manage-miscoptions',
			'images_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_image_alt_tag_to_use' );


        // scroll transition section
        add_settings_field(
			'misc_options_element_transition_on_scroll',
			__('Activate on Scroll Element Transitions', 'lay'),
			array($this, 'misc_setting_element_transition_on_scroll'),
			'manage-miscoptions',
			'scroll_transition_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_element_transition_on_scroll' );

        add_settings_field(
			'misc_options_element_transition_animate_opacity',
			__('Animate Opacity', 'lay'),
			array($this, 'misc_setting_element_transition_animate_opacity'),
			'manage-miscoptions',
			'scroll_transition_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_element_transition_animate_opacity' );

        add_settings_field(
            'misc_options_scroll_transition_duration',
            __('Transition Duration', 'lay'),
            array($this, 'misc_setting_scroll_transition_duration'),
            'manage-miscoptions',
            'scroll_transition_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_scroll_transition_duration' );

        add_settings_field(
            'misc_options_scroll_transition_easing',
            __('CSS Easing', 'lay'),
            array($this, 'misc_setting_scroll_transition_easing'),
            'manage-miscoptions',
            'scroll_transition_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_scroll_transition_easing' );

        add_settings_field(
            'misc_options_scroll_transition_offset_desktop',
            __('Offset Desktop', 'lay'),
            array($this, 'misc_setting_scroll_transition_offset_desktop'),
            'manage-miscoptions',
            'scroll_transition_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_scroll_transition_offset_desktop' );

        add_settings_field(
            'misc_options_scroll_transition_offset_mobile',
            __('Offset Mobile', 'lay'),
            array($this, 'misc_setting_scroll_transition_offset_mobile'),
            'manage-miscoptions',
            'scroll_transition_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_scroll_transition_offset_mobile' );

        // add_settings_field(
        //     'misc_options_scroll_transition_easing',
        //     __('Transition Easing', 'lay'),
        //     array($this, 'misc_setting_scroll_transition_easing'),
        //     'manage-miscoptions',
        //     'scroll_transition_section'
        // );
        // register_setting( 'manage-miscoptions', 'misc_options_scroll_transition_easing' );

        // transition section
        add_settings_field(
            'misc_options_use_revealing_transition_on_first_visit',
            __('Use "Showing Transition" when first visiting website or hard reload', 'lay'),
            array($this, 'misc_setting_use_revealing_transition_on_first_visit'),
            'manage-miscoptions',
            'transition_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_use_revealing_transition_on_first_visit' );

        add_settings_field(
            'misc_options_navigation_transition_in',
            __('Showing Transition', 'lay'),
            array($this, 'misc_setting_navigation_transition_in'),
            'manage-miscoptions',
            'transition_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_navigation_transition_in' );

        add_settings_field(
            'misc_options_navigation_transition_in_duration',
            __('Showing Transition Duration', 'lay'),
            array($this, 'misc_setting_navigation_transition_in_duration'),
            'manage-miscoptions',
            'transition_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_navigation_transition_in_duration' );
        
    add_settings_field(
        'misc_options_navigation_transition_in_easing',
        __('Showing Transition Easing', 'lay'),
        array($this, 'misc_setting_navigation_transition_in_easing'),
        'manage-miscoptions',
        'transition_section'
    );
    register_setting( 'manage-miscoptions', 'misc_options_navigation_transition_in_easing' );

    add_settings_field(
        'misc_options_navigation_transition_out',
        __('Hiding Transition', 'lay'),
        array($this, 'misc_setting_navigation_transition_out'),
        'manage-miscoptions',
        'transition_section'
    );
    register_setting( 'manage-miscoptions', 'misc_options_navigation_transition_out' );

    add_settings_field(
        'misc_options_navigation_transition_out_duration',
        __('Hiding Transition Duration', 'lay'),
        array($this, 'misc_setting_navigation_transition_out_duration'),
        'manage-miscoptions',
        'transition_section'
    );
    register_setting( 'manage-miscoptions', 'misc_options_navigation_transition_out_duration' );

    add_settings_field(
        'misc_options_navigation_transition_out_easing',
        __('Hiding Transition Easing', 'lay'),
        array($this, 'misc_setting_navigation_transition_out_easing'),
        'manage-miscoptions',
        'transition_section'
    );
    register_setting( 'manage-miscoptions', 'misc_options_navigation_transition_out_easing' );

    add_settings_field(
        'misc_options_navigation_transition_y_translate_desktop',
        __('Up/Down Desktop', 'lay'),
        array($this, 'misc_setting_navigation_transition_y_translate_desktop'),
        'manage-miscoptions',
        'transition_section'
    );
    register_setting( 'manage-miscoptions', 'misc_options_navigation_transition_y_translate_desktop' );

    add_settings_field(
        'misc_options_navigation_transition_y_translate_phone',
        __('Up/Down Phone', 'lay'),
        array($this, 'misc_setting_navigation_transition_y_translate_phone'),
        'manage-miscoptions',
        'transition_section'
    );
    register_setting( 'manage-miscoptions', 'misc_options_navigation_transition_y_translate_phone' );

 	 	// appearance section
 	 	add_settings_field(
 			'lay_breakpoint',
 			__('Phone Breakpoint', 'lay'),
 			array($this, 'misc_setting_lay_breakpoint'),
 			'manage-miscoptions',
 			'appearance_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'lay_breakpoint' );

		add_settings_field(
			'lay_tablet_breakpoint',
			__('Tablet Breakpoint', 'lay'),
			array($this, 'misc_setting_lay_tablet_breakpoint'),
			'manage-miscoptions',
			'appearance_section'
		);
		register_setting( 'manage-miscoptions', 'lay_tablet_breakpoint' );

 	 	add_settings_field(
 			'misc_options_max_width',
 			__('Max width of content', 'lay'),
 			array($this, 'misc_setting_max_width'),
 			'manage-miscoptions',
 			'appearance_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_max_width' );

 	 	add_settings_field(
 			'misc_options_max_width_apply_to_logo_and_nav',
 			__('Apply max width of content to logo and menu', 'lay'),
 			array($this, 'misc_setting_max_width_apply_to_logo_and_nav'),
 			'manage-miscoptions',
 			'appearance_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_max_width_apply_to_logo_and_nav' );

 	 	add_settings_field(
 			'misc_options_html5video_playicon',
 			__('HTML5 Video Play Icon', 'lay'),
 			array($this, 'misc_setting_html5video_playicon'),
 			'manage-miscoptions',
 			'appearance_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_html5video_playicon' );

        //colorpalette section
        add_settings_field(
            'misc_options_colorpalette',
            __('Color Palette', 'lay'),
            array($this, 'misc_setting_colorpalette'),
            'manage-miscoptions',
            'colorpalette_section'
        );
         register_setting( 'manage-miscoptions', 'misc_options_colorpalette' );
        

 	 	// meta section
 	 	add_settings_field(
 			'misc_options_website_description',
 			__('Website Description', 'lay'),
 			array($this, 'misc_setting_website_description'),
 			'manage-miscoptions',
 			'meta_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_website_description' );

 	 	add_settings_field(
 			'misc_options_fbimage',
 			__('Facebook/Twitter Share Image', 'lay'),
 			array($this, 'misc_setting_fbimage'),
 			'manage-miscoptions',
 			'meta_section'
 		);
		register_setting( 'manage-miscoptions', 'misc_options_fbimage' );
		  
 	 	// hr section
 	 	add_settings_field(
 			'misc_options_hr_color',
 			__('Line Color', 'lay'),
 			array($this, 'misc_setting_hr_color'),
 			'manage-miscoptions',
 			'hr_section'
 		);
 	 	register_setting( 'manage-miscoptions', 'misc_options_hr_color' );

 	 	add_settings_field(
 			'misc_options_hr_height',
 			__('Line Strokewidth', 'lay'),
 			array($this, 'misc_setting_hr_height'),
 			'manage-miscoptions',
 			'hr_section'
 		);
		register_setting( 'manage-miscoptions', 'misc_options_hr_height' );

        // vl section
        add_settings_field(
            'misc_options_vl_color',
            __('Line Color', 'lay'),
            array($this, 'misc_setting_vl_color'),
            'manage-miscoptions',
            'vl_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_vl_color' );

        add_settings_field(
            'misc_options_vl_width',
            __('Line Strokewidth', 'lay'),
            array($this, 'misc_setting_vl_width'),
            'manage-miscoptions',
            'vl_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_vl_width' );
		  
		// other
		add_settings_field(
			'misc_options_title_separator',
			__('Title Separator', 'lay'),
			array($this, 'misc_setting_title_separator'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_title_separator' );

        add_settings_field(
            'misc_options_prevnext_navigate_through',
            __('Next & previous project links navigate through:', 'lay'),
            array($this, 'misc_setting_prevnext_navigate_through'),
            'manage-miscoptions',
            'other_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_prevnext_navigate_through' );

		add_settings_field(
			'misc_options_disable_ajax',
			__('Disable Ajax / Compatibility Mode', 'lay'),
			array($this, 'misc_setting_disable_ajax'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_disable_ajax' );

        add_settings_field(
			'misc_options_enable_content_filter',
			__('Enable Content Filter', 'lay'),
			array($this, 'misc_setting_enable_content_filter'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_enable_content_filter' );

		add_settings_field(
			'misc_options_anchorscroll_offset_desktop',
			__('Anchor-/Expandrow Scroll Space Top for Desktop', 'lay'),
			array($this, 'misc_setting_anchorscroll_offset_desktop'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_anchorscroll_offset_desktop' );

		add_settings_field(
			'misc_options_anchorscroll_offset_phone',
			__('Anchor-/Expandrow Scroll Space Top for Phone', 'lay'),
			array($this, 'misc_setting_anchorscroll_offset_phone'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_anchorscroll_offset_phone' );

		add_settings_field(
			'misc_options_show_password_protected_posts_in_thumbnailgrid',
			__('Show Password Protected Projects in Thumbnailgrid', 'lay'),
			array($this, 'misc_setting_show_password_protected_posts_in_thumbnailgrid'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_show_password_protected_posts_in_thumbnailgrid' );

        add_settings_field(
			'misc_options_hide_current_project_from_thumbnailgrid',
			__('Hide current Project from Thumbnailgrid & Project Index Elements', 'lay'),
			array($this, 'misc_setting_hide_current_project_from_thumbnailgrid'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_hide_current_project_from_thumbnailgrid' );

        add_settings_field(
            'misc_options_never_change_scrolling_position',
            __('Never change scrolling position when navigating', 'lay'),
            array($this, 'misc_setting_never_change_scrolling_position'),
            'manage-miscoptions',
            'other_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_never_change_scrolling_position' );

        add_settings_field(
            'misc_options_swap_webfonts',
            __('Enable "font-display: swap;"', 'lay'),
            array($this, 'misc_setting_swap_webfonts'),
            'manage-miscoptions',
            'other_section'
        );
        register_setting( 'manage-miscoptions', 'misc_options_swap_webfonts' );



        add_settings_field(
			'misc_options_enable_frame_overflows_for_apl',
			__('Enable Frame Overflows for automatically generated phone layouts', 'lay'),
			array($this, 'misc_setting_enable_frame_overflows_for_apl'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_enable_frame_overflows_for_apl' );

        // im adding the same setting here as in the carousel addon
        // the youtube script adds youtube js and cookies, to be able to pause the video programmatically
        // when closing the overlay!
        add_settings_field(
			'laycarousel_youtube',
			__('Add YouTube Script', 'lay'),
			array($this, 'laycarousel_youtube_setting'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'laycarousel_youtube' );

        add_settings_field(
			'misc_options_expire_post_login_after',
			__('Expire Project/Page Password Login after', 'lay'),
			array($this, 'misc_options_expire_post_login_after_setting'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_expire_post_login_after' );

        add_settings_field(
			'misc_options_hide_menu_and_sitetitle_on_pw',
			__('Hide Site Title and Menu on password protected pages', 'lay'),
			array($this, 'misc_options_hide_menu_and_sitetitle_on_pw_setting'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_hide_menu_and_sitetitle_on_pw' );


        add_settings_field(
			'misc_options_alt_w100_h100_sizing',
			__('Use row background image/video object fit CSS compatibility mode', 'lay'),
			array($this, 'misc_options_alt_w100_h100_sizing_setting'),
			'manage-miscoptions',
			'other_section'
		);
		register_setting( 'manage-miscoptions', 'misc_options_alt_w100_h100_sizing' );
	}

    public static function misc_options_alt_w100_h100_sizing_setting(){
        $val = get_option( 'misc_options_alt_w100_h100_sizing', "" );
        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input type="checkbox" name="misc_options_alt_w100_h100_sizing" id="misc_options_alt_w100_h100_sizing" '.$checked.'><br>
        <label for="misc_options_alt_w100_h100_sizing">'.__('Use an alternative to object fit css for row background images and videos.<br>
        Use this setting if sizing of these images/videos looks weird on mobile devices.', 'lay').'</label>';
    }

    public static function misc_options_hide_menu_and_sitetitle_on_pw_setting(){
        $val = get_option( 'misc_options_hide_menu_and_sitetitle_on_pw', "" );
        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input type="checkbox" name="misc_options_hide_menu_and_sitetitle_on_pw" id="misc_options_hide_menu_and_sitetitle_on_pw" '.$checked.'><br>
        <label for="misc_options_hide_menu_and_sitetitle_on_pw">'.__('Hide the menu and site title on password protected pages.', 'lay').'</label>';
    }

    public static function laycarousel_youtube_setting(){
        $val = get_option( 'laycarousel_youtube', "" );
        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input type="checkbox" name="laycarousel_youtube" id="laycarousel_youtube" '.$checked.'><br>
        <label for="laycarousel_youtube">'.__('Adds extra YouTube script and is important if you have a YouTube video in an overlay so it pauses when you close the overlay.', 'lay').'</label>';
    }

    public static function misc_options_expire_post_login_after_setting(){
        $val = get_option( 'misc_options_expire_post_login_after', 30 );
        echo '<input type="number" name="misc_options_expire_post_login_after" id="misc_options_expire_post_login_after" min="1" step="1" value="'.$val.'"> '.__('minutes', 'lay').'';
    }

    // UNLOCKABLE
    public static function misc_setting_news_feature_name(){
        $val = get_option( 'misc_options_news_feature_name', 'News' );
        if( MiscOptions::$locked ){
            $val = 'News';
        }
        echo '<input '.MiscOptions::$disabled_input.' type="text" name="misc_options_news_feature_name" id="misc_options_news_feature_name" value="'.$val.'">'.MiscOptions::$unlock_notice.'<br>
        '.__('Default is "News"', 'lay').'';
    }

    public static function misc_setting_news_feature_slug(){
        $val = get_option( 'misc_options_news_feature_slug', 'news' );
        if( MiscOptions::$locked ){
            $val = 'news';
        }
        echo '<input '.MiscOptions::$disabled_input.' type="text" name="misc_options_news_feature_slug" id="misc_options_news_feature_slug" value="'.$val.'">'.MiscOptions::$unlock_notice.'<br>
        '.__('Default is "news"', 'lay').'';
    }

    public function misc_setting_element_transition_on_scroll(){
        $val = get_option('misc_options_element_transition_on_scroll', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_element_transition_on_scroll" id="misc_options_element_transition_on_scroll" '.$checked.'>
        <a class="lay-options-doc-link" href="https://laytheme.com/documentation/element-transitions-on-scroll.html" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_element_transition_on_scroll">'.__('Element will animate in when you scroll to it.', 'lay').'</label>';
    }

    public function misc_setting_element_transition_animate_opacity(){
        $val = get_option('misc_options_element_transition_animate_opacity', 'on');
        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_element_transition_animate_opacity" id="misc_options_element_transition_animate_opacity" '.$checked.'>'.MiscOptions::$unlock_notice.'
        <label for="misc_options_element_transition_animate_opacity">'.__('Elements will animate their opacity when they appear.', 'lay').'</label>';
    }

    public function misc_setting_gridder_sidebar(){
        $val = get_option('misc_options_gridder_sidebar', 'on');
        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_gridder_sidebar" id="misc_options_gridder_sidebar" '.$checked.'>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_gridder_sidebar">'.__('When selecting an element in the Gridder, show a sidebar with extended settings.', 'lay').'</label>';
    }

    public function misc_setting_extra_gridder_for_phone(){
        $val = get_option('misc_options_extra_gridder_for_phone', 'on');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_extra_gridder_for_phone" id="misc_options_extra_gridder_for_phone" '.$checked.'>
        <a class="lay-options-doc-link" href="https://laytheme.com/documentation/custom-phone-layouts.html" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_extra_gridder_for_phone">'.__('In the Gridder you will have new buttons to switch between the desktop and phone layout.', 'lay').'</label>';
    }

    public function misc_setting_menu_amount(){
        $val = intval(get_option('misc_options_menu_amount', 1));
        if( MiscOptions::$locked || $val == 0 ) {
            $val = 1;
        }
        $selected1 = $val == 1 ? 'selected' : '';
        $selected2 = $val == 2 ? 'selected' : '';
        $selected3 = $val == 3 ? 'selected' : '';
        $selected4 = $val == 4 ? 'selected' : '';

        echo 
        '<select '.MiscOptions::$disabled_input.' name="misc_options_menu_amount">
            <option value="1" '.$selected1.'>1</option> 
            <option value="2" '.$selected2.'>2</option>
            <option value="3" '.$selected3.'>3</option>
            <option value="4" '.$selected4.'>4</option>
            </select> <a class="lay-options-doc-link" href="https://laytheme.com/documentation/menus.html#multiple-menus" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a> '.MiscOptions::$unlock_notice;
    }

    public function misc_setting_thumbnail_video(){
        $val = get_option('misc_options_thumbnail_video', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_thumbnail_video" id="misc_options_thumbnail_video" '.$checked.'>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_thumbnail_video">'.__('When editing a project you can insert a video as a project thumbnail.', 'lay').'</label>';
    }

    public function misc_setting_thumbnail_video_and_mouseover_image_behavior(){
        $val = get_option('misc_options_thumbnail_video_and_mouseover_image_behavior', 'image');
        if( MiscOptions::$locked ){
            $val = 'image';
        }
        $selected_video = $val == 'video' ? 'checked' : '';
        $selected_image = $val == 'image' ? 'checked' : '';

        // input type radio:
        echo '<input type="radio" name="misc_options_thumbnail_video_and_mouseover_image_behavior" id="image" value="image" '.$selected_image.'><label for="image">'.__('Show Image on Mouseover', 'lay').'</label><br>';
		echo '<input type="radio" name="misc_options_thumbnail_video_and_mouseover_image_behavior" id="video" value="video" '.$selected_video.'><label for="video">'.__('Show Video on Mouseover', 'lay').'</label>';

    }

    public function misc_setting_thumbnail_mouseover_image(){
        $val = get_option('misc_options_thumbnail_mouseover_image', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_thumbnail_mouseover_image" id="misc_options_thumbnail_mouseover_image" '.$checked.'>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_thumbnail_mouseover_image">'.__('When editing a project you can insert another image that will be shown on mouseover.', 'lay').'</label>';
    }

    public function misc_setting_activate_project_description(){
        $val = get_option('misc_options_activate_project_description', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_activate_project_description" id="misc_options_activate_project_description" '.$checked.'>
        <a class="lay-options-doc-link" href="https://laytheme.com/documentation/project-description.html" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_activate_project_description">'.__('A new description textfield will appear above the Gridder when you edit projects.<br>Project descriptions will show up underneath a project thumbnail.', 'lay').'</label>';
    }

    public function misc_setting_simple_parallax(){
        $val = get_option('misc_options_simple_parallax', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        https://laytheme.com/documentation/simple-parallax.html
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_simple_parallax" id="misc_options_simple_parallax" '.$checked.'> <a class="lay-options-doc-link" href="https://laytheme.com/documentation/simple-parallax.html" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.' <br><label for="misc_options_simple_parallax">'.__('Right click on an element in the Gridder and define the "y-parallax" value to change the speed of an element while scrolling.', 'lay').'</label>';
    }

    public function misc_setting_intro(){
        $val = get_option('misc_options_intro', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_intro" id="misc_options_intro" '.$checked.'>
        <a class="lay-options-doc-link" href="https://laytheme.com/documentation/intro-image-or-video.html" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_intro">'.__('When activated, find options in "Customizer" &rarr; "Intro".', 'lay').'</label>';
    }

    public function misc_setting_cover(){
        $val = get_option('misc_options_cover', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_cover" id="misc_options_cover" '.$checked.'>
        <a class="lay-options-doc-link" href="https://laytheme.com/documentation/covers.html" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_cover">'.__('When activated, find options for Covers in "Lay Options" &rarr; "Cover Options".', 'lay').'</label>';
    }

    public function misc_setting_activate_projects_overlay_feature(){
        $val = get_option('misc_options_activate_projects_overlay_feature', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        $disabled = MiscOptions::$disabled_input;
        $extra_description = "";
        if( class_exists( 'WooCommerce' ) ) {
            $checked = "";
            $disabled = "disabled";
            $extra_description = __('When using WooCommerce, this setting will always be disabled.', 'lay');
        }
        
        $misc_options_disable_ajax = get_option( 'misc_options_disable_ajax', "" );
        if( $misc_options_disable_ajax == "on" ){
            $checked = "";
            $disabled = "disabled";
            $extra_description = __('When using "Disable Ajax" this setting will always be disabled.', 'lay');
        }

        echo '<input '.$disabled.' type="checkbox" name="misc_options_activate_projects_overlay_feature" id="misc_options_activate_projects_overlay_feature" '.$checked.'>
        <a class="lay-options-doc-link" href="https://laytheme.com/documentation/projects-overlays.html" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_activate_projects_overlay_feature">'.$extra_description.' '.__('Find settings in "Lay Options" &rarr; "Projects Overlays"', 'lay').'</label>';
    }

		public function misc_setting_activate_gridder_for_products(){
			$val = get_option('misc_options_activate_gridder_for_products', '');
			$checked = "";
			if( $val == "on" && !MiscOptions::$locked ){
				$checked = "checked";
			}
			$disabled = MiscOptions::$disabled_input;
			$extra_description = "";
			if( !class_exists( 'WooCommerce' ) ) {
				$checked = "";
				$disabled = "disabled";
				$extra_description = __('When not using WooCommerce, this setting will always be disabled.', 'lay');
			}
			echo '<input '.$disabled.' type="checkbox" name="misc_options_activate_gridder_for_products" id="misc_options_activate_gridder_for_products" '.$checked.'>
			'.MiscOptions::$unlock_notice.'
			<br><label for="misc_options_activate_gridder_for_products">'.$extra_description.'</label>';
		}

    public function misc_setting_activate_overlay_feature(){
        $val = get_option('misc_options_activate_overlay_feature', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_activate_overlay_feature" id="misc_options_activate_overlay_feature" '.$checked.'>
        <a class="lay-options-doc-link" href="https://laytheme.com/documentation/overlays.html" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.'
        <br/><label for="misc_options_activate_overlay_feature">'.__('Find Overlay settings when you edit pages', 'lay').'</label>
        <br/><strong style="font-size: 12px; line-height: 1.2;">'.__('Please only create a couple of page overlays. If you need many overlays for showing projects, use Projects Overlays.', 'lay').'<br/>
        '.__('If you create like 20 page overlays and link to all of them on one page, this might crash iPhones.', 'lay').'</strong>';
    }

    public function misc_setting_activate_news_feature(){
        $val = get_option('misc_options_activate_news_feature', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_activate_news_feature" id="misc_options_activate_news_feature" '.$checked.'>
        <a class="lay-options-doc-link" href="https://laytheme.com/documentation/newssection.html" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_activate_news_feature">'.__('Use the News feature to add a News section to your website.<br/>Create News posts in "News", then add a News Element to a page in the Gridder with "+More" &rarr; "+News Element".', 'lay').'</label>';
    }

    public function misc_setting_show_project_arrows(){
        $val = get_option('misc_options_show_project_arrows', '');
        $checked = "";
        if( $val == "on" && !MiscOptions::$locked ){
            $checked = "checked";
        }
        echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_show_project_arrows" id="misc_options_show_project_arrows" '.$checked.'>
        <a class="lay-options-doc-link" href="https://laytheme.com/documentation/navigation-between-projects.html#project-arrows" title="Documentation on laytheme.com" target="_blank"><span class="dashicons dashicons-editor-help"></span></a>'.MiscOptions::$unlock_notice.'
        <br><label for="misc_options_show_project_arrows">'.__('In projects on the left and right there will be clickable arrows to go to the previous/next project.<br>In "Customize" when you navigate inside a project you can style the arrows in the "Project Arrows" panel.', 'lay').'</label>';
    }
    // # UNLOCKABLE

	public static function misc_setting_anchorscroll_offset_desktop(){
		$val = get_option( 'misc_options_anchorscroll_offset_desktop', 0 );
		echo
		'<input type="number" step="1" value="'.$val.'" name="misc_options_anchorscroll_offset_desktop" id="misc_options_anchorscroll_offset_desktop"> px
		<br><label for="misc_options_anchorscroll_offset_desktop">'.__('You click on an anchor that is linked to <code>#hey</code>.
		The page scrolls to an element with <code>id="hey"</code>.<br>
		The offset here is the space between your top browser edge and that container.').'</label>';
	}

	public static function misc_setting_anchorscroll_offset_phone(){
		$val = get_option( 'misc_options_anchorscroll_offset_phone', 0 );
		echo
		'<input type="number" step="1" value="'.$val.'" name="misc_options_anchorscroll_offset_phone" id="misc_options_anchorscroll_offset_phone"> px
		<br><label for="misc_options_anchorscroll_offset_phone">'.__('Same as above but just for when you\'re on the phone version.', 'lay').'</label>';
	}

    public static function misc_setting_enable_content_filter(){
        $val = get_option( 'misc_options_enable_content_filter', "" );
        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input type="checkbox" name="misc_options_enable_content_filter" id="misc_options_enable_content_filter" '.$checked.'><br>
        <label for="misc_options_enable_content_filter">'.__('Activate this setting if you have issues with plugin compatibility.', 'lay').'</label>';
    }

	public static function misc_setting_disable_ajax(){

		if( class_exists( 'WooCommerce' ) ) {
			$checked = "checked";

			echo '<input disabled type="checkbox" name="misc_options_disable_ajax" id="misc_options_disable_ajax" '.$checked.'><br>
			<label for="misc_options_disable_ajax">'.__('When using WooCommerce, this setting will always be activated.<br>
			When activated page transitions and project overlays will be disabled. And when going to the frontpage the intro will always re-appear.').'
			</label>';
		} else {
			$val = get_option( 'misc_options_disable_ajax', "" );
			$checked = "";
			if( $val == "on" ){
				$checked = "checked";
			}
			echo '<input type="checkbox" name="misc_options_disable_ajax" id="misc_options_disable_ajax" '.$checked.'><br>
			<label for="misc_options_disable_ajax">'.__('Activate this setting if you have issues with plugin compatibility.<br>
			When activated there will be no page transitions. And when going to the frontpage the intro will always re-appear.').'
			</label>';
		}

	}

	public static function misc_setting_title_separator(){
		$val = get_option( 'misc_options_title_separator', '—' );
        $yoast_message = '';
        if ( is_plugin_active('wordpress-seo/wp-seo.php') ) {
			$yoast_message = 
            '<div class="lay-infobox lay-cyan-border">
                <p>
                    <strong>'.__('Yoast SEO Plugin Warning', 'lay').'</strong><br>'.__('Yoast SEO Plugin Warning', 'lay').'
                </p>
            </div>';
		}
		echo $yoast_message.'<input type="text" name="misc_options_title_separator" id="misc_options_title_separator" value="'.$val.'"><br>
		'.__('Separator for the page titles that are shown in your browser tab. Example: "My Website – My Page". Default is "–".', 'lay');
	}

	public static function misc_setting_hr_height(){
		$val = get_option( 'misc_options_hr_height', '1' );
		echo '<input type="number" min="1" step="1" name="misc_options_hr_height" id="misc_options_hr_height" value="'.$val.'"/> px';
	}

	public static function misc_setting_hr_color(){
		$val = get_option( 'misc_options_hr_color', '#000000' );
		echo '<input type="text" name="misc_options_hr_color" id="misc_options_hr_color" value="'.$val.'" class="lay-hr-color-picker"><br>
			'.__('Find out how to have a different line color for different pages with', 'lay').' <a href="https://laytheme.com/documentation/custom-css-styling.html#css-based-on-current-page" target="_blank">'.__('CSS based on current page.', 'lay').'</a>.<br>
			<code>body.slug-yourslug .lay-hr{ background-color: #f0f; }</code>';
	}

    public static function misc_setting_vl_width(){
		$val = get_option( 'misc_options_vl_width', '1' );
		echo '<input type="number" min="1" step="1" name="misc_options_vl_width" id="misc_options_vl_width" value="'.$val.'"/> px';
	}

	public static function misc_setting_vl_color(){
		$val = get_option( 'misc_options_vl_color', '#000000' );
		echo '<input type="text" name="misc_options_vl_color" id="misc_options_vl_color" value="'.$val.'" class="lay-vl-color-picker"><br>
			'.__('Find out how to have a different line color for different pages with', 'lay').' <a href="https://laytheme.com/documentation/custom-css-styling.html#css-based-on-current-page" target="_blank">'.__('CSS based on current page.', 'lay').'</a>.<br>
			<code>body.slug-yourslug .lay-vl{ background-color: #f0f; }</code>';
	}

    public static function textformats_lineheight_in_px_setting(){
        $val = get_option( 'misc_options_textformats_lineheight_in_px', "" );
        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input type="checkbox" name="misc_options_textformats_lineheight_in_px" id="misc_options_textformats_lineheight_in_px" '.$checked.'><br>
        <label for="misc_options_textformats_lineheight_in_px">'.__('Be able to set desktop, tablet and phone line heights independently and in px or em.', 'lay').'</label>';
    }

	public static function textformats_for_tablet_setting(){
		$val = get_option( 'misc_options_textformats_for_tablet', "on" );
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_textformats_for_tablet" id="misc_options_textformats_for_tablet" '.$checked.'>';
	}

	public static function textformats_tablet_breakpoint_setting(){
		$breakpoint = get_option( 'misc_options_textformats_tablet_breakpoint', 1024 );
		echo
		'<input type="number" min="0" step="1" value="'.$breakpoint.'" name="misc_options_textformats_tablet_breakpoint" id="misc_options_textformats_tablet_breakpoint"> px
		<br><label for="misc_options_textformats_tablet_breakpoint">'.__('Breakpoint between desktop and tablet version. Needs to be bigger than "Phone Breakpoint".', 'lay').'</label>';
	}

    public function misc_setting_hide_current_project_from_thumbnailgrid(){
        $val = get_option('misc_options_hide_current_project_from_thumbnailgrid', 'on');
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_hide_current_project_from_thumbnailgrid" id="misc_options_hide_current_project_from_thumbnailgrid" '.$checked.'>';
    }

	public function misc_setting_show_password_protected_posts_in_thumbnailgrid(){
		$val = get_option('misc_options_show_password_protected_posts_in_thumbnailgrid', 'on');
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_show_password_protected_posts_in_thumbnailgrid" id="misc_options_show_password_protected_posts_in_thumbnailgrid" '.$checked.'>';
	}

    public function misc_setting_showoriginalimages_for_phone(){
		$val = get_option('misc_options_showoriginalimages_for_phone', '');
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_showoriginalimages_for_phone" id="misc_options_showoriginalimages_for_phone" '.$checked.'><label for="misc_options_showoriginalimages_for_phone">'.__('Activate if your images seem blurry on phone', 'lay').'</label>';
	}

	public function misc_setting_showoriginalimages(){
		$val = get_option('misc_options_showoriginalimages', '');
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_showoriginalimages" id="misc_options_showoriginalimages" '.$checked.'><label for="misc_options_showoriginalimages">'.__('Normally, when appropriate, Lay Theme uses automatically generated smaller image sizes to speed up loading.', 'lay').'<br>
			'.__('This setting does not apply to the phone version. Activate this if your images seem blurry.', 'lay').'</label>';
	}

	public function misc_setting_max_width_apply_to_logo_and_nav(){
		$val = get_option('misc_options_max_width_apply_to_logo_and_nav', '');
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_max_width_apply_to_logo_and_nav" id="misc_options_max_width_apply_to_logo_and_nav" '.$checked.'><br>
		<label for="misc_options_max_width_apply_to_logo_and_nav">'.__('Logo and menu will only align to grids that use the same "Frame Left, Right" value as in "Lay Options" &rarr; "Gridder Defaults".', 'lay').'</label>';
	}

	public function misc_setting_prevnext_navigate_through(){
		$val = get_option('misc_options_prevnext_navigate_through', 'all_projects');

		$checked1 = $val == 'same_category' ? 'checked' : '';
		$checked2 = $val == 'all_projects' ? 'checked' : '';

		echo '<input type="radio" name="misc_options_prevnext_navigate_through" id="same_category" value="same_category" '.$checked1.'><label for="same_category">'.__('Projects of same Category', 'lay').'</label><br>';
		echo '<input type="radio" name="misc_options_prevnext_navigate_through" id="all_projects" value="all_projects" '.$checked2.'><label for="all_projects">'.__('All Projects', 'lay').'</label>';
	}

	public function misc_setting_image_loading(){
        /* 
		i changed the misc option inputs from radio to checkbox
		this used to be either "instant_load" or "lazy_load"
		but now it can be '' or 'on'
		'on' is when lazyload is turned on, '' is when lazyload is off
		i need to make sure in my code the values 'instant_load' and 'lazy_load' are handled too
		*/
		$val = get_option('misc_options_image_loading', 'on');

		$checked = "";
		if( $val == "on" || $val == "lazy_load" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_image_loading" id="misc_options_image_loading" '.$checked.'>
		<br><label for="misc_options_image_loading">'.__('When lazy loading is on, images will be loaded when you scroll to them.', 'lay').'<br>
		'.__('Recommended for when you have many images on a page.', 'lay').'</label>';
	}


    public function misc_setting_videos_lazyload(){
        $val = get_option('misc_options_videos_lazyload', '');

        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input type="checkbox" name="misc_options_videos_lazyload" id="misc_options_videos_lazyload" '.$checked.'>';
    }

    public function misc_setting_image_loading_blur(){
        $val = get_option('misc_options_image_loading_blur', '');

        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input type="checkbox" name="misc_options_image_loading_blur" id="misc_options_image_loading_blur" '.$checked.'>
        <br><label for="misc_options_image_loading_blur">'.__('When lazy loading is on, images will be loaded with a blur effect.', 'lay').'</label>';
    }

	public function misc_setting_image_placeholder_color(){
		$val = get_option( 'misc_options_image_placeholder_color', '' );
		echo '<input type="text" name="misc_options_image_placeholder_color" id="misc_options_image_placeholder_color" value="'.$val.'" class="image-placeholder-color-picker"><br><label>'.__('When an image has not loaded yet, a placeholder rectangle of this color will be visible.', 'lay').'</label>';
	}

	public function misc_setting_lay_breakpoint(){
		$breakpoint = get_option('lay_breakpoint', 600);
		echo '<input type="number" min="0" step="1" name="lay_breakpoint" id="lay_breakpoint" value="'.$breakpoint.'"> <label for="lay_breakpoint"> px (0 = disable phone version)<br>
		'.__('Below this browser-width the phone version is shown.', 'lay').'</label>';
	}

	public function misc_setting_lay_tablet_breakpoint(){
		$breakpoint = get_option('lay_tablet_breakpoint', 1024);
		echo '<input type="number" min="0" step="1" name="lay_tablet_breakpoint" id="lay_tablet_breakpoint" value="'.$breakpoint.'"> <label for="lay_breakpoint"> px<br>
		'.__('Must be bigger than phone breakpoint. Important for elements like "Thumbnailgrid", "Elementgrid" and "Project Index".', 'lay').'</label>';		
	}

	public function misc_setting_max_width(){
		$maxwidth = get_option( 'misc_options_max_width', '0' );
		echo '<input type="number" min="0" step="1" name="misc_options_max_width" id="misc_options_max_width" value="'.$maxwidth.'"> <label for="misc_options_max_width"> px (0 = '.__('off', 'lay').')</label>';
	}

	public function misc_setting_image_quality(){
		$quality = get_option( 'misc_options_image_quality', '90' );
		echo '<input type="number" min="0" max="100" step="1" name="misc_options_image_quality" id="misc_options_image_quality" value="'.$quality.'"> <label for="misc_options_image_quality"> % <br>'.__('When you change the quality you need to regenerate all images with a plugin like', 'lay').' <a href="https://wordpress.org/plugins/regenerate-thumbnails/" target="_blank">'.__('Regenerate Thumbnails', 'lay').'</a>.<br>'.__('In some cases the original uploaded image will be shown on your website. The original image is not affected by the quality set here.', 'lay').'</label>';
	}

    public function misc_setting_enable_frame_overflows_for_apl(){
        $val = get_option('misc_options_enable_frame_overflows_for_apl', 'on');

		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_enable_frame_overflows_for_apl" id="misc_options_enable_frame_overflows_for_apl" '.$checked.'>
		<br><label for="misc_options_enable_frame_overflows_for_apl">'.__('If enabled, stretch element to edge on phone layouts too.', 'lay').' <a href="https://laytheme.com/documentation/gridder-elements.html#full-width-element">'.__('Like here', 'lay').'</a></label>';
    }

		public function misc_setting_enable_video_lazyloading_for_safari(){
				$val = get_option('misc_options_enable_video_lazyloading_for_safari', '');
				$checked = "";
				if( $val == "on" ){
					$checked = "checked";
				}
				echo '<input type="checkbox" name="misc_options_enable_video_lazyloading_for_safari" id="misc_options_enable_video_lazyloading_for_safari" '.$checked.'>
				<br><label for="misc_options_enable_video_lazyloading_for_safari">'.__('If enabled, videos might not show up on Safari upon navigating on the website.', 'lay').'</label>';
		} 

    public function misc_setting_enable_video_lazyloading_for_firefox(){
        $val = get_option('misc_options_enable_video_lazyloading_for_firefox', '');

		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_enable_video_lazyloading_for_firefox" id="misc_options_enable_video_lazyloading_for_firefox" '.$checked.'>
		<br><label for="misc_options_enable_video_lazyloading_for_firefox">'.__('If enabled, videos might not show up on Firefox upon navigating on the website.', 'lay').'</label>';
    }    

    public function misc_setting_swap_webfonts(){
        $val = get_option('misc_options_swap_webfonts', 'on');

		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_swap_webfonts" id="misc_options_swap_webfonts" '.$checked.'>
		<br><label for="misc_options_swap_webfonts">'.__('If enabled, show text even when webfont hasn\'t loaded.', 'lay').'</label>';
    }

    public function misc_setting_never_change_scrolling_position(){
        $val = get_option('misc_options_never_change_scrolling_position', '');

		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="misc_options_never_change_scrolling_position" id="misc_options_never_change_scrolling_position" '.$checked.'>
		<br><label for="misc_options_never_change_scrolling_position">'.__('This option only makes sense for', 'lay').' <a target="_blank" href="https://laytheme.com/tutorials/create-a-splitscreen-website.html">'.__('split-screen websites', 'lay').'</a> '.__('and is active for desktop/tablet only.', 'lay').'</label>';
    }

    public function misc_setting_use_revealing_transition_on_first_visit(){
        $val = get_option('misc_options_use_revealing_transition_on_first_visit', '');
        if( MiscOptions::$locked ) {
            $val = 'on';
        }
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input '.MiscOptions::$disabled_input.' type="checkbox" name="misc_options_use_revealing_transition_on_first_visit" id="misc_options_use_revealing_transition_on_first_visit" '.$checked.'> '.MiscOptions::$unlock_notice.' 
        <label for="misc_options_use_revealing_transition_on_first_visit">('.__('will also fade in menu, site title etc on first page load', 'lay').').
        <br/>Please note, enabling this might lead to perceived longer loading times or in case of errors occuring your content won\'t show at all.</label>';
    }

    public function misc_setting_navigation_transition_out(){
        $val = get_option( 'misc_options_navigation_transition_out', 'fade' );

        if( MiscOptions::$locked || $val == '' ) {
            $val = 'fade';
        }
        $selected1 = $val == 'fade' ? 'selected' : '';
        $selected2 = $val == 'stagger-fade' ? 'selected' : '';
        $selected3 = $val == 'stagger-up' ? 'selected' : '';
        $selected4 = $val == 'stagger-down' ? 'selected' : '';
        $selected5 = $val == 'up' ? 'selected' : '';
        $selected6 = $val == 'down' ? 'selected' : '';
        $selected7 = $val == 'none' ? 'selected' : '';

        echo 
        '<select '.MiscOptions::$disabled_input.' name="misc_options_navigation_transition_out" id="misc_options_navigation_transition_out">
            <option value="fade" '.$selected1.'>'.__('Fade', 'lay').'</option> 
            <option value="stagger-fade" '.$selected2.'>'.__('Stagger Fade', 'lay').'</option>
            <option value="stagger-up" '.$selected3.'>'.__('Stagger Up', 'lay').'</option>
            <option value="stagger-down" '.$selected4.'>'.__('Stagger Down', 'lay').'</option>
            <option value="up" '.$selected5.'>'.__('Up', 'lay').'</option>
            <option value="down" '.$selected6.'>'.__('Down', 'lay').'</option>
            <option value="none" '.$selected7.'>'.__('None', 'lay').'</option>
        </select> '.MiscOptions::$unlock_notice.' <label for="misc_options_navigation_transition_out" id="misc_options_navigation_transition_out"> '.__('When you leave a page', 'lay').'</label>';
    }

    public function misc_setting_navigation_transition_in_easing(){
        $val = get_option( 'misc_options_navigation_transition_in_easing', 'easeInOutQuart' );
        if( MiscOptions::$locked || $val == '' ) {
            $val = 'easeInOutQuart';
        }
		echo '<input '.MiscOptions::$disabled_input.' type="text" name="misc_options_navigation_transition_in_easing" id="misc_options_navigation_transition_in_easing" value="'.$val.'"> '.MiscOptions::$unlock_notice.' <span>'.__('Find easings', 'lay').' <a target="_blank" href="https://animejs.com/documentation/#pennerFunctions">'.__('here', 'lay').'</a>. '.__('Anything with "easeOut" works well here.', 'lay').'</span>';
    }

    public function misc_setting_navigation_transition_out_easing(){
        $val = get_option( 'misc_options_navigation_transition_out_easing', 'easeInOutQuart' );
        if( MiscOptions::$locked || $val == '' ) {
            $val = 'easeInOutQuart';
        }
        echo '<input '.MiscOptions::$disabled_input.' type="text" name="misc_options_navigation_transition_out_easing" id="misc_options_navigation_transition_out_easing" value="'.$val.'"> '.MiscOptions::$unlock_notice.' <span>'.__('Find easings', 'lay').' <a target="_blank" href="https://animejs.com/documentation/#pennerFunctions">'.__('here', 'lay').'</a>. '.__('Anything with "easeIn" works well here.', 'lay').'</span>';
    }

    public function misc_setting_navigation_transition_y_translate_desktop(){
        $val = get_option( 'misc_options_navigation_transition_y_translate_desktop', 100 );
        if( MiscOptions::$locked || $val == '' ) {
            $val = 100;
        }
		echo '<input '.MiscOptions::$disabled_input.' type="number" min="10" name="misc_options_navigation_transition_y_translate_desktop" id="misc_options_navigation_transition_y_translate_desktop" value="'.$val.'"> px '.MiscOptions::$unlock_notice.' '.__('(Amount of pixel to move for up and down transitions)', 'lay');
    }

    public function misc_setting_navigation_transition_y_translate_phone(){
        $val = get_option( 'misc_options_navigation_transition_y_translate_phone', 50 );
        if( MiscOptions::$locked || $val == '' ) {
            $val = 50;
        }
        echo '<input '.MiscOptions::$disabled_input.' type="number" min="10" name="misc_options_navigation_transition_y_translate_phone" id="misc_options_navigation_transition_y_translate_phone" value="'.$val.'"> px '.MiscOptions::$unlock_notice.' '.__('(Amount of pixel to move for up and down transitions)', 'lay');
    }

    public function misc_setting_navigation_transition_in(){
        $val = get_option( 'misc_options_navigation_transition_in', 'fade' );

        if( MiscOptions::$locked || $val == '' ) {
            $val = 'fade';
        }
        $selected1 = $val == 'fade' ? 'selected' : '';
        $selected2 = $val == 'stagger-fade' ? 'selected' : '';
        $selected3 = $val == 'stagger-up' ? 'selected' : '';
        $selected4 = $val == 'stagger-down' ? 'selected' : '';
        $selected5 = $val == 'up' ? 'selected' : '';
        $selected6 = $val == 'down' ? 'selected' : '';
        $selected7 = $val == 'none' ? 'selected' : '';

        echo 
        '<select '.MiscOptions::$disabled_input.' name="misc_options_navigation_transition_in" id="misc_options_navigation_transition_in">
            <option value="fade" '.$selected1.'>'.__('Fade', 'lay').'</option> 
            <option value="stagger-fade" '.$selected2.'>'.__('Stagger Fade', 'lay').'</option>
            <option value="stagger-up" '.$selected3.'>'.__('Stagger Up', 'lay').'</option>
            <option value="stagger-down" '.$selected4.'>'.__('Stagger Down', 'lay').'</option>
            <option value="up" '.$selected5.'>'.__('Up', 'lay').'</option>
            <option value="down" '.$selected6.'>'.__('Down', 'lay').'</option>
            <option value="none" '.$selected7.'>'.__('None', 'lay').'</option>
        </select> '.MiscOptions::$unlock_notice.' <label for="misc_options_navigation_transition_in"> '.__('When you arrive on a page', 'lay').'</label>';
    }

	public function misc_setting_navigation_transition_in_duration(){
		$default = get_option( 'misc_options_navigation_transition_duration', '0.3' );
        $default = (float)$default;
        $default /= 2;
        $default *= 1000;
        $val = get_option( 'misc_options_navigation_transition_in_duration', $default );
        if( MiscOptions::$locked || $val == '' ) {
            $val = 300;
        }
		echo '<input '.MiscOptions::$disabled_input.' type="number" min="0" step="1" name="misc_options_navigation_transition_in_duration" id="misc_options_navigation_transition_in_duration" value="'.$val.'"> '.MiscOptions::$unlock_notice.' <label for="misc_options_navigation_transition_in_duration"> '.__('milliseconds', 'lay').'</label>';
	}

	public function misc_setting_navigation_transition_out_duration(){
		$default = get_option( 'misc_options_navigation_transition_duration', '0.3' );
        $default = (float)$default;
        $default /= 2;
        $default *= 1000;
        $val = get_option( 'misc_options_navigation_transition_out_duration', $default );
        if( MiscOptions::$locked || $val == '' ) {
            $val = 300;
        }
		echo '<input '.MiscOptions::$disabled_input.' type="number" min="0" step="1" name="misc_options_navigation_transition_out_duration" id="misc_options_navigation_transition_out_duration" value="'.$val.'"> '.MiscOptions::$unlock_notice.' <label for="misc_options_navigation_transition_out_duration"> '.__('milliseconds', 'lay').'</label>';
	}

    // scroll transitions:
    public function misc_setting_scroll_transition_duration(){
		$val = get_option( 'misc_options_scroll_transition_duration', '300' );
        $val = (float)$val;
        if( MiscOptions::$locked || $val == '' ) {
            $val = 300;
        }
		echo '<input '.MiscOptions::$disabled_input.' type="number" min="0" step="1" name="misc_options_scroll_transition_duration" id="misc_options_scroll_transition_duration" value="'.$val.'"> '.MiscOptions::$unlock_notice.' <label for="misc_options_scroll_transition_duration"> '.__('milliseconds', 'lay').'</label>';
	}

    public function misc_setting_scroll_transition_easing(){
        $val = get_option( 'misc_options_scroll_transition_easing', 'ease' );
        if( MiscOptions::$locked || $val == '' ) {
            $val = 'ease';
        }
        echo '<input '.MiscOptions::$disabled_input.' type="text" name="misc_options_scroll_transition_easing" id="misc_options_scroll_transition_easing" value="'.$val.'"> '.MiscOptions::$unlock_notice.' <span>'.__('Find easings', 'lay').' <a target="_blank" href="https://developer.mozilla.org/en-US/docs/Web/CSS/easing-function">'.__('here', 'lay').'</a>. '.__('Use for example: "ease", "ease-out", "ease-in", "ease-in-out" or "cubic-bezier(0.25, 0.1, 0.25, 1)". If you use a wrong value, the animation will stop working.', 'lay').'</span>';
    }

    public function misc_setting_scroll_transition_offset_desktop(){
        $val = get_option( 'misc_options_scroll_transition_offset_desktop', 90 );
        if( MiscOptions::$locked || $val == '' ) {
            $val = 0;
        }
        echo '<input '.MiscOptions::$disabled_input.' type="number" min="0" step="1" name="misc_options_scroll_transition_offset_desktop" id="misc_options_scroll_transition_offset_desktop" value="'.$val.'"> px '.MiscOptions::$unlock_notice.' '.__('(The amount of pixels elements move up)', 'lay');
    }

    public function misc_setting_scroll_transition_offset_mobile(){
        $val = get_option( 'misc_options_scroll_transition_offset_mobile', 40 );
        if( MiscOptions::$locked || $val == '' ) {
            $val = 0;
        }
        echo '<input '.MiscOptions::$disabled_input.' type="number" min="0" step="1" name="misc_options_scroll_transition_offset_mobile" id="misc_options_scroll_transition_offset_mobile" value="'.$val.'"> px '.MiscOptions::$unlock_notice.' '.__('(The amount of pixels elements move up)', 'lay');
    }

	// public function misc_setting_navigation_transition_duration(){
	// 	$value = get_option( 'misc_options_navigation_transition_duration', '0.3' );
	// 	echo '<input type="number" min="0" step="0.1" name="misc_options_navigation_transition_duration" id="misc_options_navigation_transition_duration" value="'.$value.'"> <label for="misc_options_navigation_transition_duration"> seconds <br>When you click a menu point the content fades out and the new content fades in. This is the duration of that animation.</label>';
	// }

	public function misc_setting_website_description(){
		$description = get_option( 'misc_options_website_description', '' );
		echo
		'<textarea style="max-width: 700px; width: 100%; height: 150px;" name="misc_options_website_description" id="misc_options_website_description">'.$description.'</textarea><br>
		<p>'.__('The Website description will show up on Google and when someone shares your website on Facebook or Twitter.', 'lay').'</p>';
	}

    public function misc_setting_image_alt_tag_to_use(){
        // selectbox to choose between: "Alternative Text", "Title", "Caption", "Description"

        $val = get_option( 'misc_options_image_alt_tag_to_use', 'alt' );
        $selected1 = $val == 'alt' ? 'selected' : '';
        $selected2 = $val == 'title' ? 'selected' : '';
        $selected3 = $val == 'caption' ? 'selected' : '';
        $selected4 = $val == 'description' ? 'selected' : '';

        echo
        '<select name="misc_options_image_alt_tag_to_use" id="misc_options_image_alt_tag_to_use">
            <option value="alt" '.$selected1.'>'.__('Alternative Text', 'lay').'</option> 
            <option value="title" '.$selected2.'>'.__('Title', 'lay').'</option>
            <option value="caption" '.$selected3.'>'.__('Caption', 'lay').'</option>
            <option value="description" '.$selected4.'>'.__('Description', 'lay').'</option>
        </select> <br>
        <label for="misc_options_image_alt_tag_to_use"> '.__('In the media library, files have text fields like: "Alternative Text", "Title", "Caption", "Description".<br>
        Choose which field to use for your alt tag.', 'lay').'</label>';
    }

	public function misc_setting_image_alt_tag(){
		$image_alt_tag = get_option( 'misc_options_image_alt_tag', '' );
		echo '<input type="text" name="misc_options_image_alt_tag" id="misc_options_image_alt_tag" value="'.$image_alt_tag.'"><br>
		<label for="misc_options_image_alt_tag"> '.__('This text will be added to all image alt tags. If an image has an Alt text set already, this will just be added to it.<br>
		Alt Tags are important for Search Engines. Here you could write your name or company\'s name or site title.', 'lay').'</label>';
	}

	public function misc_setting_fbimage(){
		// https://gist.github.com/hlashbrooke/9267467#file-class-php-L324
		$image_thumb_id = get_option( 'misc_options_fbimage', '' );
		$image_thumb = Setup::$uri.'/options/assets/img/nofile_small.svg';
		$noimage_image = $image_thumb;
		$hideRemoveButtonCSS = '';
		if($image_thumb_id != ''){
			$image_thumb = wp_get_attachment_image_src( $image_thumb_id, 'full' );
			$image_thumb = $image_thumb[0];
		}
		else{
			$hideRemoveButtonCSS = 'style="display:none;"';
		}
		$option_name = 'misc_options_fbimage';
		echo
		'<img id="'.$option_name.'_preview" style="max-width: 200px;" class="image_preview" data-noimagesrc="'.$noimage_image.'" src="'.$image_thumb.'">
		<p style="margin-bottom: 10px;">'.__('This image will show up when someone posts your website on Facebook or Twitter.', 'lay').'<br>
		'.__('This should be a .jpg with 1200px width and 630px height to look good on Facebook.', 'lay').'<br>
		'.__('To let Facebook know you are using a new image go', 'lay').' <a href="https://developers.facebook.com/tools/debug/" target="_blank">'.__('here', 'lay').'</a>'.__(', paste your website address, and click "Fetch new scrape information"', 'lay').'.</p>
		<input id="'.$option_name.'_button" style="margin-bottom: 5px;" type="button" class="image_upload_button button" value="'.__('Set image', 'lay').'" /><br>
		<input id="'.$option_name.'_delete" '.$hideRemoveButtonCSS.' type="button" class="image_delete_button button" value="'.__('Remove image', 'lay').'" /><br>
		<input id="'.$option_name.'" class="image_data_field" type="hidden" name="'.$option_name.'" value="'.$image_thumb_id.'"/>';
	}

    public function misc_setting_colorpalette(){
        $colorpalette = get_option( 'misc_options_colorpalette', '#000, #fff, #f00, #0f0, #00f, #ff0, #0ff, #f0f' );
        $colorpalette = Setup::turnTo6DigitHexString($colorpalette);

        echo '
        <div id="color-palette-chooser"></div>
        <input type="text" style="width:500px!important;display: none;" name="misc_options_colorpalette" id="misc_options_colorpalette" value="'.$colorpalette.'">';
    }

	public function misc_setting_html5video_playicon(){
		// https://gist.github.com/hlashbrooke/9267467#file-class-php-L324
		$image_thumb_id = get_option( 'misc_options_html5video_playicon', '' );
		$image_thumb = Setup::$uri.'/frontend/assets/img/play.svg';
		$noimage_image = $image_thumb;
		$hideRemoveButtonCSS = '';
		if($image_thumb_id != ''){
			$image_thumb = wp_get_attachment_image_src( $image_thumb_id, 'full' );
			$image_thumb = $image_thumb[0];
		}
		else{
			$hideRemoveButtonCSS = 'style="display:none;"';
		}
		$option_name = 'misc_options_html5video_playicon';
		echo
		'<img id="'.$option_name.'_preview" style="max-width: 100%;" class="image_preview" data-noimagesrc="'.$noimage_image.'" src="'.$image_thumb.'"><br>
		<input id="'.$option_name.'_button" style="margin-bottom: 5px;" type="button" class="image_upload_button button" value="'.__('Set image', 'lay').'" /><br>
		<input id="'.$option_name.'_delete" '.$hideRemoveButtonCSS.' type="button" class="image_delete_button button" value="'.__('Set to default', 'lay').'" /><br>
		<input id="'.$option_name.'" class="image_data_field" type="hidden" name="'.$option_name.'" value="'.$image_thumb_id.'"/>';
	}

}
new MiscOptions();
