<?php
// http://codex.wordpress.org/Settings_API#Examples
class ProjectsOverlaysOptions {

    public static $bg_color;

	public function __construct(){

        $bg_color = get_option( 'po_background_color', '#F5F5F5' );
        if($bg_color == ''){
            $bg_color = '#fff';
        }
        ProjectsOverlaysOptions::$bg_color = $bg_color;

		$val = get_option('misc_options_activate_projects_overlay_feature', '');
        if( class_exists( 'WooCommerce' ) ) {
            $val = '';
        }
        if(get_option( 'misc_options_disable_ajax', "" ) == "on"){
            $val = '';
        }
		if($val == "on" && !MiscOptions::$locked){
			add_action( 'admin_menu', array($this, 'projectsoverlays_options_setup_menu'), 10 );
			add_action( 'admin_init', array($this, 'misc_options_settings_api_init') );
            add_action( 'admin_enqueue_scripts', array($this, 'enqueue_scripts_for_po') );
            add_action( 'wp_enqueue_scripts', array($this, 'frontend_enqueue_scripts_for_po') );
		}
	}

    public static function getMarkup(){
        $val = get_option('po_background_behind_overlay', 'on');
        $animation = get_option('po_animation', 'slideup');
        if( $animation != 'slideup' && $animation != 'slidedown' ){
            $val = '';
        }
        $bg = '';
        if($val == 'on') {
            $bg = '<div id="lay-project-overlay-background" style="opacity:0;"></div>';
        }
        return $bg.'<div id="lay-project-overlay" style="'.ProjectsOverlaysOptions::getHidingCSS().'" class="animation-'.get_option('po_animation', 'slideup').'"></div>';
    }

    // for animejs to work
    public static function getHidingCSS(){
        $animation = get_option('po_animation', 'slideup');
        switch($animation) {
            case 'slideup':
                return 'transform: translateY(100%);';
            break;
            case 'slidedown':
                return 'transform: translateY(-100%);';
            break;
            case 'fadein':
                return 'opacity: 0;';
            break;
        }
        return '';
    }

    public function frontend_enqueue_scripts_for_po(){
        
        $breakpoint = get_option('lay_breakpoint', 600);
        $breakpoint = (int)$breakpoint;

        // when opening behaviour is "when loaded", we only set the bg color of a grid and cover etc with the css that is loaded with the grid markup itself
        // there, i needed to handle different edge cases like: cpl, cover, no cover, no cpl, bg color was not set and we use this bg color here, etc
        // but for when loading behaviour is "immediate", we need to set the bg color of the overlay itself here, because the overlay shows up first, then the grid markup with the bg color is loaded
        $po_opening_behaviour = get_option('po_opening_behaviour', 'immediate');
        $css_for_immediate_opening = '';
        if($po_opening_behaviour == 'immediate') {
            $css_for_immediate_opening = '#lay-project-overlay{ background-color: '.ProjectsOverlaysOptions::$bg_color.'; }';
        }

        $bg_opacity = get_option('po_background_behind_overlay_opacity', 50);

        wp_add_inline_style('frontend-style', '
            '.$css_for_immediate_opening.'
            .close-projects-overlay{
                color: '.get_option('po_close_button_color', '#000000').';
            }
            '.CSS_Output::generate_rgba_from_option('#lay-project-overlay-background', 'background-color', 'po_background_behind_overlay_color', '#000000', $bg_opacity).'
            @media (max-width: '.($breakpoint).'px){
                .close-projects-overlay.standard-close::before{
                    font-size: '.get_option('po_close_icon_size_phone', '15').(get_option('po_close_icon_size_phone_in', 'px') == 'px' ? 'px' : 'vw').';
                }
                .custom-close img{
                    width: '.get_option('po_close_icon_size_phone', '15').(get_option('po_close_icon_size_phone_in', 'px') == 'px' ? 'px' : 'vw').';
                }
                .close-projects-overlay{
                    '.(get_option('po_close_icon_position_phone', 'topleft') == 'topright' ? 'right' : 'left').':'.get_option('po_close_icon_spaceright_phone', '13').(get_option('po_close_icon_spaceright_phone_in', 'px') == 'px' ? 'px' : 'vw').';
                    top: '.get_option('po_close_icon_spacetop_phone', '13').(get_option('po_close_icon_spacetop_phone_in', 'px') == 'px' ? 'px' : 'vw').';
                }
            }
            @media (min-width: '.($breakpoint+1).'px){
                .close-projects-overlay.standard-close::before{
                    font-size: '.get_option('po_close_icon_size_desktop', '26').(get_option('po_close_icon_size_desktop_in', 'px') == 'px' ? 'px' : 'vw').';
                }
                .custom-close img{
                    width: '.get_option('po_close_icon_size_desktop', '26').(get_option('po_close_icon_size_desktop_in', 'px') == 'px' ? 'px' : 'vw').';
                }
                .close-projects-overlay{
                    top: 0;
                    '.(get_option('po_close_icon_position_desktop', 'topright') == 'topright' ? 'right' : 'left').': 0;
                    padding-'.(get_option('po_close_icon_position_desktop', 'topright') == 'topright' ? 'right' : 'left').': '.get_option('po_close_icon_spaceright_desktop', '20').(get_option('po_close_icon_spaceright_desktop_in', 'px') == 'px' ? 'px' : 'vw').';
                    padding-top: '.get_option('po_close_icon_spacetop_desktop', '20').(get_option('po_close_icon_spacetop_desktop_in', 'px') == 'px' ? 'px' : 'vw').';
                }
            }
        ');
    }

    public function enqueue_scripts_for_po($hook){
        if( strpos($hook, 'manage-projectsoverlaysoptions') !== false ){
			wp_enqueue_media();
			wp_enqueue_style( 'wp-color-picker' );
	        wp_enqueue_script( 'po_options-colorpicker_controller', Setup::$uri.'/options/assets/js/po_colorpicker_controller.js', array( 'wp-color-picker' ), Setup::$ver, true );
            wp_enqueue_style( 'po_options-ui', Setup::$uri.'/options/assets/css/po_ui.css', array(), Setup::$ver );
			wp_enqueue_script( 'po-image-upload', Setup::$uri.'/options/assets/js/image_upload.js', array('jquery'), Setup::$ver, true );
            wp_enqueue_script( 'po-showhide-settings', Setup::$uri.'/options/assets/js/projectsoverlays_options_showhide_settings.js', array('jquery'), Setup::$ver, true );
		}
	}

	public function misc_options_settings_api_init(){
	 	add_settings_section(
			'po_options_section',
			'',
			'',
			'manage-projectsoverlaysoptions'
		);
		
	 	add_settings_field(
			'po_show_close_icon',
			'Show "X" Close-Icon',
			array($this, 'po_show_close_icon_setting'),
			'manage-projectsoverlaysoptions',
			'po_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_show_close_icon' );

        add_settings_field(
            'po_close_button_color',
            '"X" Close-Icon Color',
            array($this, 'po_close_button_color_setting'),
            'manage-projectsoverlaysoptions',
            'po_options_section'
        );
        register_setting( 'manage-projectsoverlaysoptions', 'po_close_button_color' );

        add_settings_field(
            'po_use_custom_close_button',
            'Use Custom "X" Close-Icon',
            array($this, 'po_use_custom_close_button_setting'),
            'manage-projectsoverlaysoptions',
            'po_options_section'
        );
        register_setting( 'manage-projectsoverlaysoptions', 'po_use_custom_close_button' );

        add_settings_field(
            'po_close_icon',
            'Custom "X" Close-Icon',
            array($this, 'po_close_icon_setting'),
            'manage-projectsoverlaysoptions',
            'po_options_section'
        );
        register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon' );

        add_settings_field(
			'po_animation',
			'Animation',
			array($this, 'po_animation_setting'),
			'manage-projectsoverlaysoptions',
			'po_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_animation' );

        add_settings_field(
			'po_animation_duration',
			'Animation Duration',
			array($this, 'po_animation_duration_setting'),
			'manage-projectsoverlaysoptions',
			'po_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_animation_duration' );

         add_settings_field(
			'po_animation_easing_function',
			'Animation Easing Function',
			array($this, 'po_animation_easing_function_setting'),
			'manage-projectsoverlaysoptions',
			'po_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_animation_easing_function' );

        add_settings_field(
            'po_background_color',
            'Default Overlay Background Color',
            array($this, 'po_background_color_setting'),
            'manage-projectsoverlaysoptions',
            'po_options_section'
        );
        register_setting( 'manage-projectsoverlaysoptions', 'po_background_color' );

        add_settings_field(
            'po_opening_behaviour',
            'When to open overlay after click',
            array($this, 'po_opening_behaviour_setting'),
            'manage-projectsoverlaysoptions',
            'po_options_section'
        );
        register_setting( 'manage-projectsoverlaysoptions', 'po_opening_behaviour' );

        add_settings_field(
            'po_background_behind_overlay',
            'Background behind overlay',
            array($this, 'po_background_behind_overlay_setting'),
            'manage-projectsoverlaysoptions',
            'po_options_section'
        );
        register_setting( 'manage-projectsoverlaysoptions', 'po_background_behind_overlay' );

        add_settings_field(
            'po_background_behind_overlay_color',
            'Background behind overlay color',
            array($this, 'po_background_behind_overlay_color_setting'),
            'manage-projectsoverlaysoptions',
            'po_options_section'
        );
        register_setting( 'manage-projectsoverlaysoptions', 'po_background_behind_overlay_color' );

        add_settings_field(
            'po_background_behind_overlay_opacity',
            'Background behind overlay opacity',
            array($this, 'po_background_behind_overlay_opacity_setting'),
            'manage-projectsoverlaysoptions',
            'po_options_section'
        );
        register_setting( 'manage-projectsoverlaysoptions', 'po_background_behind_overlay_opacity' );


        // section for size and position options
        add_settings_section(
			'po_size_and_position_options_section',
			'',
			'',
			'manage-projectsoverlaysoptions'
		);

        add_settings_field(
			'po_close_icon_position_desktop',
			'Close-Icon Position (Desktop)',
			array($this, 'po_close_icon_position_desktop_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_position_desktop' );

        add_settings_field(
			'po_close_icon_position_phone',
			'Close-Icon Position (Phone)',
			array($this, 'po_close_icon_position_phone_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_position_phone' );

        add_settings_field(
			'po_close_icon_size_desktop_in',
			'Close-Icon Size In (Desktop)',
			array($this, 'po_close_icon_size_desktop_in_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_size_desktop_in' );

        add_settings_field(
			'po_close_icon_size_desktop',
			'Close-Icon Size (Desktop)',
			array($this, 'po_close_icon_size_desktop_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_size_desktop' );

         add_settings_field(
			'po_close_icon_size_phone_in',
			'Close-Icon Size In (Phone)',
			array($this, 'po_close_icon_size_phone_in_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_size_phone_in' );

        add_settings_field(
			'po_close_icon_size_phone',
			'Close-Icon Size (Phone)',
			array($this, 'po_close_icon_size_phone_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_size_phone' );

        add_settings_field(
			'po_close_icon_spaceright_desktop_in',
			'Close-Icon Space-Right In (Desktop)',
			array($this, 'po_close_icon_spaceright_desktop_in_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_spaceright_desktop_in' );

        add_settings_field(
			'po_close_icon_spaceright_desktop',
			'Close-Icon Space-Right (Desktop)',
			array($this, 'po_close_icon_spaceright_desktop_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_spaceright_desktop' );

        add_settings_field(
			'po_close_icon_spacetop_desktop_in',
			'Close-Icon Space-Top In (Desktop)',
			array($this, 'po_close_icon_spacetop_desktop_in_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_spacetop_desktop_in' );

        add_settings_field(
			'po_close_icon_spacetop_desktop',
			'Close-Icon Space-Top (Desktop)',
			array($this, 'po_close_icon_spacetop_desktop_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_spacetop_desktop' );

        add_settings_field(
			'po_close_icon_spaceright_phone_in',
			'Close-Icon Space-Right In (Phone)',
			array($this, 'po_close_icon_spaceright_phone_in_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_spaceright_phone_in' );

        add_settings_field(
			'po_close_icon_spaceright_phone',
			'Close-Icon Space-Right (Phone)',
			array($this, 'po_close_icon_spaceright_phone_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_spaceright_phone' );

        add_settings_field(
			'po_close_icon_spacetop_phone_in',
			'Close-Icon Space-Top In (Phone)',
			array($this, 'po_close_icon_spacetop_phone_in_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_spacetop_phone_in' );

        add_settings_field(
			'po_close_icon_spacetop_phone',
			'Close-Icon Space-Top (Phone)',
			array($this, 'po_close_icon_spacetop_phone_setting'),
			'manage-projectsoverlaysoptions',
			'po_size_and_position_options_section'
		);
	 	register_setting( 'manage-projectsoverlaysoptions', 'po_close_icon_spacetop_phone' );

	}

    public function po_animation_duration_setting(){
        $val = get_option('po_animation_duration', '1200');
        echo '<input type="number" name="po_animation_duration" id="po_animation_duration" value="'.$val.'"> ms';
    }

    public function po_animation_easing_function_setting(){
        $val = get_option('po_animation_easing_function', 'cubic-bezier(0.075, 0.82, 0.165, 1)');
        echo '<input type="text" name="po_animation_easing_function" id="po_animation_easing_function" value="'.$val.'"><br/>
        <strong style="font-size:12px;">Wrong values here will break things and your links to projects won\'t work anymore!</strong><br/>
        <i style="font-size:12px;color:#848484;">Default:<br/>
        cubic-bezier(0.075, 0.82, 0.165, 1)<br/>
        Use for example "linear" or "easeInSine" or any from <a target="_blank" href="https://easings.net/">here</a>.<br/>
        Or use a cubic-bezier, for example from <a href="https://www.cssportal.com/css-cubic-bezier-generator/" target="_blank">here</a>.</i>';
    }

    public function po_animation_setting(){
        $val = get_option('po_animation', 'slideup');

		$selectedNone = '';
		$selectedSlideup = '';
		$selectedSlidedown = '';
		$selectedFadein = '';

		switch ($val) {
			case 'slideup':
				$selectedSlideup = 'selected';
			break;
			case 'slidedown':
				$selectedSlidedown = 'selected';
			break;
			case 'none':
				$selectedNone = 'selected';
			break;
			case 'fadein':
				$selectedFadein = 'selected';
			break;
		}

		echo 
		'<select style="width:auto;" name="po_animation" id="po_animation">
            <option value="slideup" '.$selectedSlideup.'>Slide up</option>
			<option value="slidedown" '.$selectedSlidedown.'>Slide down</option>
			<option value="fadein" '.$selectedFadein.'>Fade in</option>
            <option value="none" '.$selectedNone.'>None</option>
		</select>'; 
    }

    public function po_close_icon_size_desktop_setting(){
        $val = get_option('po_close_icon_size_desktop', '26');
        echo '<input type="number" name="po_close_icon_size_desktop" id="po_close_icon_size_desktop" value="'.$val.'"> px';
    }

    public function po_close_icon_size_phone_setting(){
        $val = get_option('po_close_icon_size_phone', '15');
        echo '<input type="number" name="po_close_icon_size_phone" id="po_close_icon_size_phone" value="'.$val.'"> px';
    }

    public function po_close_icon_spaceright_desktop_setting(){
        $val = get_option('po_close_icon_spaceright_desktop', '20');
        echo '<input type="number" name="po_close_icon_spaceright_desktop" id="po_close_icon_spaceright_desktop" value="'.$val.'">';
    }

    public function po_close_icon_spacetop_desktop_setting(){
        $val = get_option('po_close_icon_spacetop_desktop', '20');
        echo '<input type="number" name="po_close_icon_spacetop_desktop" id="po_close_icon_spacetop_desktop" value="'.$val.'">';
    }

    public function po_close_icon_spaceright_phone_setting(){
        $val = get_option('po_close_icon_spaceright_phone', '13');
        echo '<input type="number" name="po_close_icon_spaceright_phone" id="po_close_icon_spaceright_phone" value="'.$val.'">';
    }

    public function po_close_icon_spacetop_phone_setting(){
        $val = get_option('po_close_icon_spacetop_phone', '13');
        echo '<input type="number" name="po_close_icon_spacetop_phone" id="po_close_icon_spacetop_phone" value="'.$val.'">';
    }

    public function po_close_icon_size_phone_in_setting(){
        $val = get_option('po_close_icon_size_phone_in', 'px');
        echo '<select name="po_close_icon_size_phone_in" id="po_close_icon_size_phone_in">
            <option value="px" '.($val == 'px' ? 'selected' : '').'>px</option>
            <option value="%" '.($val == '%' ? 'selected' : '').'>%</option>
        </select>';
    }

    public function po_close_icon_size_desktop_in_setting(){
        $val = get_option('po_close_icon_size_desktop_in', 'px');
        echo '<select name="po_close_icon_size_desktop_in" id="po_close_icon_size_desktop_in">
            <option value="px" '.($val == 'px' ? 'selected' : '').'>px</option>
            <option value="%" '.($val == '%' ? 'selected' : '').'>%</option>
        </select>';
    }

    public function po_close_icon_spaceright_desktop_in_setting(){
        $val = get_option('po_close_icon_spaceright_desktop_in', 'px');
        echo '<select name="po_close_icon_spaceright_desktop_in" id="po_close_icon_spaceright_desktop_in">
            <option value="px" '.($val == 'px' ? 'selected' : '').'>px</option>
            <option value="%" '.($val == '%' ? 'selected' : '').'>%</option>
        </select>';
    }

    public function po_close_icon_spacetop_desktop_in_setting(){
        $val = get_option('po_close_icon_spacetop_desktop_in', 'px');
        echo '<select name="po_close_icon_spacetop_desktop_in" id="po_close_icon_spacetop_desktop_in">
            <option value="px" '.($val == 'px' ? 'selected' : '').'>px</option>
            <option value="%" '.($val == '%' ? 'selected' : '').'>%</option>
        </select>';
    }

    public function po_close_icon_spaceright_phone_in_setting(){
        $val = get_option('po_close_icon_spaceright_phone_in', 'px');
        echo '<select name="po_close_icon_spaceright_phone_in" id="po_close_icon_spaceright_phone_in">
            <option value="px" '.($val == 'px' ? 'selected' : '').'>px</option>
            <option value="%" '.($val == '%' ? 'selected' : '').'>%</option>
        </select>';
    }

    public function po_close_icon_spacetop_phone_in_setting(){
        $val = get_option('po_close_icon_spacetop_phone_in', 'px');
        echo '<select name="po_close_icon_spacetop_phone_in" id="po_close_icon_spacetop_phone_in">
            <option value="px" '.($val == 'px' ? 'selected' : '').'>px</option>
            <option value="%" '.($val == '%' ? 'selected' : '').'>%</option>
        </select>';
    }

    public function po_use_custom_close_button_setting(){
		$val = get_option( 'po_use_custom_close_button', "" );
		$checked = "";
		if($val == "on"){
			$checked = "checked";
		}
		echo
		'<input type="checkbox" name="po_use_custom_close_button" id="po_use_custom_close_button" '.$checked.'>';
	}

    public function po_close_icon_setting(){
		ProjectsOverlaysOptions::po_button_setting('po_close_icon');
	}

    public static function po_button_setting($option_name){
		// https://gist.github.com/hlashbrooke/9267467#file-class-php-L324
		$image_thumb_id = get_option( $option_name, '' );
		$image_thumb = "";
		$noimage_image = $image_thumb;
		$hideRemoveButtonCSS = '';
		if($image_thumb_id != ''){
			$image_thumb = wp_get_attachment_image_src( $image_thumb_id, 'full' );
			$image_thumb = $image_thumb[0];
		}
		else{
			$hideRemoveButtonCSS = 'style="display:none;"';
		}
		echo 
		'<img id="'.$option_name.'_preview" style="max-width: 50px;" class="image_preview" data-noimagesrc="'.$noimage_image.'" src="'.$image_thumb.'"><br>
		<input id="'.$option_name.'_button" style="margin-bottom: 5px;" type="button" class="image_upload_button button" value="Set image" /><br>
		<input id="'.$option_name.'_delete" '.$hideRemoveButtonCSS.' type="button" class="image_delete_button button" value="Remove image" /><br>
		<input id="'.$option_name.'" class="image_data_field" type="hidden" name="'.$option_name.'" value="'.$image_thumb_id.'"/>';
	}
    
    public function po_close_button_color_setting(){
		$val = get_option( 'po_close_button_color', '#000000' );
		echo '<input type="text" name="po_close_button_color" id="po_close_button_color" value="'.$val.'" class="po-color-picker">';
    }

    public function po_show_close_icon_setting(){
		$val = get_option('po_show_close_icon', 'on');
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="po_show_close_icon" id="po_show_close_icon" '.$checked.'>';
	}

    public function po_close_icon_position_desktop_setting(){
        $val = get_option('po_close_icon_position_desktop', 'topright');

        $selectedTopright = '';
        $selectedTopleft = '';

        switch ($val) {
            case 'topright':
                $selectedTopright = 'selected';
            break;
            case 'topleft':
                $selectedTopleft = 'selected';
            break;
        }

        echo
        '<select style="width:auto;" name="po_close_icon_position_desktop" id="po_close_icon_position_desktop">
            <option value="topright" '.$selectedTopright.'>Top Right</option>
            <option value="topleft" '.$selectedTopleft.'>Top Left</option>
        </select>';
    }

    public function po_close_icon_position_phone_setting(){
        $val = get_option('po_close_icon_position_phone', 'topleft');

        $selectedTopright = '';
        $selectedTopleft = '';

        switch ($val) {
            case 'topright':
                $selectedTopright = 'selected';
            break;
            case 'topleft':
                $selectedTopleft = 'selected';
            break;
        }

        echo
        '<select style="width:auto;" name="po_close_icon_position_phone" id="po_close_icon_position_phone">
            <option value="topright" '.$selectedTopright.'>Top Right</option>
            <option value="topleft" '.$selectedTopleft.'>Top Left</option>
        </select>';
    }

    public function po_background_color_setting(){
		$val = get_option( 'po_background_color', '#F5F5F5' );
		echo '<input type="text" name="po_background_color" id="po_background_color" value="'.$val.'" class="po-color-picker"><br/>
        <i style="font-size:12px;color:#848484;">Overwrite this color in a project in your Gridder &rarr; "Set Background"</i>';
	}

    public function po_background_behind_overlay_setting(){
        $val = get_option('po_background_behind_overlay', 'on');
        $checked = '';
        if($val == 'on'){
            $checked = 'checked';
        }
        echo '<input type="checkbox" name="po_background_behind_overlay" id="po_background_behind_overlay" '.$checked.'><br/>
        <i style="font-size:12px;color:#848484;">Only use for "slide up" and "slide down" animations. This is the background behind an overlay when its animating in.</i>';
    }

    public function po_background_behind_overlay_color_setting(){
        $val = get_option('po_background_behind_overlay_color', '#000000');
        echo '<input type="text" name="po_background_behind_overlay_color" id="po_background_behind_overlay_color" value="'.$val.'" class="po-color-picker">';
    }

    public function po_background_behind_overlay_opacity_setting(){
        $val = get_option('po_background_behind_overlay_opacity', 50);
        echo '<input type="number" name="po_background_behind_overlay_opacity" id="po_background_behind_overlay_opacity" value="'.$val.'"> (0-100)';
    }

    public function po_opening_behaviour_setting(){
        $val = get_option('po_opening_behaviour', 'immediate');

        $selectedloaded = '';
        $selectedimmediate = '';

        switch ($val) {
            case 'loaded':
                $selectedloaded = 'selected';
            break;
            case 'immediate':
                $selectedimmediate = 'selected';
            break;
        }

        echo
        '<select style="width:auto;" name="po_opening_behaviour" id="po_opening_behaviour">
            <option value="immediate" '.$selectedimmediate.'>immediately</option>
            <option value="loaded" '.$selectedloaded.'>when loaded</option>
        </select>
        <br/>
        <i style="font-size:12px;color:#848484;">Use "when loaded" if you have custom background colors set in your projects in Gridder &rarr; "Set Background"</i>
        ';
    }

	public function projectsoverlays_options_setup_menu(){
		// http://wordpress.stackexchange.com/questions/66498/add-menu-page-with-different-name-for-first-submenu-item
        add_submenu_page( 'manage-layoptions', 'Projects Overlays', 'Projects Overlays', 'manage_options', 'manage-projectsoverlaysoptions', array($this, 'projectsoverlays_options_markup') );
	}
	
	public function projectsoverlays_options_markup(){
		require_once( Setup::$dir.'/options/projectsoverlays_options_markup.php' );
	}

}
new ProjectsOverlaysOptions();