<?php

class LaySetFrontpage {
    public function __construct() {
        add_action('admin_menu', array($this, 'add_frontpage_settings_page'));
        add_action('admin_init', array($this, 'handle_frontpage_settings_save'));
    }

    public function add_frontpage_settings_page() {
        add_submenu_page(
            'options-general.php', // Parent slug (Settings)
            'Set Frontpage', // Page title
            'Set Frontpage', // Menu title
            'manage_options', // Capability
            'set-frontpage', // Menu slug
            array($this, 'render_frontpage_settings_page'), // Callback function
            1 // Position (1 means second place)
        );
    }

    public function handle_frontpage_settings_save() {
        if (isset($_POST['frontpage_type']) && check_admin_referer('frontpage_settings_nonce')) {
            set_theme_mod('frontpage_type', sanitize_text_field($_POST['frontpage_type']));
            
            if ($_POST['frontpage_type'] === 'category') {
                set_theme_mod('frontpage_select_category', sanitize_text_field($_POST['frontpage_select_category']));
            } elseif ($_POST['frontpage_type'] === 'page') {
                set_theme_mod('frontpage_select_page', sanitize_text_field($_POST['frontpage_select_page']));
            } elseif ($_POST['frontpage_type'] === 'project') {
                set_theme_mod('frontpage_select_project', sanitize_text_field($_POST['frontpage_select_project']));
            }
            
            // Set a transient for the success message
            set_transient('frontpage_settings_message', 'Frontpage settings saved.', 45);
        }
    }

    public function render_frontpage_settings_page() {
        ?>
        <div class="wrap">
            <h1>Set Frontpage</h1>
            <?php 
            // Display and delete the transient message
            if ($message = get_transient('frontpage_settings_message')) {
                echo '<div class="notice notice-success is-dismissible"><p>' . esc_html($message) . '</p></div>';
                delete_transient('frontpage_settings_message');
            }
            ?>
            <div class="lay-explanation">
                <div class="lay-explanation-inner">
                    <p>This is the same setting as in Customize &rarr; Front Page.</p>
                </div>
            </div>
            <form method="post" action="">
                <?php wp_nonce_field('frontpage_settings_nonce'); ?>
                <table class="form-table">
                    <tr>
                        <th scope="row">Front Page displays</th>
                        <td>
                            <fieldset>
                                <label>
                                    <input type="radio" name="frontpage_type" value="category" <?php checked(get_theme_mod('frontpage_type', 'category'), 'category'); ?>>
                                    Category
                                </label><br>
                                <label>
                                    <input type="radio" name="frontpage_type" value="page" <?php checked(get_theme_mod('frontpage_type', 'category'), 'page'); ?>>
                                    Page
                                </label><br>
                                <label>
                                    <input type="radio" name="frontpage_type" value="project" <?php checked(get_theme_mod('frontpage_type', 'category'), 'project'); ?>>
                                    Project
                                </label>
                            </fieldset>
                        </td>
                    </tr>
                    <tr class="frontpage-option frontpage-category-option" style="<?php echo get_theme_mod('frontpage_type', 'category') === 'category' ? '' : 'display: none;'; ?>">
                        <th scope="row">Choose a Category</th>
                        <td>
                            <?php
                            wp_dropdown_categories(array(
                                'name' => 'frontpage_select_category',
                                'selected' => get_theme_mod('frontpage_select_category', '1'),
                                'hierarchical' => true,
                                'show_option_none' => 'Select a category',
                                'option_none_value' => ''
                            ));
                            ?>
                        </td>
                    </tr>
                    <tr class="frontpage-option frontpage-page-option" style="<?php echo get_theme_mod('frontpage_type', 'category') === 'page' ? '' : 'display: none;'; ?>">
                        <th scope="row">Choose a Page</th>
                        <td>
                            <?php
                            wp_dropdown_pages(array(
                                'name' => 'frontpage_select_page',
                                'selected' => get_theme_mod('frontpage_select_page', ''),
                                'show_option_none' => 'Select a page',
                                'option_none_value' => ''
                            ));
                            ?>
                        </td>
                    </tr>
                    <tr class="frontpage-option frontpage-project-option" style="<?php echo get_theme_mod('frontpage_type', 'category') === 'project' ? '' : 'display: none;'; ?>">
                        <th scope="row">Choose a Project</th>
                        <td>
                            <?php
                            $projects = get_posts(array(
                                'post_type' => 'post',
                                'posts_per_page' => -1,
                                'orderby' => 'title',
                                'order' => 'ASC'
                            ));
                            ?>
                            <select name="frontpage_select_project" id="frontpage_select_project">
                                <option value="">Select a project</option>
                                <?php foreach ($projects as $project) : ?>
                                    <option value="<?php echo $project->ID; ?>" <?php selected(get_theme_mod('frontpage_select_project', ''), $project->ID); ?>>
                                        <?php echo $project->post_title; ?>
                                    </option>
                                <?php endforeach; ?>
                            </select>
                        </td>
                    </tr>
                </table>
                <?php submit_button(); ?>
            </form>
        </div>
        <script>
        jQuery(document).ready(function() {
            // Function to update visibility based on selected type
            function updateVisibility() {
                var selectedType = jQuery('input[name="frontpage_type"]:checked').val();
                console.log('Selected type:', selectedType); // Debug log
                jQuery('.frontpage-option').hide();
                jQuery('.frontpage-' + selectedType + '-option').show();
            }

            // Initial visibility update
            updateVisibility();

            // Update visibility when radio buttons change
            jQuery('input[name="frontpage_type"]').on('change', function() {
                console.log('Radio changed to:', jQuery(this).val()); // Debug log
                updateVisibility();
            });
        });
        </script>
        <?php
    }
}

// Initialize the class
new LaySetFrontpage();

