<?php
// https://wordpress.stackexchange.com/questions/220680/how-to-change-wp-attachment-metadata-specific-value-in-sql
// http://localhost:10020/wp-admin/post.php?post=2735&action=edit

class LayAttachmentDimensions{

	public function __construct(){
		// show metabox with width and height in post type attachment
		add_action('add_meta_boxes', array($this, 'add_attachment_dimensions_metabox'));
		// when saving attachment page, save new width and height into metadata, by using wp_get_attachment_metadata and wp_update_attachment_metadata
		// https://wordpress.stackexchange.com/questions/116877/save-post-not-working-with-attachments
		add_action('attachment_updated', array($this, 'save_attachment_dimensions'), 10, 3);
	}

	public function save_attachment_dimensions($post_id, $post_after, $post_before){
		$width = isset($_POST['attachment-width']) ? intval($_POST['attachment-width']) : 0;
		$height = isset($_POST['attachment-height']) ? intval($_POST['attachment-height']) : 0;

		if(!is_numeric($width) || !is_numeric($height)){
			return;
		}
		if($width <= 0 || $height <= 0){
			return;
		}

		$metadata = wp_get_attachment_metadata($post_id);
		if($metadata){
			$metadata['width'] = $width;
			$metadata['height'] = $height;
			wp_update_attachment_metadata($post_id, $metadata);
		}
	}

	public function add_attachment_dimensions_metabox(){
		add_meta_box(
			'attachment_dimensions',
			'Edit Attachment Dimensions',
			array($this, 'attachment_dimensions_metabox'),
			'attachment',
			'side',
			'high'
		);
	}

	public function attachment_dimensions_metabox($post){
		$metadata = wp_get_attachment_metadata($post->ID);
		$width = 0;
		$height = 0;

		if($metadata){
			$width = $metadata['width'];
			$height = $metadata['height'];
		}

		?>
		<p>
			<label for="attachment-width">Width:</label>
			<input type="number" min="1" step="1" id="attachment-width" name="attachment-width" value="<?php echo $width; ?>" /> px
		</p>
		<p>
			<label for="attachment-height">Height:</label>
			<input type="number" min="1" step="1" id="attachment-height" name="attachment-height" value="<?php echo $height; ?>" /> px
		</p>
		<?php
	}

}
$layattdim = new LayAttachmentDimensions();