<?php
class VideoThumbnails{

	public static $active;

	public function __construct(){
		$val = get_option('misc_options_thumbnail_video', '');
		if( $val == "on" ){
			VideoThumbnails::$active = true;
			add_action( 'add_meta_boxes_post', array( $this, 'add_meta_box' ) );
			add_action( 'save_post', array($this, 'save_video_thumb'), 1, 2 );
			add_action( 'admin_enqueue_scripts', array( $this, 'video_thumb_script' ) );
			add_action( 'wp_ajax_get_video_post_thumb_html', array( $this, 'video_thumbnail_html_ajax_callback' ) );
		}
		else{
			VideoThumbnails::$active = false;
		}
	}

	public function video_thumb_script() {
		$screen = get_current_screen();
		if ( $screen->id == 'post' ) {
			wp_enqueue_script( 'lay-video_thumb', Setup::$uri.'/thumbnails/js/video_thumbnails.js', array(), Setup::$ver);
		}
	}

	public function save_video_thumb($postid, $post){
		if ( isset( $_POST['_lay_thumbnail_video'] ) ) {
			update_post_meta($postid, '_lay_thumbnail_video', $_POST['_lay_thumbnail_video']);
		}

        $lay_useExternalVideo = isset( $_POST['lay_useExternalVideo'] ) ? $_POST['lay_useExternalVideo'] : '';
        update_post_meta($postid, 'lay_useExternalVideo', $lay_useExternalVideo);

        if ( isset( $_POST['lay_externalVideoUrl'] ) ) {
            update_post_meta($postid, 'lay_externalVideoUrl', $_POST['lay_externalVideoUrl']);
        }
        if ( isset( $_POST['lay_externalVideoW'] ) ) {
            update_post_meta($postid, 'lay_externalVideoW', $_POST['lay_externalVideoW']);
        }
        if ( isset( $_POST['lay_externalVideoH'] ) ) {
            update_post_meta($postid, 'lay_externalVideoH', $_POST['lay_externalVideoH']);
        }
	}

	public function add_meta_box(){
		add_meta_box( 'lay-video-thumbnail', 'Project Thumbnail Video', array( $this, 'video_thumbnail_meta_box'), 'post', 'side', 'low' );
	}

	// mostly just copied over from the standard featured image metabox
	// https://github.com/WordPress/WordPress/blob/a7a85856402272b465cde097b725653082db1884/wp-admin/includes/post.php#L1376
	function video_thumbnail_meta_box($post) {
		$thumbnail_id = get_post_meta( $post->ID, '_lay_thumbnail_video', true );
		echo VideoThumbnails::video_thumbnail_html( $thumbnail_id, $post->ID );
	}

	public static function video_thumbnail_html_ajax_callback(){
		$post_id = intval( $_POST['post_id'] );
		$thumbnail_id = intval( $_POST['thumbnail_id'] );
        $externalVideoOff = isset( $_POST['externalVideoOff'] ) ? $_POST['externalVideoOff'] : '';
		echo VideoThumbnails::video_thumbnail_html( $thumbnail_id, $post_id, $externalVideoOff );
		wp_die();
	}

	public static function video_thumbnail_html( $thumbnail_id = null, $postid = null, $externalVideoOff = false ){

        // get post meta for lay_useExternalVideo
        $useExternalVideo = get_post_meta($postid, 'lay_useExternalVideo', false);
        $externalVideoUrl = get_post_meta($postid, 'lay_externalVideoUrl', true);
        $externalVideoW = get_post_meta($postid, 'lay_externalVideoW', true);
        $externalVideoH = get_post_meta($postid, 'lay_externalVideoH', true);

        // when we set a mp4 video through the media library, we want to turn off the external video, we ajax this here
        if($externalVideoOff == true) {
            $useExternalVideo = array('off');
        }

        $checked = is_array($useExternalVideo) && count($useExternalVideo) > 0 && $useExternalVideo[0] == 'on' ? "checked" : "";

        // set default values if nothing was set yet:
        if( $externalVideoW == "" ){
            $externalVideoW = 16;
        }
        if( $externalVideoH == "" ){
            $externalVideoH = 9;
        }

        $hideExternal = $checked == '' ? 'display: none;' : '';
        $hideNormal = $checked == 'checked' ? 'display: none;' : '';

		$post               = get_post( $postid );
		$post_type_object   = get_post_type_object( $post->post_type );
		$set_thumbnail_link = '<p style="'.$hideNormal.'" class="hide-if-no-js hide-if-external-video"><a href="%s" id="set-video-post-thumbnail"%s class="thickbox">%s</a></p>';
		$upload_iframe_src  = get_upload_iframe_src( 'video', $post->ID );

        $content = '<div style="margin-top: 1em;"><label><input type="checkbox" id="lay_useExternalVideo" name="lay_useExternalVideo" value="on" '.$checked.' /> Use External Video</label></div>
        <div class="show-if-external-video" style="margin-top: 1em; '.$hideExternal.'"><label>URL to a Mp4 File<br/><input type="text" id="lay_externalVideoUrl" name="lay_externalVideoUrl" value="'.$externalVideoUrl.'" /></label></div>
        <div class="show-if-external-video" style="margin-top: 1em; '.$hideExternal.'"><label>Aspect Ratio Width<br/><input type="number" id="lay_externalVideoW" name="lay_externalVideoW" value="'.$externalVideoW.'" /></label></div>
        <div class="show-if-external-video" style="margin-top: 1em; '.$hideExternal.'"><label>Aspect Ratio Height<br/><input type="number" id="lay_externalVideoH" name="lay_externalVideoH" value="'.$externalVideoH.'" /></label></div>';
		
        $content .= sprintf( $set_thumbnail_link,
			esc_url( $upload_iframe_src ),
			'', // Empty when there's no featured image set, `aria-describedby` attribute otherwise.
			'Set a .mp4 video'
		);
		if ( $thumbnail_id && get_post( $thumbnail_id ) ) {
			$video = wp_get_attachment_url($thumbnail_id);
			$thumbnail_html = '<video style="'.$hideNormal.'" loop autoplay muted><source src="'.$video.'" type="video/mp4"></video>';
			if ( ! empty( $thumbnail_html ) ) {
				$content = sprintf( $set_thumbnail_link,
					esc_url( $upload_iframe_src ),
					' aria-describedby="set-video-post-thumbnail-desc"',
					$thumbnail_html
				);
				$content .= '<p class="hide-if-no-js howto hide-if-external-video" style="'.$hideNormal.'" id="set-video-post-thumbnail-desc">' . __( 'Click the video to edit or update' ) . '</p>';
				$content .= '<p class="hide-if-no-js hide-if-external-video" style="'.$hideNormal.'"><a href="#" id="remove-video-post-thumbnail">' . 'Remove video' . '</a></p>';
			}
		}


		$content .= 
        '<input type="hidden" id="_lay_thumbnail_video" name="_lay_thumbnail_video" value="' . esc_attr( $thumbnail_id ? $thumbnail_id : '-1' ) . '" />';
		echo $content;
	}

}
new VideoThumbnails();