<?php

// note to myself: remember to always release plugin updates first, then theme updates to make this work properly
// https://wordpress.stackexchange.com/questions/17394/how-to-know-what-functions-are-hooked-to-an-action-filter

// also note to myself: dont change the path to the file plugin_update_check.php, so dont change folder name "update_plugins_after_theme_update/plugin_update_check.php"
// because this file is included in fetchLayPluginUpdates() and when we update the theme, this function (of the old code) is called, but the path to the file could be wrong if we rename/move the folder/file

class LayUpdatePluginsAfterThemeUpdate{

	public function __construct() {
        // it is only triggered when it already existed in the theme before the update:
        // because a theme update is triggered using ajax, but the person stays on the same page
        // but for the hook to be triggered, the page needs to be reloaded if the hook hasnt been added before
        add_action( 'upgrader_process_complete', array($this, 'lay_on_upgrader_process_complete'),10, 2);
        
        // so my solution is:
        // calling this on the first time we dont have the theme version with above hook yet
        // and inside the function we check if we already did this before with get_option('lay_plugins_update_once_before_upgrader_process_complete_exists', 'yes');
        // this only works if the person doesnt have automatic theme updates on and goes to another page while logged in as admin, after updating the theme
        // if he has automatic updates on, this will work the next time he enters the admin area i think :D
        add_action('after_setup_theme', array($this, 'update_lay_plugins_on_theme_update'));
    }

    public static function update_lay_plugins_on_theme_update(){
        // https://preventdirectaccess.com/how-to-update-wordpress-plugins-automatically/
        $theme = wp_get_theme();

        $do_update_once = get_option('lay_plugins_update_once_before_upgrader_process_complete_exists', 'yes');

        if( $do_update_once == 'yes' ){
            
            LayUpdatePluginsAfterThemeUpdate::updateLayPlugins();
        }
    }

    public static function lay_on_upgrader_process_complete($upgrader_object, $hook_extra) {
        
        $theme_name = 'lay';
        $theme_wasupdated = 'theme_not_updated';
    
        if ( isset( $hook_extra['themes'] ) && is_array( $hook_extra['themes'] ) ) {
            foreach ( $hook_extra['themes'] as $index => $updated_theme ) {
                if ( $theme_name == $updated_theme ) {
                    $theme_wasupdated = 'theme_updated';
                    break;
                }
            }
        }
        
        
        if ( $theme_wasupdated == 'theme_updated' ) {
            LayUpdatePluginsAfterThemeUpdate::updateLayPlugins();
            
        }

    }

    public static function fetchLayPluginUpdates(){
        // ususally, the kernl plugin updates are only fetched when we are in the admin panel, visiting pages like update, plugins, etc.
        // but here, we can fetch them manually
        $all_plugins = get_plugins();

        foreach ($all_plugins as $key => $value) {
            if( strpos($key, 'laytheme') === false ){
                unset($all_plugins[$key]);
            }
        }

        

        if(count($all_plugins) > 0) {
            include_once dirname(__FILE__).'/plugin_update_check.php';

            foreach ($all_plugins as $key => $value) {

                $plugin_path = WP_PLUGIN_DIR . '/' . $key;
                

                switch($value['TextDomain']) {
                    case 'laytheme-lightbox':
                        
                        $KernlUpdaterLightbox = new PluginUpdateChecker_2_0 (
                            'https://kernl.us/api/v1/updates/56b3219b681396fc05d44cf8/',
                            $plugin_path,
                            'laytheme-lightbox',
                            // 0 means dont create cronjobs to check for updates
                            0
                        );
                        $KernlUpdaterLightbox->checkForUpdates();
                    break;
                    case 'laytheme-carousel':
                        
                        $KernlUpdaterCarousel = new PluginUpdateChecker_2_0 (
                            'https://kernl.us/api/v1/updates/56b09a95ffb0be4f0769e299/',
                            $plugin_path,
                            'laytheme-carousel',
                            // 0 means dont create cronjobs to check for updates
                            0
                        );
                        $KernlUpdaterCarousel->checkForUpdates();
                    break;
                    case 'laytheme-imagehover':
                        
                        $KernlUpdaterImagehover = new PluginUpdateChecker_2_0 (
                            'https://kernl.us/api/v1/updates/5ea4015ce8bd8b2fc0e51870/',
                            $plugin_path,
                            'laytheme-imagehover',
                            // 0 means dont create cronjobs to check for updates
                            0
                        );
                        $KernlUpdaterImagehover->checkForUpdates();
                    break;
                    case 'laytheme-magneticslides':
                        
                        $KernlUpdaterMagneticslides = new PluginUpdateChecker_2_0 (
                            'https://kernl.us/api/v1/updates/56b32245681396fc05d44cf9/',
                            $plugin_path,
                            'laytheme-magneticslides',
                            // 0 means dont create cronjobs to check for updates
                            0
                        );
                        $KernlUpdaterMagneticslides->checkForUpdates();
                    break;
                }
            }
        }
    }

    public static function updateLayPlugins(){
        // usually, a call to wp_update_plugins(); would fetch the updates for all plugins
        // but for kernl updates, we need to fetch them manually first. otherwise kernl only fetches updates like once an hour / day
        // or when ure on the updates page or plugins page. but with this function call i fetch them right now. so even if im not logged in as an admin
        // this should see if there are updates for the lay plugins:
        LayUpdatePluginsAfterThemeUpdate::fetchLayPluginUpdates();

        include_once ABSPATH . 'wp-admin/includes/file.php';
        include_once ABSPATH . 'wp-admin/includes/update.php';
        include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';

        // fetch if plugins are updateable
        // https://developer.wordpress.org/reference/functions/wp_update_plugins/
        // calling this function calls the hook "site_transient_update_plugins". this hook is binded to in kernl plugin update checker
        // where it inserts its update info into the 'update_plugins' transient, which is later used in "get_plugin_updates"
        wp_update_plugins();

        $skin = new WP_Ajax_Upgrader_Skin();
        $upgrader = new Plugin_Upgrader( $skin );


        $updates = get_plugin_updates();
        
        

        foreach ( $updates as $plugin ) {
            if( strpos($plugin->TextDomain, 'laytheme') !== false ){
                // we update all lay theme plugins. but only some of them are active, save active state here
                $plugin_is_active = is_plugin_active($plugin->update->plugin);

                
                
                
                

                // update the plugin!
                $result = $upgrader->upgrade($plugin->update->plugin);
                // // upgrading a plugin seems to disable it, so we enable it here again:
                if ( $result && $plugin_is_active ) {
                    
                    // updating a plugin disables it, if it was enabled before, we enable it again
                    activate_plugin( $plugin->update->plugin, '', false, true );
                }
            }
        }
        // only set this option after plugins have really been updated
        update_option('lay_plugins_update_once_before_upgrader_process_complete_exists', 'no');
    }
}
new LayUpdatePluginsAfterThemeUpdate();