<?php
/**
 * Single Product Image
 *
 * This template can be overridden by copying it to yourtheme/woocommerce/single-product/product-image.php.
 *
 * HOWEVER, on occasion WooCommerce will need to update template files and you
 * (the theme developer) will need to copy the new files to your theme to
 * maintain compatibility. We try to do this as little as possible, but it does
 * happen. When this occurs the version of the template file will be bumped and
 * the readme will list any important changes.
 *
 * @see     https://docs.woocommerce.com/document/template-structure/
 * @package WooCommerce/Templates
 * @version 9.7.0
 */

defined( 'ABSPATH' ) || exit;

// Note: `wc_get_gallery_image_html` was added in WC 3.3.2 and did not exist prior. This check protects against theme overrides being used on older versions of WC.
if ( ! function_exists( 'wc_get_gallery_image_html' ) ) {
	return;
}

global $product;

$post_thumbnail_id = array( $product->get_image_id() );

$nav = get_theme_mod('lay_woocommerce_singleproduct_show_carousel_buttons', false) ? 'lay-woocommerce-gallery-navigation-enabled' : '';
$attachment_ids = $product->get_gallery_image_ids();
if( get_theme_mod('lay_woocommerce_singleproduct_show_first_image', true) == 1 ) {
    $attachment_ids = array_merge($post_thumbnail_id, $attachment_ids);
}

$layout_type = get_theme_mod('lay_woocommerce_singleproduct_layout_type', 'one_image_per_row');
$thumbnails = '';

$variations = array();
if( method_exists($product, 'get_available_variations') ) {
    $variations = $product->get_available_variations();
}

if($layout_type == 'carousel_with_thumbnails') {
    $thumbnails .= '<div class="lay-woocommerce-gallery-thumbnails">';

    // here, just like in the carousel below, i add all variation images and hide them
    if( method_exists($product, 'get_available_variations') ) {
        foreach ( $variations as $variation ) {
            $variation_image_ids = array();
            $variation_image_ids[] = $variation['image_id'];

            // get additional images
            $variation_id = $variation['variation_id'];
            $variation_object = wc_get_product($variation_id);

            $additional_img_attachment_ids = $variation_object->get_meta( 'additional_img_ids' );
            if(!empty($additional_img_attachment_ids) && is_array($additional_img_attachment_ids)) {
                $variation_image_ids = array_merge($variation_image_ids, $additional_img_attachment_ids);
            }
            // #get additional images

            foreach( $variation_image_ids as $variation_image_id ) {
                // if( !in_array( $variation_image_id, $attachment_ids ) ) {

                    // note to myself: i commented out the above line because
                    // i now changed it to: im still adding variation images, even if they already exist in the product gallery
                    // this way, i get the variation_data_attributes, so this way the image can appear at the top, when the variation is chosen
                    // and it can be scrolled to that way

                    $landscape_portrait_class = 'portrait';
                    $img = wp_get_attachment_image_src( $variation_image_id );
                    if( is_array($img) ) {
                        if( $img[1] > $img[2] ) {
                            $landscape_portrait_class = 'landscape';
                        }
                        
                        $variation_data_attributes = '';
            
                        $keys = array_keys($variation['attributes']);
                        foreach( $keys as $key ) {
                            $variation_data_attributes .= 'data-'.$key.'="'.$variation['attributes'][$key].'" ';
                        }
                        $thumbnails .= '<div style="display:none;" '.$variation_data_attributes.' class="lay-woocommerce-gallery-thumbnail lay-woocommerce-product-variation-slide">'.LayElFunctions::getLazyImgByImageId($variation_image_id).'</div>';
                    }
                // }
            }
        }
    }
    
    $ix = 0;
    foreach ( $attachment_ids as $attachment_id ) {
        $thumbnails .= '<div class="lay-woocommerce-gallery-thumbnail '.($ix == 0 ? 'active' : '').'">'.LayElFunctions::getLazyImgByImageId($attachment_id).'</div>';
        $ix++;
    }
    $thumbnails .= '</div>';
}

$lay_woocommerce_singleproduct_desktop_carousel_slide_duration = get_theme_mod('lay_woocommerce_singleproduct_desktop_carousel_slide_duration', 300);
$drag_cursor_class = $lay_woocommerce_singleproduct_desktop_carousel_slide_duration > 0 ? 'drag-cursor' : 'no-drag-cursor';

$carousel_height_class = '';
if($layout_type == 'carousel_with_thumbnails' || $layout_type == 'carousel') {
    $val = get_theme_mod('lay_woocommerce_singleproduct_desktop_carousel_height', 'auto');
    $carousel_height_class = 'carousel-height-'.$val;
}
$html = 
$thumbnails.'<div class="lay-woocommerce-gallery swiper-container '.$nav.' '.$carousel_height_class.'">
<div class="swiper-wrapper '.$drag_cursor_class.'">';


if( method_exists($product, 'get_available_variations') ) {
    foreach ( $variations as $variation ) {
        $variation_image_ids = array();
        $variation_image_ids[] = $variation['image_id'];

        // get additional images
        $variation_id = $variation['variation_id'];
        $variation_object = wc_get_product($variation_id);

        $additional_img_attachment_ids = $variation_object->get_meta( 'additional_img_ids' );
        if(!empty($additional_img_attachment_ids) && is_array($additional_img_attachment_ids)) {
            $variation_image_ids = array_merge($variation_image_ids, $additional_img_attachment_ids);
        }
        // #get additional images
        
        foreach( $variation_image_ids as $variation_image_id ) {
            // if( !in_array( $variation_image_id, $attachment_ids ) ) {

                // note to myself: i commented out the above line because
                // i now changed it to: im still adding variation images, even if they already exist in the product gallery
                // this way, i get the variation_data_attributes, so this way the image can appear at the top, when the variation is chosen
                // and it can be scrolled to that way

                $landscape_portrait_class = 'portrait';
                $img = wp_get_attachment_image_src( $variation_image_id );
                if( is_array($img) ) {
                    if( $img[1] > $img[2] ) {
                        $landscape_portrait_class = 'landscape';
                    }
                    
                    $variation_data_attributes = '';
        
                    $keys = array_keys($variation['attributes']);
                    foreach( $keys as $key ) {
                        $variation_data_attributes .= 'data-'.$key.'="'.$variation['attributes'][$key].'" ';
                    }
                    $html .= '<div style="display:none;" '.$variation_data_attributes.' class="swiper-slide lay-woocommerce-product-variation-slide">'.LayElFunctions::getLazyImgByImageId($variation_image_id).'</div>';
                }
            // }
        }
    }
}

foreach ( $attachment_ids as $attachment_id ) {
    $landscape_portrait_class = 'portrait';
    $img = wp_get_attachment_image_src( $attachment_id );
    if( $img[1] > $img[2] ) {
        $landscape_portrait_class = 'landscape';
    }
    $html .= '<div class="swiper-slide">'.LayElFunctions::getLazyImgByImageId($attachment_id).'</div>';
}

$use_buttons = get_theme_mod('lay_woocommerce_singleproduct_show_carousel_buttons', false) ? true : false;
$buttons = '';
if( $use_buttons ) {
    $buttons = '<span class="lay-product-button-next">
        <svg viewBox="0 0 100 100">
            <path d="M 20,50 L 75,100 L 80,95 L 30,50  L 80,5 L 75,0 Z" class="arrow" transform="translate(100, 100) rotate(180) "></path>
        </svg>
    </span>';
    $buttons .= '<span class="lay-product-button-prev">
        <svg viewBox="0 0 100 100">
            <path d="M 20,50 L 75,100 L 80,95 L 30,50  L 80,5 L 75,0 Z" class="arrow"></path>
        </svg>
    </span>';
}

$html .= 
'</div>
'.$buttons;

$html .=
'<div class="swiper-pagination"></div>
</div>';
echo $html;