<?php
/**
 * Plugin Name: Lay Theme Imagehover
 * Plugin URI: https://laytheme.com/addons/
 * Description: When hovering over a textlink an image can show up! Only works with Lay Theme!
 * Version: 1.4.5
 * Author: 100k Studio
 * Author URI: http://100k.studio
 */

// todo: option to have an image just fixed in center, not dependend on mouse and text position √
// todo: width / height in % √
// todo: transition fade: maybe like do it like hellome √
// todo: think about how to solve z-index issues √
// todo: how does opacity work for this when its a lazyloading image √ 
// todo: need to preload all these images? √
// todo: are these images loaded by content_ready and my image loading javascript? √
// todo: how does this work if we have many links side by side. are there gaps and flickering? √
// todo: think about how to do this for touchdevices - nothing to do here √
// todo: there seems to be a bug where when i scroll and move the mouse, two images are shown √
// todo: if i have 2 or 3 of the same images, test if there are doubles in the markup √
// todo: if there are doubles, test if they show up on fast switching between images √
// todo: make this work with cover, cpl √

// todo: only show modal sethoverimage buttons when coming from a texteditor for a textelement (not caption etc)
// todo: test on major browsers √

// 2023 update:
// when using any hoverimage "image position" that follows the mouse, currently I put the image
// next to the col. this plays nicely with z-indices and the cover feature √


// todo: either only load desktop or cpl content based on where im on 

// todo: projectindex video when videothumbnail is active √
// todo: expand-row link modal, expand-row link √
// todo: when i sort projectindex element on frontend, scroll doesn't work :) √
// todo: cpl and apl :) √
// todo: projectindex √
// todo: row hoverimage √
// todo: sizes √
// todo: videos √
// todo: choose Image Position per link :D √
// todo: imagehover settings: put mu and value in one line :) √
// todo: cover compatibility with rightbg, leftbg, centerfixed (…) and z-index setting √
// todo: space around setting for rightbg and leftbg - created custom position √
// todo: position settings for top left corner, bottom left corner, top right corner, bottom right corner √
// todo: tabs so you can set width, height, top, left, etc for each position separately √
// todo: add a setting so an imagehover image/video can be shown from the start, even without having hovered a link! √
// todo: make it possible to add an imagehover even without a link! √

class LayThemeImagehover{

	public static $url;
	public static $dir;
	public static $ver;
    public static $clickable;

	public function __construct(){
		LayThemeImagehover::$url = plugins_url( 'laytheme-imagehover' );
		LayThemeImagehover::$dir = plugin_dir_path( __FILE__ );
		LayThemeImagehover::$ver = '1.4.5';

        LayThemeImagehover::$clickable = get_option( 'imagehover_clickable', '' );

		// backend
		add_action( 'admin_menu', array($this, 'imagehover_setup_menu'), 20 );
		add_action( 'admin_init', array($this, 'imagehover_settings_api_init') );
		add_filter( 'plugin_action_links_'.plugin_basename(__FILE__), array($this, 'add_action_links') );
		add_action( 'admin_enqueue_scripts', array($this, 'add_imagehover_admin_scripts_and_styles') );

		// frontend
		add_action( 'wp_enqueue_scripts', array( $this, 'imagehover_scripts' ), 11 );
		add_action( 'wp_footer', array( $this, 'imagehover_styles' ) );
	}

    public static function getMarkupForEl( $desktop_markup = '' ){
        if( class_exists('LTUtility') && property_exists( 'LTUtility', 'getFullURL' ) && LTUtility::$isTouchDevice == true ){
			return '';
		}
    }

    public static function getImagehoverVideo($media){
        // always lazyload videos so desktop videos dont get loaded on cpl mobile :)
        // also safari crashes at the moment when too many videos are loaded at the same time
        return '<video data-id="'.$media['attid'].'" '.$media['extra_attrs'].' class="'.$media['extra_classes'].' video-lazyload" playsinline loop autoplay muted>
            <source data-src="'.$media['url'].'" type="video/mp4">
        </video>';
    }

	public static function getMarkup( $array_of_rows, $desktop_markup = '' ){
		if( class_exists('LTUtility') && property_exists( 'LTUtility', 'getFullURL' ) && LTUtility::$isTouchDevice == true ){
			return '';
		}
		$existing_image_ids = array();
		$imagesOrVideos = array();
		foreach( $array_of_rows as $rows ) {
			if( !is_array($rows) ){
				continue;
			}
			// images of linked rows
			for( $i = 0; $i < count($rows); $i++ ) {
				$row = $rows[$i];
				if( array_key_exists('link', $row) ) {
					$rowlink = $row['link'];
					if( array_key_exists('hoverimageid', $rowlink) && $rowlink['hoverimageid'] != null ) {
						$attid = $rowlink['hoverimageid'];
						if( !in_array($attid, $existing_image_ids) ){
							$media = LayThemeImagehover::getImgOrVideoObject( $attid );
							if( $media != false ) {
                                if( array_key_exists('type', $media) && $media['type'] == 'video' ) {
                                    $imagesOrVideos []= LayThemeImagehover::getImagehoverVideo($media);
                                } else {
                                    $imagesOrVideos []= LayElFunctions::getLazyImg( $media );
                                }
							}
						}
						$existing_image_ids []= $attid;
					}
				}
			}

			// images of links in content
			for( $i = 0; $i < count($rows); $i++ ) {
				$row = $rows[$i];
				if( is_array($row) && array_key_exists('columns', $row) ) {
					for( $i3=0; $i3<count($row['columns']); $i3++ ) {
						$col = $row['columns'][$i3];
						switch( $col['type'] ) {
                            case 'html':
							case 'text':
								LayThemeImagehover::getImageFromCol($col, $imagesOrVideos, $existing_image_ids);
							break;
							case 'stack':
								$stack_cols = $col['cont'];
								foreach( $stack_cols as $col_in_stack ) {
									if( $col_in_stack['type'] == 'text' || $col_in_stack['type'] == 'html' ) {
										LayThemeImagehover::getImageFromCol($col_in_stack, $imagesOrVideos, $existing_image_ids);
									}
                                    else if( $col_in_stack['type'] == 'projectindex' ){
                                        LayThemeImagehover::getImageFromProjectIndex($col_in_stack, $imagesOrVideos, $existing_image_ids);
                                    }
								}
							break;
							case 'projectindex':
								LayThemeImagehover::getImageFromProjectIndex($col, $imagesOrVideos, $existing_image_ids);
							break;
						}
					}
				}
			}
		}

		if( isset($desktop_markup) && trim($desktop_markup) != '' ) {
			// Create DOM from Markup string
			$desktop_document = new DomDocument();
			libxml_use_internal_errors(true);
			$desktop_document->loadHTML( $desktop_markup );
			$desktop_xpath = new DomXPath( $desktop_document );
			// Let's see whether we have some thumbnailgrids
			$thumbnailgrid_elements = $desktop_xpath->query( "//div[contains(@data-type, 'thumbnailgrid')]" );

			foreach( $thumbnailgrid_elements as $thumbnailgrid_el ) {
				LayThemeImagehover::getImageFromThumbnailgridMarkup($desktop_xpath, $thumbnailgrid_el, $imagesOrVideos, $existing_image_ids);
			}

            $shortcode_elements = $desktop_xpath->query( "//div[contains(@data-type, 'shortcode')]" );
            foreach( $shortcode_elements as $shortcode_el ) {
				LayThemeImagehover::getImageFromMarkup($desktop_xpath, $shortcode_el, $imagesOrVideos, $existing_image_ids);
			}
		}

		return implode($imagesOrVideos);
	}

	public static function getImageFromProjectIndex($col, &$imagesOrVideos, &$existing_image_ids) {
        $config = $col['config'];
		if( $config['useImageHover'] == 1 ) {

            $useImageHoverVideos = false;
            if ( array_key_exists('useImageHoverVideos', $config) && $config['useImageHoverVideos'] == 1 && VideoThumbnails::$active == true ) {
                $useImageHoverVideos = true;
            }

			// only the projects of these categories are shown. empty for all. this is the value of the multiselector for categories in the modal
			$category_term_ids = array();
			if( is_array( $config['category'] ) ) {
				foreach ($config['category'] as $cat) {
					$term = get_term_by('slug', $cat['value'], 'category');
					$category_term_ids []= $term->term_id;
				}
			}
			$query_args = array(
				'post_type' => array('post'),
				'post_status' => 'publish',
				'posts_per_page' => -1
			);
			if( count($category_term_ids) > 0 ) {
				$query_args['tax_query'] = array(
					'taxonomy' => 'category',
					'field' => 'term_id',
					'terms' => $category_term_ids
				);
			}
            
			$query = new WP_Query($query_args);
			
			if ( $query->have_posts() ) {
				foreach ($query->posts as $post) {
					$attid = get_post_thumbnail_id( $post->ID );

                    if ( $useImageHoverVideos ) {
                        $video_id = get_post_meta( $post->ID, '_lay_thumbnail_video', true );
                        if($video_id != -1){
                            $attid = $video_id;
                        }
                    }

					if( !in_array($attid, $existing_image_ids) ) {
						$media = LayThemeImagehover::getImgOrVideoObject( $attid );
                        if( $media != false ) {
                            if( array_key_exists('type', $media) && $media['type'] == 'video' ) {
                                $imagesOrVideos []= LayThemeImagehover::getImagehoverVideo($media);
                            } else {
                                $imagesOrVideos []= LayElFunctions::getLazyImg( $media );
                            }
                        }
					}
					$existing_image_ids []= $attid;
				}
			}
		}
	}

	public static function getImageFromCol($col, &$imagesOrVideos, &$existing_image_ids) {
		$cont = $col['cont'];
		if( strpos($cont, 'data-hoverimageid') !== false ) {
			$dom = new DomDocument();
			$dom->loadHTML( $cont );
			$anchors = $dom->getElementsByTagName( "a" );
			foreach( $anchors as $anchor ){
				$attid = $anchor->getAttribute('data-hoverimageid');
				if( $attid != '' ) {
					// make sure that we don't have doubles
					if( !in_array($attid, $existing_image_ids) ){
                        $media = LayThemeImagehover::getImgOrVideoObject( $attid );
                        if( $media != false ) {
                            if( array_key_exists('type', $media) && $media['type'] == 'video' ) {
                                $imagesOrVideos []= LayThemeImagehover::getImagehoverVideo($media);
                            } else {
                                $imagesOrVideos []= LayElFunctions::getLazyImg( $media );
                            }
                        }
					}
					$existing_image_ids []= $attid;
				}
			}
		}
	}

    public static function getImageFromMarkup2($elMarkup) {
        $imagesOrVideos = array();
		$cont = $elMarkup;

        if( is_string($cont) ) {
            if( strpos($cont, 'data-hoverimageid') !== false ) {
                $dom = new DomDocument();
                // https://stackoverflow.com/questions/1148928/disable-warnings-when-loading-non-well-formed-html-by-domdocument-php
                libxml_use_internal_errors(true);
                $dom->loadHTML( $cont );
                $anchors = $dom->getElementsByTagName( "a" );
                foreach( $anchors as $anchor ){
                    $attid = $anchor->getAttribute('data-hoverimageid');
                    $position = $anchor->getAttribute('data-hoverimageposition');
                    if($position == 'inherit' || $position == '') {
                        $position = get_option( 'imagehover_imageposition', 'followmousex' );
                    }
                    $positionforscroll = $anchor->getAttribute('data-hoverimagepositionforscroll');
                    if($positionforscroll == 'inherit' || $positionforscroll == '') {
                        $positionforscroll = get_option( 'imagehover_imageposition_for_scroll', 'centerfixed' );
                    }

                    $showon = $anchor->getAttribute('data-hoverimageshowon');
                    if($showon == 'inherit' || $showon == '') {
                        $showon = get_option( 'imagehover_scrolltrigger', 'onhoveronly' );
                    }

                    $preactivated = $anchor->getAttribute('data-hoverimagepreactivated');

                    $href = $anchor->getAttribute('href');
                    $dataId = $anchor->getAttribute('data-id');
                    $dataType = $anchor->getAttribute('data-type');
                    $dataTitle = $anchor->getAttribute('data-title');
                    $dataCatid = $anchor->getAttribute('data-catid');

                    if( $attid != '' ) {
                        
                        $media = LayThemeImagehover::getImgOrVideoObject( $attid );

                        if( $media != false ) {
                            if( array_key_exists('type', $media) && $media['type'] == 'video' ) {
                                $imagesOrVideos []= array('href' => $href, 'dataId' => $dataId, 'dataType' => $dataType, 'dataTitle' => $dataTitle, 'dataCatid' => $dataCatid, 'showon' => $showon, 'position' => $position, 'positionforscroll' => $positionforscroll, 'preactivated' => $preactivated, 'media' => LayThemeImagehover::getImagehoverVideo($media));
                            } else {
                                $imagesOrVideos []= array('href' => $href, 'dataId' => $dataId, 'dataType' => $dataType, 'dataTitle' => $dataTitle, 'dataCatid' => $dataCatid, 'showon' => $showon, 'position' => $position, 'positionforscroll' => $positionforscroll, 'preactivated' => $preactivated, 'media' => LayElFunctions::getLazyImg( $media ));
                            }
                        }
                    }
                }
            }
            return $imagesOrVideos;
        }
        return false;
	}

    // old lay theme versions use this function:
    public static function getImageFromMarkup($dom_xpath, $shortcode_el, &$imagesOrVideos, &$existing_image_ids) {
        $cont = $shortcode_el->c14n();
		if( strpos($cont, 'data-hoverimageid') !== false ) {
			$dom = new DomDocument();
			$dom->loadHTML( $cont );
			$anchors = $dom->getElementsByTagName( "a" );
			foreach( $anchors as $anchor ){
				$attid = $anchor->getAttribute('data-hoverimageid');
				if( $attid != '' ) {
					// make sure that we don't have doubles
					if( !in_array($attid, $existing_image_ids) ){
						$media = LayThemeImagehover::getImgOrVideoObject( $attid );
                        if( $media != false ) {
                            if( array_key_exists('type', $media) && $media['type'] == 'video' ) {
                                $imagesOrVideos []= LayThemeImagehover::getImagehoverVideo($media);
                            } else {
                                $imagesOrVideos []= LayElFunctions::getLazyImg( $media );
                            }
                        }
					}
					$existing_image_ids []= $attid;
				}
			}
		}
    }

	// Todo: No need to search thumbnailgrids in markup.
	// Reason why I do that now: we don't store the thumb images of a thumbnailgrid in the json
	//													 thus we need to first generate the markup (done in laytheme's layout.php)
	// 													 and then pass it to this function
	public static function getImageFromThumbnailgridMarkup($dom_xpath, $thumbnailgrid_el, &$imagesOrVideos, &$existing_image_ids) {
		// If for whatever reason we have multiple thumb regions in the thumbnail grid element, we iterate over them
		// Kinda uneccessary
		$thumbregion_elements = $dom_xpath->query( "//*[contains(@class, 'thumbs-collection-region')]", $thumbnailgrid_el );

		foreach( $thumbregion_elements as $thumbregion_el ){
			$config = $thumbregion_el->getAttribute( 'data-config' );

			if( $config != null ){
				$config_json = json_decode( $config, true );

				$useImagehover = false;
				if( isset($config_json["useImagehover"]) ) {
						$useImagehover = $config_json["useImagehover"] ? true : false;
				}

				if( $config_json != null && $useImagehover ){
					$thumb_images_dom = $dom_xpath->query( "//*[contains(@class, 'thumb')]//img", $thumbregion_el );

					foreach( $thumb_images_dom as $thumb_image_dom ){
						$attid = $thumb_image_dom->getAttribute('data-id');
						if( $attid != '' ) {
							// make sure that we don't have doubles
							if( !in_array($attid, $existing_image_ids) ){
								$media = LayThemeImagehover::getImgOrVideoObject( $attid );
                                if( $media != false ) {
                                    if( array_key_exists('type', $media) && $media['type'] == 'video' ) {
                                        $imagesOrVideos []= LayThemeImagehover::getImagehoverVideo($media);
                                    } else {
                                        $imagesOrVideos []= LayElFunctions::getLazyImg( $media );
                                    }
                                }
							}
							$existing_image_ids []= $attid;
						}
					}
				}
			}
		}
	}

	public static function getImgOrVideoObject( $attid ) {
        $type = get_post_mime_type($attid);
        if(  strpos($type, 'video') === false ){
            // just to calculate aspect ratio
            $full = wp_get_attachment_image_src( $attid, 'full' );
            if( $full == false ) {
                return false;
            }
            $sizeclass = '';
            $ar = $full[1] != 0 ? $full[2] / $full[1] : 0;
            $sizeclass = $full[1] > $full[2] ? 'w100' : 'h100';

            $sizesToBeFilled = ['_265', '_512', '_768', '_1024', '_1280', '_1920', '_2560', '_3200', '_3840', '_4096'];
            $sizes = array('full' => $full[0]);

            for($i=0; $i<count($sizesToBeFilled); $i++){
                $att = wp_get_attachment_image_src( $attid, $sizesToBeFilled[$i] );
                if( is_array($att) ){
                    $sizes[$sizesToBeFilled[$i]] = $att[0];
                } else {
                    $sizes[$sizesToBeFilled[$i]] = $full[0];
                } 
            }

            $img = array(
                'type' => 'image',
                'sizes' => $sizes,
                'attid' => $attid,
                'ar' => $ar,
                'extra_attrs' => 'data-hoverimageid="'.$attid.'"',
                'extra_classes' => $sizeclass
            );
            return $img;
        } else {
            $video_attachment_meta = wp_get_attachment_metadata($attid);
            $ar = $video_attachment_meta['width'] != 0 ? $video_attachment_meta['height'] / $video_attachment_meta['width'] : 0;

            $sizeclass = '';
            if( get_option('imagehover_imageposition', 'followmousex') != 'centerfixed' ) {
                $sizeclass = $video_attachment_meta['width'] > $video_attachment_meta['height'] ? 'w100' : 'h100';
            }
            // video
            $video = array(
                'type' => 'video',
                'attid' => $attid,
                'ar' => $ar,
                'extra_attrs' => 'data-hoverimageid="'.$attid.'"',
                'extra_classes' => $sizeclass,
                'url' => wp_get_attachment_url($attid)
            );
            return $video;
        }
	}

	// http://code.tutsplus.com/articles/how-to-use-wordpress-color-picker-api--wp-33067
	public static function add_imagehover_admin_scripts_and_styles( $hook ) {
        if( strpos($hook, 'manage-layimagehover') !== false ){
			wp_enqueue_style( 'imagehover-admin-css', LayThemeImagehover::$url.'/backend/assets/css/admin.css', array(), LayThemeImagehover::$ver );
	        wp_enqueue_script( 'imagehover-showhide-settings', LayThemeImagehover::$url.'/backend/assets/js/settings/showhide_settings.js', array('jquery'), LayThemeImagehover::$ver, true );
		}
	}

    public static function percToVwVh($val, $pos, $heightOrWidth){
        if($pos == 'centerfixed' && $val == '%' && $heightOrWidth == 'height') {
            return 'vh';
        }
        if($val == '%') {
            return 'vw';
        }
        return $val;
    }

	public static function imagehover_styles(){
		wp_enqueue_style( 'imagehover-style', LayThemeImagehover::$url."/frontend/assets/css/frontend.style.css", array(), LayThemeImagehover::$ver );

		$imagehover_imageposition = get_option( 'imagehover_imageposition', 'followmousex' );

		$imagehover_under_other_elements = get_option( 'imagehover_zindex', 'under-all' );
		$zindex_css_desktop = '';

        if( $imagehover_under_other_elements == '' ) {
            // Put images above all elements
			$zindex_css_desktop = '.lay-imagehover-on-element{ z-index: 99; }';
		}
        if( $imagehover_under_other_elements == 'under-all' ) {
            // under all!
            $zindex_css_desktop = '.lay-imagehover-on-element{ z-index: 0; }
            .col {
                z-index: 1;
            }';
        }

        $imagehover_under_other_elements_for_phone = get_option( 'imagehover_zindex_for_phone', 'under-all' );
		$zindex_css_phone = '';
        if( $imagehover_under_other_elements_for_phone == '' ) {
            // Put images above all elements
			$zindex_css_phone = '.lay-imagehover-on-element{ z-index: 99; }';
		}
        if( $imagehover_under_other_elements_for_phone == 'under-all' ) {
            // under all!
            $zindex_css_phone = '.lay-imagehover-on-element{ z-index: 0; }
            .col {
                z-index: 1;
            }';
        }

		$imageposition = get_option( 'imagehover_imageposition', 'followmousex' );

        $phone_breakpoint = get_option('lay_breakpoint', 600);
        $phone_breakpoint = (int)$phone_breakpoint;

        wp_add_inline_style( 'imagehover-style',
            '@media (min-width: '.($phone_breakpoint+1).'px){
                '.$zindex_css_desktop.'
            }
            @media (max-width: '.($phone_breakpoint).'px){
                '.$zindex_css_phone.'
            }'
        );

		$positions1 = ['followmousex', 'followmousexy', 'sticktolink', 'centerfixed', 'csp'];

        $style = '';
        for ($i=0; $i < count($positions1); $i++) {
            $pos = $positions1[$i];

            $prefix = $pos.'_';
            if($pos == 'followmousex') {
                $prefix = '';
            }
            if($pos == 'csp'){
                $prefix = 'csp_';
                $pos = 'custom';
            }

            $d = get_option( 'imagehover_desktop_max_width', '400' );
            $desktop_max_width = get_option( $prefix.'imagehover_desktop_max_width', $d );

            $d = get_option( 'imagehover_desktop_max_height', '400' );
            $desktop_max_height = get_option( $prefix.'imagehover_desktop_max_height', $d );

            $d = get_option( 'imagehover_tablet_max_width', '300' );
            $tablet_max_width = get_option( $prefix.'imagehover_tablet_max_width', $d );

            $d = get_option( 'imagehover_tablet_max_width', '300' );
            $tablet_max_height = get_option( $prefix.'imagehover_tablet_max_height', $d );

            $d = get_option( 'imagehover_phone_max_width', '100' );
            $phone_max_width = get_option( $prefix.'imagehover_phone_max_width', $d );

            $d = get_option( 'imagehover_phone_max_height', '100' );
            $phone_max_height = get_option( $prefix.'imagehover_phone_max_height', $d );
            
            $d = get_option( 'imagehover_desktop_max_width_mu', 'px' );
            $desktop_max_width_mu = get_option( $prefix.'imagehover_desktop_max_width_mu', $d );

            $d = get_option( 'imagehover_desktop_max_height_mu', 'px' );
            $desktop_max_height_mu = get_option( $prefix.'imagehover_desktop_max_height_mu', $d );
            // if( $desktop_max_height_mu == '%' ) {
            // 	$desktop_max_height_mu = 'vw';
                // makes more sense to have vh here, because this way when someone says 90% height, the image won't be out of bounds
                // if( $imagehover_imageposition == 'centerfixed' ) {
                // 	$desktop_max_height_mu = 'vh';
                // }
            // }
            $d = get_option( 'imagehover_tablet_max_width_mu', 'px' );
            $tablet_max_width_mu = get_option( $prefix.'imagehover_tablet_max_width_mu', $d );

            $d = get_option( 'imagehover_tablet_max_height_mu', 'px' );
            $tablet_max_height_mu = get_option( $prefix.'imagehover_tablet_max_height_mu', $d );
            // if( $tablet_max_height_mu == '%' ) {
                // $tablet_max_height_mu = 'vw';
                // makes more sense to have vh here, because this way when someone says 90% height, the image won't be out of bounds
                // if( $imagehover_imageposition == 'centerfixed' ) {
                // 	$tablet_max_height_mu = 'vh';
                // }
            // }
            $d = get_option( 'imagehover_phone_max_width_mu', 'px' );
            $phone_max_width_mu = get_option( $prefix.'imagehover_phone_max_width_mu', $d );

            $d = get_option( 'imagehover_phone_max_height_mu', 'px' );
            $phone_max_height_mu = get_option( $prefix.'imagehover_phone_max_height_mu', $d );
            // if( $phone_max_height_mu == '%' ) {
                // $phone_max_height_mu = 'vw';
                // makes more sense to have vh here, because this way when someone says 90% height, the image won't be out of bounds
                // if( $imagehover_imageposition == 'centerfixed' ) {
                // 	$phone_max_height_mu = 'vh';
                // }
            // }

            if( class_exists('MiscOptions') ) {
                $style .= '@media (min-width: '.(MiscOptions::$tablet_breakpoint+1).'px){
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] img.w100,
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] video.w100{ width:'.$desktop_max_width.LayThemeImagehover::percToVwVh($desktop_max_width_mu, $pos, 'width').'; height:auto; }
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] img.h100,
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] video.h100{ width:auto; height:'.$desktop_max_height.LayThemeImagehover::percToVwVh($desktop_max_height_mu, $pos, 'height').'; }
                }
                @media (min-width: '.(MiscOptions::$phone_breakpoint+1).'px) and (max-width: '.(MiscOptions::$tablet_breakpoint).'px){
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] img.w100,
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] video.w100{ width:'.$tablet_max_width.LayThemeImagehover::percToVwVh($tablet_max_width_mu, $pos, 'width').'; height:auto; }
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] img.h100,
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] video.h100{ width:auto; height:'.$tablet_max_height.LayThemeImagehover::percToVwVh($tablet_max_height_mu, $pos, 'height').'; }
                }
                @media (max-width: '.(MiscOptions::$phone_breakpoint).'px){
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] img.w100,
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] video.w100{ width:'.$phone_max_width.LayThemeImagehover::percToVwVh($phone_max_width_mu, $pos, 'width').'; height:auto; }
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] img.h100,
                    .lay-imagehover-on-element[data-imageposition="'.$pos.'"] video.h100{ width:auto; height:'.$phone_max_height.LayThemeImagehover::percToVwVh($phone_max_height_mu, $pos, 'height').'; }
                }';
            }

        }

        // csp style :)
        $csp_imagehover_desktop_horizontal_space_around = get_option( 'csp_imagehover_desktop_horizontal_space_around', '10' );
        $csp_imagehover_desktop_horizontal_space_around_mu = get_option( 'csp_imagehover_desktop_horizontal_space_around_mu', 'px' );
        $csp_imagehover_desktop_vertical_space_around = get_option( 'csp_imagehover_desktop_vertical_space_around', '10' );
        $csp_imagehover_desktop_vertical_space_around_mu = get_option( 'csp_imagehover_desktop_vertical_space_around_mu', 'px' );
        
        $csp_imagehover_phone_horizontal_space_around = get_option( 'csp_imagehover_phone_horizontal_space_around', '10' );
        $csp_imagehover_phone_horizontal_space_around_mu = get_option( 'csp_imagehover_phone_horizontal_space_around_mu', 'px' );
        $csp_imagehover_phone_vertical_space_around = get_option( 'csp_imagehover_phone_vertical_space_around', '10' );
        $csp_imagehover_phone_vertical_space_around_mu = get_option( 'csp_imagehover_phone_vertical_space_around_mu', 'px' );
        
        $csp_imagehover_desktop_position = get_option( 'csp_imagehover_desktop_position', 'center' );
        $csp_imagehover_phone_position = get_option( 'csp_imagehover_phone_position', 'bottomright' );

        $csp_desktop_position_css = 'position:fixed;';
        switch($csp_imagehover_desktop_position){
            case 'topleft':
                $csp_desktop_position_css .= 'top:0;left:0;';
            break;
            case 'topcenter':
                $csp_desktop_position_css .= 'top:0;left:50%;transform:translateX(-50%);';
            break;
            case 'topright':
                $csp_desktop_position_css .= 'top:0;right:0;left:auto;';
            break;
            case 'centerleft':
                $csp_desktop_position_css .= 'top:50%;transform:translateY(-50%);left:0;';
            break;
            case 'center':
                $csp_desktop_position_css .= 'top:50%;left:50%;transform:translate(-50%,-50%);';
            break;
            case 'centerright':
                $csp_desktop_position_css .= 'top:50%;transform:translateY(-50%);right:0;left:auto;';
            break;
            case 'bottomleft':
                $csp_desktop_position_css .= 'bottom:0;left:0;top:auto;';
            break;
            case 'bottomcenter':
                $csp_desktop_position_css .= 'bottom:0;left:50%;transform:translateX(-50%);top:auto;';
            break;
            case 'bottomright':
                $csp_desktop_position_css .= 'bottom:0;right:0;left:auto;top:auto;';
            break;
        }

        $csp_phone_position_css = 'position:fixed;';
        switch($csp_imagehover_phone_position){
            case 'topleft':
                $csp_phone_position_css .= 'top:0;left:0;';
            break;
            case 'topcenter':
                $csp_phone_position_css .= 'top:0;left:50%;transform:translateX(-50%);';
            break;
            case 'topright':
                $csp_phone_position_css .= 'top:0;right:0;left:auto;';
            break;
            case 'centerleft':
                $csp_phone_position_css .= 'top:50%;transform:translateY(-50%);left:0;';
            break;
            case 'center':
                $csp_phone_position_css .= 'top:50%;left:50%;transform:translate(-50%,-50%);';
            break;
            case 'centerright':
                $csp_phone_position_css .= 'top:50%;transform:translateY(-50%);right:0;left:auto;';
            break;
            case 'bottomleft':
                $csp_phone_position_css .= 'bottom:0;left:0;top:auto;';
            break;
            case 'bottomcenter':
                $csp_phone_position_css .= 'bottom:0;left:50%;transform:translateX(-50%);top:auto;';
            break;
            case 'bottomright':
                $csp_phone_position_css .= 'bottom:0;right:0;left:auto;top:auto;';
            break;
        }

        if( class_exists('MiscOptions') ) {
            $style .= 
            '@media (min-width: '.(MiscOptions::$phone_breakpoint+1).'px){
                .lay-imagehover-on-element[data-imageposition="custom"]{
                    '.$csp_desktop_position_css.'
                    box-sizing: content-box;
                    padding: '.$csp_imagehover_desktop_vertical_space_around.LayThemeImagehover::percToVwVh($csp_imagehover_desktop_vertical_space_around_mu, 'csp', 'verticalspace').' '.$csp_imagehover_desktop_horizontal_space_around.LayThemeImagehover::percToVwVh($csp_imagehover_desktop_horizontal_space_around_mu, 'csp', 'verticalspace').';
                }
            }
            @media (max-width: '.(MiscOptions::$phone_breakpoint).'px){
                .lay-imagehover-on-element[data-imageposition="custom"]{
                    '.$csp_phone_position_css.'
                    box-sizing: content-box;
                    padding: '.$csp_imagehover_phone_vertical_space_around.LayThemeImagehover::percToVwVh($csp_imagehover_phone_vertical_space_around_mu, 'csp', 'horizontalspace').' '.$csp_imagehover_phone_horizontal_space_around.LayThemeImagehover::percToVwVh($csp_imagehover_phone_horizontal_space_around_mu, 'csp', 'horizontalspace').';
                }
            }';
        }
        wp_add_inline_style( 'imagehover-style', $style);
	}

	public static function imagehover_scripts(){
		wp_enqueue_script( 'imagehover-app', LayThemeImagehover::$url."/frontend/assets/js/imagehover.plugin.min.js", array( 'jquery' ), LayThemeImagehover::$ver, true);
		
		$imagehover_imageposition = get_option( 'imagehover_imageposition', 'followmousex' );
		$desktop_max_width = get_option( 'imagehover_desktop_max_width', '400' );
		$desktop_max_height = get_option( 'imagehover_desktop_max_height', '400' );
		$tablet_max_width = get_option( 'imagehover_tablet_max_width', '300' );
		$tablet_max_height = get_option( 'imagehover_tablet_max_height', '300' );
		$phone_max_width = get_option( 'imagehover_phone_max_width', '100' );
		$phone_max_height = get_option( 'imagehover_phone_max_height', '100' );
		$position_horizontal = get_option( 'imagehover_position_horizontal', 'right' );
		$offset_vertical = get_option( 'imagehover_offset_vertical', '5' );

        $d = get_option( 'imagehover_offset_vertical', '5' );
        $mousecursor_offset_vertical = get_option( 'imagehover_mousecursor_offset_vertical', $d );

		$position_vertical = get_option( 'imagehover_position_vertical', 'top' );
		$transition = get_option( 'imagehover_transition', 'fade2' );
		$imagehover_adjustposition = get_option( 'imagehover_adjustposition', 'on' );
        $imagehover_donthideonmouseleave = get_option( 'imagehover_donthideonmouseleave', '' );

		wp_localize_script( 'imagehover-app', 'imagehoverPassedData',
			array(
				'imagehover_imageposition' => $imagehover_imageposition,
				'desktop_max_width' => $desktop_max_width,
				'desktop_max_height' => $desktop_max_height,
				'tablet_max_width' => $tablet_max_width,
				'tablet_max_height' => $tablet_max_height,
				'phone_max_width' => $phone_max_width,
				'phone_max_height' => $phone_max_height,
				'position_horizontal' => $position_horizontal,
				'offset_vertical' => $offset_vertical,
                'mousecursor_offset_vertical' => $mousecursor_offset_vertical,
				'position_vertical' => $position_vertical,
				'transition' => $transition,
				'imagehover_adjustposition' => $imagehover_adjustposition,
                'imagehover_donthideonmouseleave' => $imagehover_donthideonmouseleave,
                'imagehover_scrolltrigger' => get_option( 'imagehover_scrolltrigger', 'onhoveronly' ),
                'imagehover_imageposition_for_scroll' => get_option( 'imagehover_imageposition_for_scroll', 'centerfixed' ),
                'imagehover_scrolltrigger_y_desktop' => get_option( 'imagehover_scrolltrigger_y_desktop', '350' ),
                'imagehover_scrolltrigger_y_phone' => get_option( 'imagehover_scrolltrigger_y_phone', '250' ),
                'imagehover_clickable' => get_option( 'imagehover_clickable', '' )
			)
		);
	}

	public function add_action_links( $links ){
		$mylinks = array(
			'<a href="' . admin_url( 'admin.php?page=manage-layimagehover' ) . '">Settings</a>',
		);
		return array_merge( $links, $mylinks );
	}

	public function imagehover_settings_api_init(){
	 	add_settings_section(
			'imagehoveroptions_section',
			'',
			'',
			'manage-layimagehover'
		);

        add_settings_field(
			'imagehover_scrolltrigger',
			'Show Image on',
			array($this, 'imagehover_scrolltrigger_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_scrolltrigger' );

		add_settings_field(
			'imagehover_imageposition',
			'Image Position for show on hover',
			array($this, 'imagehover_imageposition_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_imageposition' );

        add_settings_field(
			'imagehover_imageposition_for_scroll',
			'Image Position for show on scroll',
			array($this, 'imagehover_imageposition_for_scroll_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_imageposition_for_scroll' );

        add_settings_field(
			'imagehover_scrolltrigger_y_desktop',
			'Show Image on Scroll Y Trigger Point Desktop',
			array($this, 'imagehover_scrolltrigger_y_desktop_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_scrolltrigger_y_desktop' );

        add_settings_field(
			'imagehover_scrolltrigger_y_phone',
			'Show Image on Scroll Y Trigger Point Phone',
			array($this, 'imagehover_scrolltrigger_y_phone_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_scrolltrigger_y_phone' );

		add_settings_field(
			'imagehover_adjustposition',
			'Adjust Image Position so it is always in Viewport',
			array($this, 'imagehover_adjustposition_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_adjustposition' );

		add_settings_field(
			'imagehover_zindex',
			'Desktop and Tablet: Put Image under/above other elements',
			array($this, 'imagehover_zindex_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_zindex' );

        add_settings_field(
			'imagehover_zindex_for_phone',
			'Phone: Put Image under/above other elements',
			array($this, 'imagehover_zindex_for_phone_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_zindex_for_phone' );

		add_settings_field(
			'imagehover_transition',
			'Transition',
			array($this, 'imagehover_transition_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_transition' );

		add_settings_field(
			'imagehover_position_horizontal',
			'Horizontal Image Position',
			array($this, 'imagehover_position_horizontal_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_position_horizontal' );

		add_settings_field(
			'imagehover_position_vertical',
			'Vertical Image Position',
			array($this, 'imagehover_position_vertical_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_position_vertical' );

		add_settings_field(
			'imagehover_offset_vertical',
			'Vertical Space between Text and Image',
			array($this, 'imagehover_offset_vertical_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_offset_vertical' );

        add_settings_field(
			'imagehover_mousecursor_offset_vertical',
			'Vertical Space between Mousecursor and Image',
			array($this, 'imagehover_mousecursor_offset_vertical_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_mousecursor_offset_vertical' );

		add_settings_section(
			'imagehoverwidthsandheights_section',
			'',
			'',
			'manage-layimagehover'
		);

        add_settings_field(
			'imagehover_donthideonmouseleave',
			'Don\'t hide image when mouse leaves link',
			array($this, 'imagehover_donthideonmouseleave_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_donthideonmouseleave' );

        add_settings_field(
			'imagehover_clickable',
			'Make hoverimages clickable (link)',
			array($this, 'imagehover_clickable_setting'),
			'manage-layimagehover',
			'imagehoveroptions_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_clickable' );


        // follow mouse x
		add_settings_field(
			'imagehover_desktop_max_width_mu',
			'Desktop Width in',
			array($this, 'imagehover_desktop_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
	   	register_setting( 'manage-layimagehover', 'imagehover_desktop_max_width_mu' );

 	 	add_settings_field(
 			'imagehover_desktop_max_width',
 			'Desktop Width (maximum)',
 			array($this, 'imagehover_desktop_max_width_setting'),
 			'manage-layimagehover',
 			'imagehoverwidthsandheights_section'
 		);
		register_setting( 'manage-layimagehover', 'imagehover_desktop_max_width' );


		add_settings_field(
			'imagehover_desktop_max_height_mu',
			'Desktop Height in',
			array($this, 'imagehover_desktop_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_desktop_max_height_mu' );

		add_settings_field(
			'imagehover_desktop_max_height',
			'Desktop Height (maximum)',
			array($this, 'imagehover_desktop_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_desktop_max_height' );

		add_settings_field(
			'imagehover_tablet_max_width_mu',
			'Tablet Width in',
			array($this, 'imagehover_tablet_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_tablet_max_width_mu' );

		add_settings_field(
			'imagehover_tablet_max_width',
			'Tablet Width (maximum)',
			array($this, 'imagehover_tablet_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_tablet_max_width' );

		add_settings_field(
			'imagehover_tablet_max_height_mu',
			'Tablet Height in',
			array($this, 'imagehover_tablet_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_tablet_max_height_mu' );

		add_settings_field(
			'imagehover_tablet_max_height',
			'Tablet Height (maximum)',
			array($this, 'imagehover_tablet_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_tablet_max_height' );

		add_settings_field(
			'imagehover_phone_max_width_mu',
			'Phone Width in',
			array($this, 'imagehover_phone_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_phone_max_width_mu' );

		add_settings_field(
			'imagehover_phone_max_width',
			'Phone Width (maximum)',
			array($this, 'imagehover_phone_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_phone_max_width' );

		add_settings_field(
			'imagehover_phone_max_height_mu',
			'Phone Height in',
			array($this, 'imagehover_phone_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_phone_max_height_mu' );

		add_settings_field(
			'imagehover_phone_max_height',
			'Phone Height (maximum)',
			array($this, 'imagehover_phone_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_phone_max_height' );

        // followmousexy
		add_settings_field(
			'followmousexy_imagehover_desktop_max_width_mu',
			'followmousexy Desktop Width in',
			array($this, 'followmousexy_imagehover_desktop_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
	   	register_setting( 'manage-layimagehover', 'followmousexy_imagehover_desktop_max_width_mu' );

 	 	add_settings_field(
 			'followmousexy_imagehover_desktop_max_width',
 			'followmousexy Desktop Width (maximum)',
 			array($this, 'followmousexy_imagehover_desktop_max_width_setting'),
 			'manage-layimagehover',
 			'imagehoverwidthsandheights_section'
 		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_desktop_max_width' );


		add_settings_field(
			'followmousexy_imagehover_desktop_max_height_mu',
			'followmousexy Desktop Height in',
			array($this, 'followmousexy_imagehover_desktop_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_desktop_max_height_mu' );

		add_settings_field(
			'followmousexy_imagehover_desktop_max_height',
			'followmousexy Desktop Height (maximum)',
			array($this, 'followmousexy_imagehover_desktop_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_desktop_max_height' );

		add_settings_field(
			'followmousexy_imagehover_tablet_max_width_mu',
			'followmousexy Tablet Width in',
			array($this, 'followmousexy_imagehover_tablet_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_tablet_max_width_mu' );

		add_settings_field(
			'followmousexy_imagehover_tablet_max_width',
			'followmousexy Tablet Width (maximum)',
			array($this, 'followmousexy_imagehover_tablet_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_tablet_max_width' );

		add_settings_field(
			'followmousexy_imagehover_tablet_max_height_mu',
			'followmousexy Tablet Height in',
			array($this, 'followmousexy_imagehover_tablet_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_tablet_max_height_mu' );

		add_settings_field(
			'followmousexy_imagehover_tablet_max_height',
			'followmousexy Tablet Height (maximum)',
			array($this, 'followmousexy_imagehover_tablet_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_tablet_max_height' );

		add_settings_field(
			'followmousexy_imagehover_phone_max_width_mu',
			'followmousexy Phone Width in',
			array($this, 'followmousexy_imagehover_phone_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_phone_max_width_mu' );

		add_settings_field(
			'followmousexy_imagehover_phone_max_width',
			'followmousexy Phone Width (maximum)',
			array($this, 'followmousexy_imagehover_phone_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_phone_max_width' );

		add_settings_field(
			'followmousexy_imagehover_phone_max_height_mu',
			'followmousexy Phone Height in',
			array($this, 'followmousexy_imagehover_phone_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_phone_max_height_mu' );

		add_settings_field(
			'followmousexy_imagehover_phone_max_height',
			'followmousexy Phone Height (maximum)',
			array($this, 'followmousexy_imagehover_phone_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'followmousexy_imagehover_phone_max_height' );


        // for stick to link
		add_settings_field(
			'sticktolink_imagehover_desktop_max_width_mu',
			'sticktolink Desktop Width in',
			array($this, 'sticktolink_imagehover_desktop_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
	   	register_setting( 'manage-layimagehover', 'sticktolink_imagehover_desktop_max_width_mu' );

 	 	add_settings_field(
 			'sticktolink_imagehover_desktop_max_width',
 			'sticktolink Desktop Width (maximum)',
 			array($this, 'sticktolink_imagehover_desktop_max_width_setting'),
 			'manage-layimagehover',
 			'imagehoverwidthsandheights_section'
 		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_desktop_max_width' );


		add_settings_field(
			'sticktolink_imagehover_desktop_max_height_mu',
			'sticktolink Desktop Height in',
			array($this, 'sticktolink_imagehover_desktop_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_desktop_max_height_mu' );

		add_settings_field(
			'sticktolink_imagehover_desktop_max_height',
			'sticktolink Desktop Height (maximum)',
			array($this, 'sticktolink_imagehover_desktop_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_desktop_max_height' );

		add_settings_field(
			'sticktolink_imagehover_tablet_max_width_mu',
			'sticktolink Tablet Width in',
			array($this, 'sticktolink_imagehover_tablet_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_tablet_max_width_mu' );

		add_settings_field(
			'sticktolink_imagehover_tablet_max_width',
			'sticktolink Tablet Width (maximum)',
			array($this, 'sticktolink_imagehover_tablet_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_tablet_max_width' );

		add_settings_field(
			'sticktolink_imagehover_tablet_max_height_mu',
			'sticktolink Tablet Height in',
			array($this, 'sticktolink_imagehover_tablet_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_tablet_max_height_mu' );

		add_settings_field(
			'sticktolink_imagehover_tablet_max_height',
			'sticktolink Tablet Height (maximum)',
			array($this, 'sticktolink_imagehover_tablet_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_tablet_max_height' );

		add_settings_field(
			'sticktolink_imagehover_phone_max_width_mu',
			'sticktolink Phone Width in',
			array($this, 'sticktolink_imagehover_phone_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_phone_max_width_mu' );

		add_settings_field(
			'sticktolink_imagehover_phone_max_width',
			'sticktolink Phone Width (maximum)',
			array($this, 'sticktolink_imagehover_phone_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_phone_max_width' );

		add_settings_field(
			'sticktolink_imagehover_phone_max_height_mu',
			'sticktolink Phone Height in',
			array($this, 'sticktolink_imagehover_phone_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_phone_max_height_mu' );

		add_settings_field(
			'sticktolink_imagehover_phone_max_height',
			'sticktolink Phone Height (maximum)',
			array($this, 'sticktolink_imagehover_phone_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'sticktolink_imagehover_phone_max_height' );

        
        // for fixed and centered
		add_settings_field(
			'centerfixed_imagehover_desktop_max_width_mu',
			'centerfixed Desktop Width in',
			array($this, 'centerfixed_imagehover_desktop_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
	   	register_setting( 'manage-layimagehover', 'centerfixed_imagehover_desktop_max_width_mu' );

 	 	add_settings_field(
 			'centerfixed_imagehover_desktop_max_width',
 			'centerfixed Desktop Width (maximum)',
 			array($this, 'centerfixed_imagehover_desktop_max_width_setting'),
 			'manage-layimagehover',
 			'imagehoverwidthsandheights_section'
 		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_desktop_max_width' );


		add_settings_field(
			'centerfixed_imagehover_desktop_max_height_mu',
			'centerfixed Desktop Height in',
			array($this, 'centerfixed_imagehover_desktop_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_desktop_max_height_mu' );

		add_settings_field(
			'centerfixed_imagehover_desktop_max_height',
			'centerfixed Desktop Height (maximum)',
			array($this, 'centerfixed_imagehover_desktop_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_desktop_max_height' );

		add_settings_field(
			'centerfixed_imagehover_tablet_max_width_mu',
			'centerfixed Tablet Width in',
			array($this, 'centerfixed_imagehover_tablet_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_tablet_max_width_mu' );

		add_settings_field(
			'centerfixed_imagehover_tablet_max_width',
			'centerfixed Tablet Width (maximum)',
			array($this, 'centerfixed_imagehover_tablet_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_tablet_max_width' );

		add_settings_field(
			'centerfixed_imagehover_tablet_max_height_mu',
			'centerfixed Tablet Height in',
			array($this, 'centerfixed_imagehover_tablet_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_tablet_max_height_mu' );

		add_settings_field(
			'centerfixed_imagehover_tablet_max_height',
			'centerfixed Tablet Height (maximum)',
			array($this, 'centerfixed_imagehover_tablet_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_tablet_max_height' );

		add_settings_field(
			'centerfixed_imagehover_phone_max_width_mu',
			'centerfixed Phone Width in',
			array($this, 'centerfixed_imagehover_phone_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_phone_max_width_mu' );

		add_settings_field(
			'centerfixed_imagehover_phone_max_width',
			'centerfixed Phone Width (maximum)',
			array($this, 'centerfixed_imagehover_phone_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_phone_max_width' );

		add_settings_field(
			'centerfixed_imagehover_phone_max_height_mu',
			'centerfixed Phone Height in',
			array($this, 'centerfixed_imagehover_phone_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_phone_max_height_mu' );

		add_settings_field(
			'centerfixed_imagehover_phone_max_height',
			'centerfixed Phone Height (maximum)',
			array($this, 'centerfixed_imagehover_phone_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'centerfixed_imagehover_phone_max_height' );

        // for custom size and position
		add_settings_field(
			'csp_imagehover_desktop_max_width_mu',
			'csp Desktop Width in',
			array($this, 'csp_imagehover_desktop_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
	   	register_setting( 'manage-layimagehover', 'csp_imagehover_desktop_max_width_mu' );

 	 	add_settings_field(
 			'csp_imagehover_desktop_max_width',
 			'csp Desktop Width (maximum)',
 			array($this, 'csp_imagehover_desktop_max_width_setting'),
 			'manage-layimagehover',
 			'imagehoverwidthsandheights_section'
 		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_desktop_max_width' );


		add_settings_field(
			'csp_imagehover_desktop_max_height_mu',
			'csp Desktop Height in',
			array($this, 'csp_imagehover_desktop_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_desktop_max_height_mu' );

		add_settings_field(
			'csp_imagehover_desktop_max_height',
			'csp Desktop Height (maximum)',
			array($this, 'csp_imagehover_desktop_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_desktop_max_height' );

		add_settings_field(
			'csp_imagehover_tablet_max_width_mu',
			'csp Tablet Width in',
			array($this, 'csp_imagehover_tablet_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_tablet_max_width_mu' );

		add_settings_field(
			'csp_imagehover_tablet_max_width',
			'csp Tablet Width (maximum)',
			array($this, 'csp_imagehover_tablet_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_tablet_max_width' );

		add_settings_field(
			'csp_imagehover_tablet_max_height_mu',
			'csp Tablet Height in',
			array($this, 'csp_imagehover_tablet_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_tablet_max_height_mu' );

		add_settings_field(
			'csp_imagehover_tablet_max_height',
			'csp Tablet Height (maximum)',
			array($this, 'csp_imagehover_tablet_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_tablet_max_height' );

		add_settings_field(
			'csp_imagehover_phone_max_width_mu',
			'csp Phone Width in',
			array($this, 'csp_imagehover_phone_max_width_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_phone_max_width_mu' );

		add_settings_field(
			'csp_imagehover_phone_max_width',
			'csp Phone Width (maximum)',
			array($this, 'csp_imagehover_phone_max_width_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_phone_max_width' );

		add_settings_field(
			'csp_imagehover_phone_max_height_mu',
			'csp Phone Height in',
			array($this, 'csp_imagehover_phone_max_height_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_phone_max_height_mu' );

		add_settings_field(
			'csp_imagehover_phone_max_height',
			'csp Phone Height (maximum)',
			array($this, 'csp_imagehover_phone_max_height_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_phone_max_height' );

        // csp horizontal space around
		add_settings_field(
			'csp_imagehover_desktop_horizontal_space_around',
			'csp Horizontal Desktop Space Around',
			array($this, 'csp_imagehover_desktop_horizontal_space_around_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_desktop_horizontal_space_around' );

        add_settings_field(
			'csp_imagehover_desktop_horizontal_space_around_mu',
			'csp Horizontal Desktop Space Around in',
			array($this, 'csp_imagehover_desktop_horizontal_space_around_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_desktop_horizontal_space_around_mu' );

        add_settings_field(
			'csp_imagehover_phone_horizontal_space_around',
			'csp Horizontal Phone Space Around',
			array($this, 'csp_imagehover_phone_horizontal_space_around_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_phone_horizontal_space_around' );

        add_settings_field(
			'csp_imagehover_phone_horizontal_space_around_mu',
			'csp Horizontal Phone Space Around in',
			array($this, 'csp_imagehover_phone_horizontal_space_around_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_phone_horizontal_space_around_mu' );

        // csp vertical space around
        add_settings_field(
			'csp_imagehover_desktop_vertical_space_around',
			'csp vertical Desktop Space Around',
			array($this, 'csp_imagehover_desktop_vertical_space_around_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_desktop_vertical_space_around' );

        add_settings_field(
			'csp_imagehover_desktop_vertical_space_around_mu',
			'csp vertical Desktop Space Around in',
			array($this, 'csp_imagehover_desktop_vertical_space_around_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_desktop_vertical_space_around_mu' );

        add_settings_field(
			'csp_imagehover_phone_vertical_space_around',
			'csp vertical Phone Space Around',
			array($this, 'csp_imagehover_phone_vertical_space_around_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_phone_vertical_space_around' );

        add_settings_field(
			'csp_imagehover_phone_vertical_space_around_mu',
			'csp vertical Phone Space Around in',
			array($this, 'csp_imagehover_phone_vertical_space_around_mu_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_phone_vertical_space_around_mu' );

        // csp position
        add_settings_field(
			'csp_imagehover_desktop_position',
			'csp desktop position',
			array($this, 'csp_imagehover_desktop_position_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_desktop_position' );

        add_settings_field(
			'csp_imagehover_phone_position',
			'csp phone position',
			array($this, 'csp_imagehover_phone_position_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'csp_imagehover_phone_position' );

        // which tab was selected :)
        add_settings_field(
			'imagehover_widths_and_height_selected_tab',
			'imagehover widths and heights - selected tab',
			array($this, 'imagehover_widths_and_height_selected_tab_setting'),
			'manage-layimagehover',
			'imagehoverwidthsandheights_section'
		);
		register_setting( 'manage-layimagehover', 'imagehover_widths_and_height_selected_tab' );
	}

    public function imagehover_clickable_setting(){
        $val = get_option( 'imagehover_clickable', '' );
        $checked = "";
        if( $val == "on" ){
            $checked = "checked";
        }
        echo '<input type="checkbox" name="imagehover_clickable" id="imagehover_clickable" '.$checked.'><label for="imagehover_clickable"></label>
        <p>This is useful for when you use "Show image on scroll" for "Show Image on". Doesn\'t work for show on hover.</p>';
    }

    public function imagehover_donthideonmouseleave_setting(){
        $val = get_option( 'imagehover_donthideonmouseleave', '' );
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="imagehover_donthideonmouseleave" id="imagehover_donthideonmouseleave" '.$checked.'><label for="imagehover_donthideonmouseleave"></label>'; 
    }

	public function imagehover_position_horizontal_setting(){
		$val = get_option( 'imagehover_position_horizontal', 'right' );

		$selectedLeft = '';
		$selectedLeftInside = '';
		$selectedCenter = '';
		$selectedRight = '';
		$selectedRightInside = '';
        $selectedCenterLeft = '';
        $selectedCenterRight = '';

		switch ($val) {
            case 'left-inside':
				$selectedLeftInside = 'selected';
			break;
			case 'left':
				$selectedLeft = 'selected';
			break;
			case 'center-left':
				$selectedCenterLeft = 'selected';
			break;
            case 'center':
				$selectedCenter = 'selected';
			break;
            case 'center-right':
				$selectedCenterRight = 'selected';
			break;
			case 'right':
				$selectedRight = 'selected';
			break;
			case 'right-inside':
				$selectedRightInside = 'selected';
			break;
		}

		echo 
		'<select name="imagehover_position_horizontal" id="imagehover_position_horizontal">
			<option value="left" '.$selectedLeft.'>Left outside</option>
			<option value="left-inside" '.$selectedLeftInside.'>Left inside</option>
            <option value="center-left" '.$selectedCenterLeft.'>Center left edge</option>
			<option value="center" '.$selectedCenter.'>Center</option>
            <option value="center-right" '.$selectedCenterRight.'>Center right edge</option>
			<option value="right" '.$selectedRight.'>Right outside</option>
			<option value="right-inside" '.$selectedRightInside.'>Right inside</option>
		</select>
		<p>Some of these settings only have an effect if "Image Position" is "Stick to Link".</p>';  		
	}
	// todo: hide this control if vertical image position is center
	public function imagehover_offset_vertical_setting(){
		$val = get_option( 'imagehover_offset_vertical', '5' );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_offset_vertical" id="imagehover_offset_vertical" value="'.$val.'"> <label for="imagehover_offset_vertical">px</label>
        <p>for "Follow Mouse X", "Stick to Link"</p>';
	}

    public function imagehover_mousecursor_offset_vertical_setting(){
        $d = get_option( 'imagehover_offset_vertical', '5' );
        $val = get_option( 'imagehover_mousecursor_offset_vertical', $d );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_mousecursor_offset_vertical" id="imagehover_mousecursor_offset_vertical" value="'.$val.'"> <label for="imagehover_mousecursor_offset_vertical">px</label>
        <p>for "Follow Mouse XY"</p>';
    }

	public function imagehover_position_vertical_setting(){
		$val = get_option( 'imagehover_position_vertical', 'top' );

		$selectedTop = '';
		$selectedCenter = '';
		$selectedBottom = '';

		switch ($val) {
			case 'top':
				$selectedTop = 'selected';
			break;
			case 'center':
				$selectedCenter = 'selected';
			break;
			case 'bottom':
				$selectedBottom = 'selected';
			break;
		}

		echo 
		'<select name="imagehover_position_vertical" id="imagehover_position_vertical">
			<option value="top" '.$selectedTop.'>Top</option>
			<option value="center" '.$selectedCenter.'>Center</option>
			<option value="bottom" '.$selectedBottom.'>Bottom</option>
		</select>'; 
	}

	public function imagehover_transition_setting(){
		$val = get_option( 'imagehover_transition', 'fade2' );

		$selectedFade = '';
		$selectedFade2 = '';
		$selectedImmediate = '';
        $selectedBlur = '';
        $selectedZoom = '';
        $selectedClip = '';

		switch ($val) {
			case 'immediate':
				$selectedImmediate = 'selected';
			break;
			case 'fade':
				$selectedFade = 'selected';
			break;
			case 'fade2':
				$selectedFade2 = 'selected';
			break;
            case 'blur':
                $selectedBlur = 'selected';
            break;
            case 'zoom':
                $selectedZoom = 'selected';
            break;
            case 'clip':
                $selectedClip = 'selected';
            break;
		}

		echo 
		'<select name="imagehover_transition" id="imagehover_transition">
			<option value="fade2" '.$selectedFade2.'>Fade in at first, but when switching between images quickly don\'t fade</option>
			<option value="fade" '.$selectedFade.'>Fade</option>
			<option value="immediate" '.$selectedImmediate.'>Immediate</option>
            <option value="zoom" '.$selectedZoom.'>Zoom</option>
            <option value="blur" '.$selectedBlur.'>Blur</option>
            <option value="clip" '.$selectedClip.'>Clip</option>
		</select>';
	}

    public function imagehover_zindex_for_phone_setting(){
        $val = get_option( 'imagehover_zindex_for_phone', 'under-all' );

		$selected2 = '';
		$selected3 = '';

		switch ($val) {
            case '':
                $selected2 = 'selected';
            break;
            case 'under-all':
                $selected3 = 'selected';
            break;
		}

        echo 
        '<select name="imagehover_zindex_for_phone" id="imagehover_zindex_for_phone">
            <option value="" '.$selected2.'>Put images above all elements</option>
            <option value="under-all" '.$selected3.'>Put images under all elements</option>
        </select>';
    }

	public function imagehover_zindex_setting(){
		$val = get_option( 'imagehover_zindex', 'under-all' );

		$selected2 = '';
		$selected3 = '';

		switch ($val) {
            case '':
                $selected2 = 'selected';
            break;
            case 'under-all':
                $selected3 = 'selected';
            break;
		}

        echo 
        '<select name="imagehover_zindex" id="imagehover_zindex">
            <option value="" '.$selected2.'>Put images above all elements</option>
            <option value="under-all" '.$selected3.'>Put images under all elements</option>
        </select>';        
	}

	public function imagehover_adjustposition_setting(){
		$val = get_option( 'imagehover_adjustposition', 'on' );
		$checked = "";
		if( $val == "on" ){
			$checked = "checked";
		}
		echo '<input type="checkbox" name="imagehover_adjustposition" id="imagehover_adjustposition" '.$checked.'><label for="imagehover_adjustposition">(If your image width and height settings are too large for the browser dimensions this won\'t work.)</label>'; 
	}

    public function imagehover_scrolltrigger_setting(){

        $val = get_option( 'imagehover_scrolltrigger', 'onhoveronly' );
		$onhoveronly_selected = '';
		$onscrollonly_selected = '';
		$onscrollandhover_selected = '';
		$onscrollfortouchdevices_onhoverfordesktop_selected = '';

		switch( $val ) {
			case 'onhoveronly':
				$onhoveronly_selected = 'selected';
			break;
			case 'onscrollonly':
				$onscrollonly_selected = 'selected';
			break;
			case 'onscrollandhover':
				$onscrollandhover_selected = 'selected';
			break;
            case 'onscrollfortouchdevices_onhoverfordesktop':
				$onscrollfortouchdevices_onhoverfordesktop_selected = 'selected';
			break;
		}
		echo 
		'<select name="imagehover_scrolltrigger" id="imagehover_scrolltrigger">
            <option value="onhoveronly" '.$onhoveronly_selected.'>Show image on hover only</option>
            <option value="onscrollonly" '.$onscrollonly_selected.'>Show image on scroll only</option>
			<option value="onscrollandhover" '.$onscrollandhover_selected.'>Show image on scroll and hover</option>
			<option value="onscrollfortouchdevices_onhoverfordesktop" '.$onscrollfortouchdevices_onhoverfordesktop_selected.'>Show image on scroll for touch devices and on hover for desktop</option>
		</select> ';
    }

    public function imagehover_imageposition_for_scroll_setting(){
		$val = get_option( 'imagehover_imageposition_for_scroll', 'centerfixed' );

		$centerfixed_selected = '';
		$sticktolink_selected = '';
		$leftbackground_selected = '';
		$rightbackground_selected = '';
		$fullsize_selected = '';
        $custom_selected = '';

		switch( $val ) {
			case 'centerfixed':
				$centerfixed_selected = 'selected';
			break;
			case 'sticktolink':
				$sticktolink_selected = 'selected';
			break;
			case 'leftbackground':
				$leftbackground_selected = 'selected';
			break;
			case 'rightbackground':
				$rightbackground_selected = 'selected';
			break;
			case 'fullsize':
				$fullsize_selected = 'selected';
			break;
            case 'custom':
				$custom_selected = 'selected';
			break;
		}
		echo 
		'<select name="imagehover_imageposition_for_scroll" id="imagehover_imageposition_for_scroll">
			<option value="sticktolink" '.$sticktolink_selected.'>Stick to Link</option>
			<option value="centerfixed" '.$centerfixed_selected.'>Fixed and Centered</option>
			<option value="leftbackground" '.$leftbackground_selected.'>Left Background</option>
			<option value="rightbackground" '.$rightbackground_selected.'>Right Background</option>
			<option value="fullsize" '.$fullsize_selected.'>Full Size</option>
            <option value="custom" '.$custom_selected.'>Custom Size and Position</option>
		</select> <br/>Set widths and heights →'; 	
    }

	public function imagehover_imageposition_setting(){
		$val = get_option( 'imagehover_imageposition', 'followmousex' );
		$followmousex_selected = '';
		$followmousexy_selected = '';
		$centerfixed_selected = '';
		$sticktolink_selected = '';
		$leftbackground_selected = '';
		$rightbackground_selected = '';
		$fullsize_selected = '';
        $custom_selected = '';

		switch( $val ) {
			case 'followmousex':
				$followmousex_selected = 'selected';
			break;
			case 'followmousexy':
				$followmousexy_selected = 'selected';
			break;
			case 'centerfixed':
				$centerfixed_selected = 'selected';
			break;
			case 'sticktolink':
				$sticktolink_selected = 'selected';
			break;
			case 'leftbackground':
				$leftbackground_selected = 'selected';
			break;
			case 'rightbackground':
				$rightbackground_selected = 'selected';
			break;
			case 'fullsize':
				$fullsize_selected = 'selected';
			break;
            case 'custom':
                $custom_selected = 'selected';
            break;
		}
		echo 
		'<select name="imagehover_imageposition" id="imagehover_imageposition">
			<option value="followmousex" '.$followmousex_selected.'>Follow Mouse X</option>
			<option value="followmousexy" '.$followmousexy_selected.'>Follow Mouse XY</option>
			<option value="sticktolink" '.$sticktolink_selected.'>Stick to Link</option>
			<option value="centerfixed" '.$centerfixed_selected.'>Fixed and Centered</option>
			<option value="leftbackground" '.$leftbackground_selected.'>Left Background</option>
			<option value="rightbackground" '.$rightbackground_selected.'>Right Background</option>
			<option value="fullsize" '.$fullsize_selected.'>Full Size</option>
            <option value="custom" '.$custom_selected.'>Custom Size and Position</option>
		</select> <br/>Set widths and heights →'; 	
	}

	public function imagehover_setup_menu(){
		add_submenu_page( 'manage-layoptions', 'Imagehover Addon', 'Imagehover Addon', 'manage_options', 'manage-layimagehover', array($this, 'layimagehover_options_markup') );
	}

	public function layimagehover_options_markup(){
		require_once( LayThemeImagehover::$dir.'/backend/imagehover_options_markup.php' );
	}

    // follow mousex:
    public function imagehover_desktop_max_width_mu_setting(){
		$val = get_option( 'imagehover_desktop_max_width_mu', 'px' );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="imagehover_desktop_max_width_mu" id="imagehover_desktop_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function imagehover_desktop_max_width_setting(){
		$val = get_option( 'imagehover_desktop_max_width', '400' );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_desktop_max_width" id="imagehover_desktop_max_width" value="'.$val.'">';
	}

    public function imagehover_scrolltrigger_y_desktop_setting(){
		$val = get_option( 'imagehover_scrolltrigger_y_desktop', '350' );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_scrolltrigger_y_desktop" id="imagehover_scrolltrigger_y_desktop" value="'.$val.'"> px';
	}

    public function imagehover_scrolltrigger_y_phone_setting(){
		$val = get_option( 'imagehover_scrolltrigger_y_phone', '250' );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_scrolltrigger_y_phone" id="imagehover_scrolltrigger_y_phone" value="'.$val.'"> px';
	}

	public function imagehover_desktop_max_height_mu_setting(){
		$val = get_option( 'imagehover_desktop_max_height_mu', 'px' );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="imagehover_desktop_max_height_mu" id="imagehover_desktop_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function imagehover_desktop_max_height_setting(){
		$val = get_option( 'imagehover_desktop_max_height', '400' );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_desktop_max_height" id="imagehover_desktop_max_height" value="'.$val.'">';
	}

	public function imagehover_tablet_max_width_mu_setting(){
		$val = get_option( 'imagehover_tablet_max_width_mu', 'px' );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="imagehover_tablet_max_width_mu" id="imagehover_tablet_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function imagehover_tablet_max_width_setting(){
		$val = get_option( 'imagehover_tablet_max_width', '300' );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_tablet_max_width" id="imagehover_tablet_max_width" value="'.$val.'">';
	}

	public function imagehover_tablet_max_height_mu_setting(){
		$val = get_option( 'imagehover_tablet_max_height_mu', 'px' );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="imagehover_tablet_max_height_mu" id="imagehover_tablet_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function imagehover_tablet_max_height_setting(){
		$val = get_option( 'imagehover_tablet_max_height', '300' );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_tablet_max_height" id="imagehover_tablet_max_height" value="'.$val.'">';
	}

	public function imagehover_phone_max_width_mu_setting(){
		$val = get_option( 'imagehover_phone_max_width_mu', 'px' );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="imagehover_phone_max_width_mu" id="imagehover_phone_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function imagehover_phone_max_width_setting(){
		$val = get_option( 'imagehover_phone_max_width', '100' );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_phone_max_width" id="imagehover_phone_max_width" value="'.$val.'">';
	}

	public function imagehover_phone_max_height_mu_setting(){
		$val = get_option( 'imagehover_phone_max_height_mu', 'px' );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="imagehover_phone_max_height_mu" id="imagehover_phone_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function imagehover_phone_max_height_setting(){
		$val = get_option( 'imagehover_phone_max_height', '100' );
		echo '<input type="number" min="0" max="" step="1" name="imagehover_phone_max_height" id="imagehover_phone_max_height" value="'.$val.'">';
	}

    // follow mousexy
    public function followmousexy_imagehover_desktop_max_width_mu_setting(){
        $d = get_option( 'imagehover_desktop_max_width_mu', 'px' );
		$val = get_option( 'followmousexy_imagehover_desktop_max_width_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="followmousexy_imagehover_desktop_max_width_mu" id="followmousexy_imagehover_desktop_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function followmousexy_imagehover_desktop_max_width_setting(){
        $d = get_option( 'imagehover_desktop_max_width', '400' );
		$val = get_option( 'followmousexy_imagehover_desktop_max_width', $d );
		echo '<input type="number" min="0" max="" step="1" name="followmousexy_imagehover_desktop_max_width" id="followmousexy_imagehover_desktop_max_width" value="'.$val.'">';
	}

	public function followmousexy_imagehover_desktop_max_height_mu_setting(){
        $d = get_option( 'imagehover_desktop_max_height_mu', 'px' );
		$val = get_option( 'followmousexy_imagehover_desktop_max_height_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="followmousexy_imagehover_desktop_max_height_mu" id="followmousexy_imagehover_desktop_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function followmousexy_imagehover_desktop_max_height_setting(){
        $d = get_option( 'imagehover_desktop_max_height', '400' );
		$val = get_option( 'followmousexy_imagehover_desktop_max_height', $d );
		echo '<input type="number" min="0" max="" step="1" name="followmousexy_imagehover_desktop_max_height" id="followmousexy_imagehover_desktop_max_height" value="'.$val.'">';
	}

	public function followmousexy_imagehover_tablet_max_width_mu_setting(){
        $d = get_option( 'imagehover_tablet_max_width_mu', 'px' );
		$val = get_option( 'followmousexy_imagehover_tablet_max_width_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="followmousexy_imagehover_tablet_max_width_mu" id="followmousexy_imagehover_tablet_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function followmousexy_imagehover_tablet_max_width_setting(){
        $d = get_option( 'imagehover_tablet_max_width', '300' );
		$val = get_option( 'followmousexy_imagehover_tablet_max_width', $d );
		echo '<input type="number" min="0" max="" step="1" name="followmousexy_imagehover_tablet_max_width" id="followmousexy_imagehover_tablet_max_width" value="'.$val.'">';
	}

	public function followmousexy_imagehover_tablet_max_height_mu_setting(){
        $d = get_option( 'imagehover_tablet_max_height_mu', 'px' );
		$val = get_option( 'followmousexy_imagehover_tablet_max_height_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="followmousexy_imagehover_tablet_max_height_mu" id="followmousexy_imagehover_tablet_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function followmousexy_imagehover_tablet_max_height_setting(){
        $d = get_option( 'imagehover_tablet_max_height', '300' );
		$val = get_option( 'followmousexy_imagehover_tablet_max_height', $d );
		echo '<input type="number" min="0" max="" step="1" name="followmousexy_imagehover_tablet_max_height" id="followmousexy_imagehover_tablet_max_height" value="'.$val.'">';
	}

	public function followmousexy_imagehover_phone_max_width_mu_setting(){
        $d = get_option( 'imagehover_phone_max_width_mu', 'px' );
		$val = get_option( 'followmousexy_imagehover_phone_max_width_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="followmousexy_imagehover_phone_max_width_mu" id="followmousexy_imagehover_phone_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function followmousexy_imagehover_phone_max_width_setting(){
        $d = get_option( 'imagehover_phone_max_width', '100' );
		$val = get_option( 'followmousexy_imagehover_phone_max_width', $d );
		echo '<input type="number" min="0" max="" step="1" name="followmousexy_imagehover_phone_max_width" id="followmousexy_imagehover_phone_max_width" value="'.$val.'">';
	}

	public function followmousexy_imagehover_phone_max_height_mu_setting(){
        $d = get_option( 'imagehover_phone_max_height_mu', 'px' );
		$val = get_option( 'followmousexy_imagehover_phone_max_height_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="followmousexy_imagehover_phone_max_height_mu" id="followmousexy_imagehover_phone_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function followmousexy_imagehover_phone_max_height_setting(){
        $d = get_option( 'imagehover_phone_max_height', '100' );
		$val = get_option( 'followmousexy_imagehover_phone_max_height', $d );
		echo '<input type="number" min="0" max="" step="1" name="followmousexy_imagehover_phone_max_height" id="followmousexy_imagehover_phone_max_height" value="'.$val.'">';
	}

    // for sticktolink
    public function sticktolink_imagehover_desktop_max_width_mu_setting(){
        $d = get_option( 'imagehover_desktop_max_width_mu', 'px' );
		$val = get_option( 'sticktolink_imagehover_desktop_max_width_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="sticktolink_imagehover_desktop_max_width_mu" id="sticktolink_imagehover_desktop_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function sticktolink_imagehover_desktop_max_width_setting(){
        $d = get_option( 'imagehover_desktop_max_width', '400' );
		$val = get_option( 'sticktolink_imagehover_desktop_max_width', $d );
		echo '<input type="number" min="0" max="" step="1" name="sticktolink_imagehover_desktop_max_width" id="sticktolink_imagehover_desktop_max_width" value="'.$val.'">';
	}

	public function sticktolink_imagehover_desktop_max_height_mu_setting(){
        $d = get_option( 'imagehover_desktop_max_height_mu', 'px' );
		$val = get_option( 'sticktolink_imagehover_desktop_max_height_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="sticktolink_imagehover_desktop_max_height_mu" id="sticktolink_imagehover_desktop_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function sticktolink_imagehover_desktop_max_height_setting(){
        $d = get_option( 'imagehover_desktop_max_height', '400' );
		$val = get_option( 'sticktolink_imagehover_desktop_max_height', $d );
		echo '<input type="number" min="0" max="" step="1" name="sticktolink_imagehover_desktop_max_height" id="sticktolink_imagehover_desktop_max_height" value="'.$val.'">';
	}

	public function sticktolink_imagehover_tablet_max_width_mu_setting(){
        $d = get_option( 'imagehover_tablet_max_width_mu', 'px' );
		$val = get_option( 'sticktolink_imagehover_tablet_max_width_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="sticktolink_imagehover_tablet_max_width_mu" id="sticktolink_imagehover_tablet_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function sticktolink_imagehover_tablet_max_width_setting(){
        $d = get_option( 'imagehover_tablet_max_width', '300' );
		$val = get_option( 'sticktolink_imagehover_tablet_max_width', $d );
		echo '<input type="number" min="0" max="" step="1" name="sticktolink_imagehover_tablet_max_width" id="sticktolink_imagehover_tablet_max_width" value="'.$val.'">';
	}

	public function sticktolink_imagehover_tablet_max_height_mu_setting(){
        $d = get_option( 'imagehover_tablet_max_height_mu', 'px' );
		$val = get_option( 'sticktolink_imagehover_tablet_max_height_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="sticktolink_imagehover_tablet_max_height_mu" id="sticktolink_imagehover_tablet_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function sticktolink_imagehover_tablet_max_height_setting(){
        $d = get_option( 'imagehover_tablet_max_height', '300' );
		$val = get_option( 'sticktolink_imagehover_tablet_max_height', $d );
		echo '<input type="number" min="0" max="" step="1" name="sticktolink_imagehover_tablet_max_height" id="sticktolink_imagehover_tablet_max_height" value="'.$val.'">';
	}

	public function sticktolink_imagehover_phone_max_width_mu_setting(){
        $d = get_option( 'imagehover_phone_max_width_mu', 'px' );
		$val = get_option( 'sticktolink_imagehover_phone_max_width_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="sticktolink_imagehover_phone_max_width_mu" id="sticktolink_imagehover_phone_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function sticktolink_imagehover_phone_max_width_setting(){
        $d = get_option( 'imagehover_phone_max_width', '100' );
		$val = get_option( 'sticktolink_imagehover_phone_max_width', $d );
		echo '<input type="number" min="0" max="" step="1" name="sticktolink_imagehover_phone_max_width" id="sticktolink_imagehover_phone_max_width" value="'.$val.'">';
	}

	public function sticktolink_imagehover_phone_max_height_mu_setting(){
        $d = get_option( 'imagehover_phone_max_height_mu', 'px' );
		$val = get_option( 'sticktolink_imagehover_phone_max_height_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="sticktolink_imagehover_phone_max_height_mu" id="sticktolink_imagehover_phone_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function sticktolink_imagehover_phone_max_height_setting(){
        $d = get_option( 'imagehover_phone_max_height', '100' );
		$val = get_option( 'sticktolink_imagehover_phone_max_height', $d );
		echo '<input type="number" min="0" max="" step="1" name="sticktolink_imagehover_phone_max_height" id="sticktolink_imagehover_phone_max_height" value="'.$val.'">';
	}

    // for fixed and centered
    public function centerfixed_imagehover_desktop_max_width_mu_setting(){
        $d = get_option( 'imagehover_desktop_max_width_mu', 'px' );
		$val = get_option( 'centerfixed_imagehover_desktop_max_width_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="centerfixed_imagehover_desktop_max_width_mu" id="centerfixed_imagehover_desktop_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function centerfixed_imagehover_desktop_max_width_setting(){
        $d = get_option( 'imagehover_desktop_max_width', '400' );
		$val = get_option( 'centerfixed_imagehover_desktop_max_width', $d );
		echo '<input type="number" min="0" max="" step="1" name="centerfixed_imagehover_desktop_max_width" id="centerfixed_imagehover_desktop_max_width" value="'.$val.'">';
	}

	public function centerfixed_imagehover_desktop_max_height_mu_setting(){
        $d = get_option( 'imagehover_desktop_max_height_mu', 'px' );
		$val = get_option( 'centerfixed_imagehover_desktop_max_height_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="centerfixed_imagehover_desktop_max_height_mu" id="centerfixed_imagehover_desktop_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function centerfixed_imagehover_desktop_max_height_setting(){
        $d = get_option( 'imagehover_desktop_max_height', '400' );
		$val = get_option( 'centerfixed_imagehover_desktop_max_height', $d );
		echo '<input type="number" min="0" max="" step="1" name="centerfixed_imagehover_desktop_max_height" id="centerfixed_imagehover_desktop_max_height" value="'.$val.'">';
	}

	public function centerfixed_imagehover_tablet_max_width_mu_setting(){
        $d = get_option( 'imagehover_tablet_max_width_mu', 'px' );
		$val = get_option( 'centerfixed_imagehover_tablet_max_width_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="centerfixed_imagehover_tablet_max_width_mu" id="centerfixed_imagehover_tablet_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function centerfixed_imagehover_tablet_max_width_setting(){
        $d = get_option( 'imagehover_tablet_max_width', '300' );
		$val = get_option( 'centerfixed_imagehover_tablet_max_width', $d );
		echo '<input type="number" min="0" max="" step="1" name="centerfixed_imagehover_tablet_max_width" id="centerfixed_imagehover_tablet_max_width" value="'.$val.'">';
	}

	public function centerfixed_imagehover_tablet_max_height_mu_setting(){
        $d = get_option( 'imagehover_tablet_max_height_mu', 'px' );
		$val = get_option( 'centerfixed_imagehover_tablet_max_height_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="centerfixed_imagehover_tablet_max_height_mu" id="centerfixed_imagehover_tablet_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function centerfixed_imagehover_tablet_max_height_setting(){
        $d = get_option( 'imagehover_tablet_max_height', '300' );
		$val = get_option( 'centerfixed_imagehover_tablet_max_height', $d );
		echo '<input type="number" min="0" max="" step="1" name="centerfixed_imagehover_tablet_max_height" id="centerfixed_imagehover_tablet_max_height" value="'.$val.'">';
	}

	public function centerfixed_imagehover_phone_max_width_mu_setting(){
        $d = get_option( 'imagehover_phone_max_width_mu', 'px' );
		$val = get_option( 'centerfixed_imagehover_phone_max_width_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="centerfixed_imagehover_phone_max_width_mu" id="centerfixed_imagehover_phone_max_width_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function centerfixed_imagehover_phone_max_width_setting(){
        $d = get_option( 'imagehover_phone_max_width', '100' );
		$val = get_option( 'centerfixed_imagehover_phone_max_width', $d );
		echo '<input type="number" min="0" max="" step="1" name="centerfixed_imagehover_phone_max_width" id="centerfixed_imagehover_phone_max_width" value="'.$val.'">';
	}

	public function centerfixed_imagehover_phone_max_height_mu_setting(){
        $d = get_option( 'imagehover_phone_max_height_mu', 'px' );
		$val = get_option( 'centerfixed_imagehover_phone_max_height_mu', $d );
		$selected_px = '';
		$selected_percent = '';
		switch( $val ) {
			case 'px':
				$selected_px = 'selected';
			break;
			case '%':
				$selected_percent = 'selected';
			break;
		}
		echo 
		'<select name="centerfixed_imagehover_phone_max_height_mu" id="centerfixed_imagehover_phone_max_height_mu">
			<option value="px" '.$selected_px.'>px</option>
			<option value="%" '.$selected_percent.'>%</option>
		</select>'; 
	}
	public function centerfixed_imagehover_phone_max_height_setting(){
        $d = get_option( 'imagehover_phone_max_height', '100' );
		$val = get_option( 'centerfixed_imagehover_phone_max_height', $d );
		echo '<input type="number" min="0" max="" step="1" name="centerfixed_imagehover_phone_max_height" id="centerfixed_imagehover_phone_max_height" value="'.$val.'">';
	}

    // for custom position and size
    public function csp_imagehover_desktop_max_width_mu_setting(){
        $d = get_option( 'imagehover_desktop_max_width_mu', 'px' );
        $val = get_option( 'csp_imagehover_desktop_max_width_mu', $d );
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_desktop_max_width_mu" id="csp_imagehover_desktop_max_width_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }
    public function csp_imagehover_desktop_max_width_setting(){
        $d = get_option( 'imagehover_desktop_max_width', '400' );
        $val = get_option( 'csp_imagehover_desktop_max_width', $d );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_desktop_max_width" id="csp_imagehover_desktop_max_width" value="'.$val.'">';
    }

    public function csp_imagehover_desktop_max_height_mu_setting(){
        $d = get_option( 'imagehover_desktop_max_height_mu', 'px' );
        $val = get_option( 'csp_imagehover_desktop_max_height_mu', $d );
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_desktop_max_height_mu" id="csp_imagehover_desktop_max_height_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }
    public function csp_imagehover_desktop_max_height_setting(){
        $d = get_option( 'imagehover_desktop_max_height', '400' );
        $val = get_option( 'csp_imagehover_desktop_max_height', $d );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_desktop_max_height" id="csp_imagehover_desktop_max_height" value="'.$val.'">';
    }

    public function csp_imagehover_tablet_max_width_mu_setting(){
        $d = get_option( 'imagehover_tablet_max_width_mu', 'px' );
        $val = get_option( 'csp_imagehover_tablet_max_width_mu', $d );
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_tablet_max_width_mu" id="csp_imagehover_tablet_max_width_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }
    public function csp_imagehover_tablet_max_width_setting(){
        $d = get_option( 'imagehover_tablet_max_width', '300' );
        $val = get_option( 'csp_imagehover_tablet_max_width', $d );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_tablet_max_width" id="csp_imagehover_tablet_max_width" value="'.$val.'">';
    }

    public function csp_imagehover_tablet_max_height_mu_setting(){
        $d = get_option( 'imagehover_tablet_max_height_mu', 'px' );
        $val = get_option( 'csp_imagehover_tablet_max_height_mu', $d );
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_tablet_max_height_mu" id="csp_imagehover_tablet_max_height_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }
    public function csp_imagehover_tablet_max_height_setting(){
        $d = get_option( 'imagehover_tablet_max_height', '300' );
        $val = get_option( 'csp_imagehover_tablet_max_height', $d );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_tablet_max_height" id="csp_imagehover_tablet_max_height" value="'.$val.'">';
    }

    public function csp_imagehover_phone_max_width_mu_setting(){
        $d = get_option( 'imagehover_phone_max_width_mu', 'px' );
        $val = get_option( 'csp_imagehover_phone_max_width_mu', $d );
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_phone_max_width_mu" id="csp_imagehover_phone_max_width_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }
    public function csp_imagehover_phone_max_width_setting(){
        $d = get_option( 'imagehover_phone_max_width', '100' );
        $val = get_option( 'csp_imagehover_phone_max_width', $d );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_phone_max_width" id="csp_imagehover_phone_max_width" value="'.$val.'">';
    }

    public function csp_imagehover_phone_max_height_mu_setting(){
        $d = get_option( 'imagehover_phone_max_height_mu', 'px' );
        $val = get_option( 'csp_imagehover_phone_max_height_mu', $d );
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_phone_max_height_mu" id="csp_imagehover_phone_max_height_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }
    public function csp_imagehover_phone_max_height_setting(){
        $d = get_option( 'imagehover_phone_max_height', '100' );
        $val = get_option( 'csp_imagehover_phone_max_height', $d );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_phone_max_height" id="csp_imagehover_phone_max_height" value="'.$val.'">';
    }

    // csp space around
    public function csp_imagehover_desktop_horizontal_space_around_setting(){
        $val = get_option( 'csp_imagehover_desktop_horizontal_space_around', '10' );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_desktop_horizontal_space_around" id="csp_imagehover_desktop_horizontal_space_around" value="'.$val.'">';
    }
    public function csp_imagehover_desktop_horizontal_space_around_mu_setting(){
        $val = get_option( 'csp_imagehover_desktop_horizontal_space_around_mu', 'px' );
        
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_desktop_horizontal_space_around_mu" id="csp_imagehover_desktop_horizontal_space_around_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }

    public function csp_imagehover_phone_horizontal_space_around_setting(){
        $val = get_option( 'csp_imagehover_phone_horizontal_space_around', '10' );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_phone_horizontal_space_around" id="csp_imagehover_phone_horizontal_space_around" value="'.$val.'">';
    }
    public function csp_imagehover_phone_horizontal_space_around_mu_setting(){
        $val = get_option( 'csp_imagehover_phone_horizontal_space_around_mu', 'px' );
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_phone_horizontal_space_around_mu" id="csp_imagehover_phone_horizontal_space_around_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }

    public function csp_imagehover_desktop_vertical_space_around_setting(){
        $val = get_option( 'csp_imagehover_desktop_vertical_space_around', '10' );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_desktop_vertical_space_around" id="csp_imagehover_desktop_vertical_space_around" value="'.$val.'">';
    }
    public function csp_imagehover_desktop_vertical_space_around_mu_setting(){
        $val = get_option( 'csp_imagehover_desktop_vertical_space_around_mu', 'px' );
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_desktop_vertical_space_around_mu" id="csp_imagehover_desktop_vertical_space_around_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }

    public function csp_imagehover_phone_vertical_space_around_setting(){
        $val = get_option( 'csp_imagehover_phone_vertical_space_around', '10' );
        echo '<input type="number" min="0" max="" step="1" name="csp_imagehover_phone_vertical_space_around" id="csp_imagehover_phone_vertical_space_around" value="'.$val.'">';
    }
    public function csp_imagehover_phone_vertical_space_around_mu_setting(){
        $val = get_option( 'csp_imagehover_phone_vertical_space_around_mu', 'px' );
        $selected_px = '';
        $selected_percent = '';
        switch( $val ) {
            case 'px':
                $selected_px = 'selected';
            break;
            case '%':
                $selected_percent = 'selected';
            break;
        }
        echo 
        '<select name="csp_imagehover_phone_vertical_space_around_mu" id="csp_imagehover_phone_vertical_space_around_mu">
            <option value="px" '.$selected_px.'>px</option>
            <option value="%" '.$selected_percent.'>%</option>
        </select>'; 
    }

    // csp positions
    public function csp_imagehover_desktop_position_setting() {
        $val = get_option( 'csp_imagehover_desktop_position', 'center' );

        $selected_topleft = '';
        $selected_topcenter = '';
        $selected_topright = '';
        $selected_centerleft = '';
        $selected_center = '';
        $selected_centerright = '';
        $selected_bottomleft = '';
        $selected_bottomcenter = '';
        $selected_bottomright = '';

        switch( $val ) {
            case 'topleft':
                $selected_topleft = 'selected';
            break;
            case 'topcenter':
                $selected_topcenter = 'selected';
            break;
            case 'topright':
                $selected_topright = 'selected';
            break;
            case 'centerleft':
                $selected_centerleft = 'selected';
            break;
            case 'center':
                $selected_center = 'selected';
            break;
            case 'centerright':
                $selected_centerright = 'selected';
            break;
            case 'bottomleft':
                $selected_bottomleft = 'selected';
            break;
            case 'bottomcenter':
                $selected_bottomcenter = 'selected';
            break;
            case 'bottomright':
                $selected_bottomright = 'selected';
            break;
        }

        echo 
        '<select name="csp_imagehover_desktop_position" id="csp_imagehover_desktop_position">
            <option '.$selected_topleft.' value="topleft">Top Left</option>
            <option '.$selected_topcenter.' value="topcenter">Top Center</option>
            <option '.$selected_topright.' value="topright">Top Right</option>
            <option '.$selected_centerleft.' value="centerleft">Center Left</option>
            <option '.$selected_center.' value="center">Center</option>
            <option '.$selected_centerright.' value="centerright">Center Right</option>
            <option '.$selected_bottomleft.' value="bottomleft">Bottom Left</option>
            <option '.$selected_bottomcenter.' value="bottomcenter">Bottom Center</option>
            <option '.$selected_bottomright.' value="bottomright">Bottom Right</option>
        </select>'; 
    }

    public function csp_imagehover_phone_position_setting() {
        $val = get_option( 'csp_imagehover_phone_position', 'bottomright' );

        $selected_topleft = '';
        $selected_topcenter = '';
        $selected_topright = '';
        $selected_centerleft = '';
        $selected_center = '';
        $selected_centerright = '';
        $selected_bottomleft = '';
        $selected_bottomcenter = '';
        $selected_bottomright = '';

        switch( $val ) {
            case 'topleft':
                $selected_topleft = 'selected';
            break;
            case 'topcenter':
                $selected_topcenter = 'selected';
            break;
            case 'topright':
                $selected_topright = 'selected';
            break;
            case 'centerleft':
                $selected_centerleft = 'selected';
            break;
            case 'center':
                $selected_center = 'selected';
            break;
            case 'centerright':
                $selected_centerright = 'selected';
            break;
            case 'bottomleft':
                $selected_bottomleft = 'selected';
            break;
            case 'bottomcenter':
                $selected_bottomcenter = 'selected';
            break;
            case 'bottomright':
                $selected_bottomright = 'selected';
            break;
        }

        echo 
        '<select name="csp_imagehover_phone_position" id="csp_imagehover_phone_position">
            <option '.$selected_topleft.' value="topleft">Top Left</option>
            <option '.$selected_topcenter.' value="topcenter">Top Center</option>
            <option '.$selected_topright.' value="topright">Top Right</option>
            <option '.$selected_centerleft.' value="centerleft">Center Left</option>
            <option '.$selected_center.' value="center">Center</option>
            <option '.$selected_centerright.' value="centerright">Center Right</option>
            <option '.$selected_bottomleft.' value="bottomleft">Bottom Left</option>
            <option '.$selected_bottomcenter.' value="bottomcenter">Bottom Center</option>
            <option '.$selected_bottomright.' value="bottomright">Bottom Right</option>
        </select>'; 
    }

    public function imagehover_widths_and_height_selected_tab_setting() {
        $val = get_option( 'imagehover_widths_and_height_selected_tab', '0' );
        echo '<input type="number" name="imagehover_widths_and_height_selected_tab" id="imagehover_widths_and_height_selected_tab" value="'.$val.'">';
    }
}
new LayThemeImagehover();

require 'plugin_update_check.php';
$KernlUpdater = new PluginUpdateChecker_2_0 (
    'https://kernl.us/api/v1/updates/5ea4015ce8bd8b2fc0e51870/',
    __FILE__,
    'laytheme-imagehover',
    1
);
