<?php
/**
 * Plugin Name: Lay Theme Lightbox
 * Plugin URI: https://laytheme.com/addons/
 * Description: Zoom images by clicking on them. Flick, Swipe or click in the zoomed-mode to navigate through images. Works for desktop and tablet (landscape orientation). Only works with Lay Theme!
 * Version: 1.8.0
 * Author: 100k Studio
 * Author URI: http://100k.studio
 */

class LayThemeLightbox{

	public static $url;
	public static $dir;
	public static $ver;

	public static $activeInProjects;
	public static $activeInPages;
    public static $activeInNews;
	public static $activeInCategories;

	public static $individualProjectIds;
	public static $individualPageIds;
    public static $individualNewsIds;
	public static $individualCategoryIds;

	public function __construct(){
		LayThemeLightbox::$url = plugins_url( 'laytheme-lightbox' );
		LayThemeLightbox::$dir = plugin_dir_path( __FILE__ );
		LayThemeLightbox::$ver = '1.8.0';

		// backend
		add_action( 'admin_menu', array($this, 'lb_setup_menu'), 20 );
		add_action( 'admin_init', array($this, 'lb_settings_api_init') );
		add_filter( 'plugin_action_links_'.plugin_basename(__FILE__), array($this, 'add_action_links') );
		add_action( 'admin_enqueue_scripts', array($this, 'add_lightbox_admin_scripts') );
		// add_action( 'lay_gridder_context_menu_end', array($this, 'add_lightbox_gridder_buttons') );
		// add_action( 'admin_enqueue_scripts', array( $this, 'gridder_lightbox_textmodal_script' ), 20 );

		// frontend
		add_action( 'wp_enqueue_scripts', array( $this, 'lightbox_scripts' ), 11 );
		// add_action( 'wp_enqueue_scripts', array( $this, 'lightbox_styles' ) );
		add_action( 'wp_footer', array( $this, 'lightbox_styles' ) );
		add_action( 'wp_footer', array( $this, 'include_markup' ) );
	}

	public static function set_active_where_vars(){
		LayThemeLightbox::$activeInProjects = get_option( 'lightbox_active_in_projects', "all" );
		LayThemeLightbox::$activeInPages = get_option( 'lightbox_active_in_pages', "" );
        LayThemeLightbox::$activeInNews = get_option( 'lightbox_active_in_news', "" );
		LayThemeLightbox::$activeInCategories = get_option( 'lightbox_active_in_categories', "" );

		// for an unknown reason in rare cases the arrays are saved as associative arrays, so flatten them here
		$individualProjectIds = get_option('lightbox_individual_project_ids', '');
		$individualProjectIds = json_decode( $individualProjectIds, true );
		LayThemeLightbox::$individualProjectIds = json_encode( LayThemeLightbox::flattenArray($individualProjectIds) );

		$individualPageIds = get_option('lightbox_individual_page_ids', '');
		$individualPageIds = json_decode( $individualPageIds, true );
		LayThemeLightbox::$individualPageIds = json_encode( LayThemeLightbox::flattenArray($individualPageIds) );

		$individualNewsIds = get_option('lightbox_individual_news_ids', '');
		$individualNewsIds = json_decode( $individualNewsIds, true );
		LayThemeLightbox::$individualNewsIds = json_encode( LayThemeLightbox::flattenArray($individualNewsIds) );

		$individualCategoryIds = get_option('lightbox_individual_category_ids', '');
		$individualCategoryIds = json_decode( $individualCategoryIds, true );
		LayThemeLightbox::$individualCategoryIds = json_encode( LayThemeLightbox::flattenArray($individualCategoryIds) );
	}

	// public function gridder_lightbox_textmodal_script(){
	// 	$screen = get_current_screen();
	// 	if ( $screen->id == 'post' || $screen->id == 'page' || $screen->id == 'edit-category') {
	// 		wp_enqueue_script( 'laylightbox-admin', LayThemeLightbox::$url.'/backend/assets/js/lightbox.plugin.js', array('jquery'), LayThemeLightbox::$ver );
	// 	}
	// }


	public static function is_active_in_current_admin_page(){
		LayThemeLightbox::set_active_where_vars();

		$isactive = false;
		$screen = get_current_screen();
		$post_id = isset($_GET['post']) ? $_GET['post'] : '';

		$individualProjectIds = json_decode( LayThemeLightbox::$individualProjectIds, true );
		$individualPageIds = json_decode( LayThemeLightbox::$individualPageIds, true );
        $individualNewsIds = json_decode( LayThemeLightbox::$individualNewsIds, true );
        $individualCategoryIds = json_decode( LayThemeLightbox::$individualCategoryIds, true );

        if($screen->id == "post"){
			if(LayThemeLightbox::$activeInProjects == "all" || LayThemeLightbox::$activeInProjects == "on"){
				$isactive = true;
			}
			else if( LayThemeLightbox::$activeInProjects == "individual" && is_array($individualProjectIds) ){
				$isactive = in_array($post_id, $individualProjectIds);
			}
		}
        else if($screen->id == "lay_news"){
			if(LayThemeLightbox::$activeInNews == "all" || LayThemeLightbox::$activeInNews == "on"){
				$isactive = true;
			}
			else if( LayThemeLightbox::$activeInNews == "individual" && is_array($individualNewsIds) ){
				$isactive = in_array($post_id, $individualNewsIds);
			}
		}
		else if($screen->id == "page"){
			if(LayThemeLightbox::$activeInPages == "all" || LayThemeLightbox::$activeInPages == "on"){
				$isactive = true;
			}
			else if( LayThemeLightbox::$activeInPages == "individual" && is_array($individualPageIds) ){
				$isactive = in_array($post_id, $individualPageIds);
			}
		}
		else if($screen->id == "edit-category"){
			if(LayThemeLightbox::$activeInCategories == "all" || LayThemeLightbox::$activeInCategories == "on"){
				$isactive = true;
			}else if( LayThemeLightbox::$activeInCategories == "individual" && is_array($individualCategoryIds) ){
				global $tag;
				if( is_object($tag) ){
					$term_id = $tag->term_id;
					$isactive = in_array($term_id, $individualCategoryIds);
				}
			}
		}

		return $isactive;
	}

	// public static function add_lightbox_gridder_buttons(){
	// 	$isactive = LayThemeLightbox::is_active_in_current_admin_page();
	// 	if($isactive){
	// 		echo
	// 		'<button type="button" class="js-set-lightbox-off btn btn-default"><span class="glyphicon glyphicon-ok"></span> Lightbox off for this image</button>
	// 		<button type="button" class="js-set-lightbox-caption btn btn-default">Edit Lightbox Caption</button>';
	// 	}
	// }

	// http://code.tutsplus.com/articles/how-to-use-wordpress-color-picker-api--wp-33067
	public static function add_lightbox_admin_scripts( $hook ) {
        if( strpos($hook, 'manage-laylightbox') !== false ){
	        wp_enqueue_style( 'wp-color-picker' );
	        wp_enqueue_script( 'lightbox-showhide-settings', LayThemeLightbox::$url.'/backend/assets/js/settings/showhide_settings.js', array('jquery'), LayThemeLightbox::$ver, true );
			wp_enqueue_script( 'lightbox-colorpicker-control', LayThemeLightbox::$url.'/backend/assets/js/settings/colorpicker_control.js', array( 'wp-color-picker' ), LayThemeLightbox::$ver, true );
			wp_enqueue_media();
			wp_enqueue_script( 'lightbox-image-upload', LayThemeLightbox::$url.'/backend/assets/js/settings/image_upload.js', array('jquery'), LayThemeLightbox::$ver, true );
		}
	}

	public static function include_markup(){
		require_once( LayThemeLightbox::$dir."/frontend/markup.php" );
	}

	public static function lightbox_styles(){
		wp_enqueue_style( 'lightbox-style', LayThemeLightbox::$url."/frontend/assets/css/frontend.style.css", array(), LayThemeLightbox::$ver );

		$bgOpacity = (float)get_option( 'lightbox_background_opacity', '90' );

		$bgOpacity /= 100;
		if($bgOpacity > 1){
			$bgOpacity = 1;
		}else if($bgOpacity < 0){
			$bgOpacity = 0;
		}

		$bgColor = get_option( 'lightbox_background_color', '#ffffff' );
		$xColor = get_option( 'lightbox_close_button_color', '#000000' );
		$spaceAround = intval(get_option( 'lightbox_space_around', 50 ));
		$spaceAroundPhone = intval(get_option( 'lightbox_space_around_phone', 30 ));
		$spaceBelow = intval(get_option( 'lightbox_space_below', 50 ));
		$spaceBelowPhone = intval(get_option( 'lightbox_space_below', 30 ));

		$onlyDrag = get_option( 'lightbox_only_drag', "" );
		$onlyDragCSS = "";
		if( $onlyDrag == "on" ){
			$onlyDragCSS =
			"body.lightbox-css-on #lightbox-region #lightbox-inner{ cursor: -webkit-grab; cursor: grab; }
			body.lightbox-css-on #lightbox-region #lightbox-inner.mousedown{ cursor: -webkit-grabbing; cursor: grabbing; }
			body.lightbox-css-on #lightbox-region #lightbox-inner.mousedown img{ cursor: -webkit-grabbing; cursor: grabbing; }";
		}

		$onlyZoom = get_option( 'lightbox_only_zoom', "" );
		$onlyZoomCSS = "";
		if( $onlyZoom == "on" ){
			$onlyZoomCSS =
			"body.lightbox-css-on #lightbox-region #lightbox-inner{ cursor: pointer; }";
			$onlyDragCSS = "";
		}

		$minimalStyle = get_option( 'lightbox_minimal_style', "" );
		$minimalStyleCSS = "";
		if( $minimalStyle == "on" && $onlyDrag != "on"){
			$minimalStyleCSS =
			"body.lightbox-css-on #lightbox-region.minimal-style #lightbox-inner{ cursor: pointer; }";
		}

		$cursorMarkup = '';
		if( $onlyDrag != 'on' ){
			$left_cursor = get_option( 'lightbox_arrowleft', '' );
			if($left_cursor != ''){
				$left_cursor = wp_get_attachment_image_src( $left_cursor, 'full' );
				if($left_cursor){
					// body.lightbox-css-on #lightbox-region.minimal-style #lightbox-inner.cursor-left img
					// body.lightbox-css-on #lightbox-region.normal-style #lightbox-inner.cursor-left
					$cursorMarkup .= 'body.lightbox-css-on #lightbox-region.minimal-style #lightbox-inner.cursor-left img{cursor:url("'.$left_cursor[0].'") '.($left_cursor[1]/2).' '.($left_cursor[2]/2).', w-resize;}';
					$cursorMarkup .= 'body.lightbox-css-on #lightbox-region.normal-style #lightbox-inner.cursor-left{cursor:url("'.$left_cursor[0].'") '.($left_cursor[1]/2).' '.($left_cursor[2]/2).', w-resize;}';
				}
			}

			$right_cursor = get_option( 'lightbox_arrowright', '' );
			if($right_cursor != ''){
				$right_cursor = wp_get_attachment_image_src( $right_cursor, 'full' );
				if($right_cursor){
					// body.lightbox-css-on #lightbox-region.minimal-style #lightbox-inner.cursor-right img
					// body.lightbox-css-on #lightbox-region.normal-style #lightbox-inner.cursor-right
					$cursorMarkup .= 'body.lightbox-css-on #lightbox-region.minimal-style #lightbox-inner.cursor-right img{cursor:url("'.$right_cursor[0].'") '.($right_cursor[1]/2).' '.($right_cursor[2]/2).', e-resize;}';
					$cursorMarkup .= 'body.lightbox-css-on #lightbox-region.normal-style #lightbox-inner.cursor-right{cursor:url("'.$right_cursor[0].'") '.($right_cursor[1]/2).' '.($right_cursor[2]/2).', e-resize;}';
				}
			}
		}

		$breakpoint = get_option('lay_breakpoint', 600);
        $breakpoint = (int)$breakpoint;

		// $lightbox_animation_style = get_option('lightbox_animation_style', 'zoom');

		wp_add_inline_style( 'lightbox-style',
			'/* lightbox css */
			body.lightbox-css-on #lightbox-region .lightbox-bg{
				background-color:'.$bgColor.';
				opacity:'.$bgOpacity.';
			}
			body.lightbox-css-on .close-lightbox.standard-close::before{
				color:'.$xColor.';
			}
			@media (min-width: '.($breakpoint+1).'px){
				body.lightbox-css-on .close-lightbox{
					top: 0;
					right: 0;
					position: fixed;
					padding: 20px;
					z-index: 20;
					cursor: pointer;
					font-size: 0;
				}
				body.lightbox-css-on .close-lightbox.standard-close::before {
					font-size: 26px;
				}
            }
            @media (max-width: '.($breakpoint).'px){
				body.lightbox-css-on .close-lightbox{
					top: 0;
					right: 0;
					position: fixed;
					padding: 20px;
					z-index: 20;
					cursor: pointer;
					font-size: 0;
				}
				body.lightbox-css-on .close-lightbox.standard-close::before {
					font-size: 23px;
				}
            }
			'.$onlyDragCSS
			.$onlyZoomCSS
			.$minimalStyleCSS
			.$cursorMarkup
		);
	}

	public static function flattenArray($arrayIn){
		$arrayOut = array();
		if(is_array($arrayIn)){
			foreach ($arrayIn as $key => $value) {
				$arrayOut []= $arrayIn[$key];
			}
			return $arrayOut;
		}
	}

	public static function lightbox_scripts(){
		wp_enqueue_script( 'lightbox-app', LayThemeLightbox::$url."/frontend/assets/js/lightbox.plugin.min.js", array( 'swiper' ), LayThemeLightbox::$ver, true);

		$activeInProjects = get_option( 'lightbox_active_in_projects', "all" );
		$activeInPages = get_option( 'lightbox_active_in_pages', "" );
		$activeInCategories = get_option( 'lightbox_active_in_categories', "" );

		LayThemeLightbox::set_active_where_vars();

		$onlyZoom = get_option( 'lightbox_only_zoom', '' );
		$onlyDrag = get_option( 'lightbox_only_drag', '' );

		$lightbox_sliding_duration = get_option( 'lightbox_sliding_duration', '1000' );

		$spaceAround = intval(get_option( 'lightbox_space_around', 50 ));
		$spaceBelow = intval(get_option( 'lightbox_space_below', 50 ));

		$spaceAroundPhone = intval(get_option( 'lightbox_space_around_phone', 30 ));
		$spaceBelowPhone = intval(get_option( 'lightbox_space_below_phone', 30 ));

		$lightbox_space_between_image_and_caption = intval(get_option( 'lightbox_space_between_image_and_caption', 50 ));
		$lightbox_space_between_image_and_caption_phone = intval(get_option( 'lightbox_space_between_image_and_caption_phone', 10 ));

		$showOriginalImages = get_option( 'lightbox_showoriginalimages', '' );

		$breakpoint = get_option('lay_breakpoint', 600);

		$activeOnPhone = get_option( 'lightbox_activate_on_phone', '' );
		$animation_style = get_option('lightbox_animation_style', 'zoom');

		wp_localize_script( 'lightbox-app', 'layLightboxPassedData',
			array(
				'minimalStyle' => get_option( 'lightbox_minimal_style', "" ),
				'activeInProjects' => LayThemeLightbox::$activeInProjects,
				'activeInPages' => LayThemeLightbox::$activeInPages,
				'activeInCategories' => LayThemeLightbox::$activeInCategories,
                'activeInNews' => LayThemeLightbox::$activeInNews,
				'individualProjectIds' => LayThemeLightbox::$individualProjectIds,
				'individualPageIds' => LayThemeLightbox::$individualPageIds,
				'individualCategoryIds' => LayThemeLightbox::$individualCategoryIds,
                'individualNewsIds' => LayThemeLightbox::$individualNewsIds,
				'onlyZoom' => $onlyZoom,
				'onlyDrag' => $onlyDrag,
				'slidingDuration' => $lightbox_sliding_duration,
				'spaceAround' => $spaceAround,
				'spaceAroundPhone' => $spaceAroundPhone,
				'spaceBelow' => $spaceBelow,
				'spaceBelowPhone' => $spaceBelowPhone,
				'space_between_image_and_caption' => $lightbox_space_between_image_and_caption,
				'space_between_image_and_caption_phone' => $lightbox_space_between_image_and_caption_phone,
				'showOriginalImages' => $showOriginalImages,
				'breakpoint' => $breakpoint,
				'activeOnPhone' => $activeOnPhone,
				'lightbox_animation_style' => $animation_style,
                'scrollBackground' => get_option( 'lightbox_scroll_background', "on" )
			)
		);
	}

	public function add_action_links( $links ){
		$mylinks = array(
			'<a href="' . admin_url( 'admin.php?page=manage-laylightbox' ) . '">Settings</a>',
		);
		return array_merge( $links, $mylinks );
	}

	public function lb_settings_api_init(){
	 	add_settings_section(
			'lightboxoptions_section',
			'Active',
			'',
			'manage-laylightbox'
		);

        add_settings_section(
			'lightboxoptions_captions_section',
			'Captions',
			'',
			'manage-laylightbox'
		);

		add_settings_section(
			'lightboxoptions_style_section',
			'Style',
			'',
			'manage-laylightbox'
		);

		add_settings_section(
			'lightboxoptions_phone_section',
			'Phone Options',
			'',
			'manage-laylightbox'
		);

        // captions
        add_settings_field(
			'lightbox_default_caption',
			'Default Caption',
			array($this, 'lb_default_caption_setting'),
			'manage-laylightbox',
			'lightboxoptions_captions_section'
		);
	 	register_setting( 'manage-laylightbox', 'lightbox_default_caption' );

         add_settings_field(
            'lightbox_captionTextformat',
            'Textformat for Default Caption',
            array($this, 'lightbox_captionTextformat_setting'),
            'manage-laylightbox',
            'lightboxoptions_captions_section'
        );
        register_setting( 'manage-laylightbox', 'lightbox_captionTextformat' );
         

        add_settings_field(
            'lightbox_space_below',
            'Space below Image for Captions (only applies if image has caption)',
            array($this, 'lightbox_space_below_setting'),
            'manage-laylightbox',
            'lightboxoptions_captions_section'
        );
        register_setting( 'manage-laylightbox', 'lightbox_space_below' );

        add_settings_field(
            'lightbox_space_between_image_and_caption',
            'Space Between Image and Caption',
            array($this, 'lightbox_space_between_image_and_caption_setting'),
            'manage-laylightbox',
            'lightboxoptions_captions_section'
        );
        register_setting( 'manage-laylightbox', 'lightbox_space_between_image_and_caption' );

        // active where
        add_settings_field(
			'lightbox_active_in_projects',
			'Lightbox for Projects',
			array($this, 'lb_active_in_projects_setting'),
			'manage-laylightbox',
			'lightboxoptions_section'
		);
	 	register_setting( 'manage-laylightbox', 'lightbox_active_in_projects' );

 	 	add_settings_field(
 			'lightbox_active_in_pages',
 			'Lightbox for Pages',
 			array($this, 'lb_active_in_pages_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_section'
 		);
 	 	register_setting( 'manage-laylightbox', 'lightbox_active_in_pages' );

          add_settings_field(
			'lightbox_active_in_news',
			'Lightbox for News',
			array($this, 'lb_active_in_news_setting'),
			'manage-laylightbox',
			'lightboxoptions_section'
		);
	 	register_setting( 'manage-laylightbox', 'lightbox_active_in_news' );

 	 	add_settings_field(
 			'lightbox_active_in_categories',
 			'Lightbox for Categories',
 			array($this, 'lb_active_in_categories_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_section'
 		);
 	 	register_setting( 'manage-laylightbox', 'lightbox_active_in_categories' );

		add_settings_field(
			'lightbox_animation',
			'Opening/Closing Animation',
			array($this, 'lightbox_animation_style_setting'),
			'manage-laylightbox',
			'lightboxoptions_style_section'
		);
		register_setting( 'manage-laylightbox', 'lightbox_animation_style' );

 	 	add_settings_field(
 			'lightbox_minimal_style',
 			'Minimal Style',
 			array($this, 'lightbox_minimal_style_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_style_section'
 		);
 	 	register_setting( 'manage-laylightbox', 'lightbox_minimal_style' );

 	 	add_settings_field(
 			'lightbox_only_zoom',
 			'Only Open & Close',
 			array($this, 'lightbox_only_zoom_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_style_section'
 		);
 	 	register_setting( 'manage-laylightbox', 'lightbox_only_zoom' );

 	 	add_settings_field(
 			'lightbox_only_drag',
 			'"Drag hand" cursor',
 			array($this, 'lightbox_only_drag_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_style_section'
 		);
		register_setting( 'manage-laylightbox', 'lightbox_only_drag' );
		  
		add_settings_field(
			'lightbox_arrowleft',
			'Custom Mouse Cursor "Previous"',
			array($this, 'lightbox_arrowleft_setting'),
			'manage-laylightbox',
			'lightboxoptions_style_section'
		);
		register_setting( 'manage-laylightbox', 'lightbox_arrowleft' );

		add_settings_field(
			'lightbox_arrowright',
			'Custom Mouse Cursor "Next"',
			array($this, 'lightbox_arrowright_setting'),
			'manage-laylightbox',
			'lightboxoptions_style_section'
		);
		register_setting( 'manage-laylightbox', 'lightbox_arrowright' );

 	 	add_settings_field(
 			'lightbox_close_button_color',
 			'"X" Close-Icon Color',
 			array($this, 'lightbox_close_button_color_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_style_section'
 		);
		register_setting( 'manage-laylightbox', 'lightbox_close_button_color' );

		add_settings_field(
			'lightbox_use_custom_close_button',
			'Use Custom "X" Close-Icon',
			array($this, 'lightbox_use_custom_close_button_setting'),
			'manage-laylightbox',
			'lightboxoptions_style_section'
		);
		register_setting( 'manage-laylightbox', 'lightbox_use_custom_close_button' );

		add_settings_field(
			'lightbox_close_icon',
			'Custom "X" Close-Icon',
			array($this, 'lightbox_close_icon_setting'),
			'manage-laylightbox',
			'lightboxoptions_style_section'
		);
		register_setting( 'manage-laylightbox', 'lightbox_close_icon' );

 	 	add_settings_field(
 			'lightbox_background_color',
 			'Background Color',
 			array($this, 'lightbox_background_color_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_style_section'
 		);
 	 	register_setting( 'manage-laylightbox', 'lightbox_background_color' );

 	 	add_settings_field(
 			'lightbox_background_opacity',
 			'Background Opacity',
 			array($this, 'lightbox_background_opacity_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_style_section'
 		);
 	 	register_setting( 'manage-laylightbox', 'lightbox_background_opacity' );

 	 	add_settings_field(
 			'lightbox_space_around',
 			'Space Around Image',
 			array($this, 'lightbox_space_around_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_style_section'
 		);
 	 	register_setting( 'manage-laylightbox', 'lightbox_space_around' );

 	 	add_settings_field(
 			'lightbox_sliding_duration',
 			'Sliding Duration',
 			array($this, 'lightbox_sliding_duration_setting'),
 			'manage-laylightbox',
 			'lightboxoptions_style_section'
 		);
 	 	register_setting( 'manage-laylightbox', 'lightbox_sliding_duration' );

          add_settings_field(
            'lightbox_scroll_background',
            'Scroll Background Page',
            array($this, 'lightbox_scroll_background_setting'),
            'manage-laylightbox',
            'lightboxoptions_style_section'
        );
         register_setting( 'manage-laylightbox', 'lightbox_scroll_background' );
		  
 	 	add_settings_field(
			'lightbox_activate_on_phone',
			'Activate on Phone',
			array($this, 'lightbox_activate_on_phone_setting'),
			'manage-laylightbox',
			'lightboxoptions_phone_section'
		);
		register_setting( 'manage-laylightbox', 'lightbox_activate_on_phone' );


		add_settings_field(
			'lightbox_space_around_phone',
			'Phone Space Around Image',
			array($this, 'lightbox_space_around_phone_setting'),
			'manage-laylightbox',
			'lightboxoptions_phone_section'
		);
		register_setting( 'manage-laylightbox', 'lightbox_space_around_phone' );

		add_settings_field(
			'lightbox_space_below_phone',
			'Space below Image for Captions (only applies if image has caption)',
			array($this, 'lightbox_space_below_phone_setting'),
			'manage-laylightbox',
			'lightboxoptions_phone_section'
		);
		register_setting( 'manage-laylightbox', 'lightbox_space_below_phone' );

		add_settings_field(
			'lightbox_space_between_image_and_caption_phone',
			'Phone Space Between Image and Caption',
			array($this, 'lightbox_space_between_image_and_caption_phone_setting'),
			'manage-laylightbox',
			'lightboxoptions_phone_section'
		);
		register_setting( 'manage-laylightbox', 'lightbox_space_between_image_and_caption_phone' );

	}

    public function lightbox_captionTextformat_setting(){
		// display a selectable list of textformats
		$val = get_option( 'lightbox_captionTextformat', "Default" );

		echo '<select name="lightbox_captionTextformat" id="lightbox_captionTextformat">';
		foreach (Customizer::$textformatsSelect as $format) {
			$selected = ($val == $format) ? 'selected' : '';
			echo '<option value="'.$format.'" '.$selected.'>'.$format.'</option>';
		}	
		echo '</select>'; 
	}

	public function lightbox_space_between_image_and_caption_setting(){
		$val = get_option( 'lightbox_space_between_image_and_caption', '10' );
		echo '<input type="number" min="" max="" step="1" name="lightbox_space_between_image_and_caption" id="lightbox_space_between_image_and_caption" value="'.$val.'"> <label for="lightbox_space_between_image_and_caption">px</label>';
	}

	public function lightbox_space_between_image_and_caption_phone_setting(){
		$val = get_option( 'lightbox_space_between_image_and_caption_phone', 10 );
		echo '<input type="number" min="" max="" step="1" name="lightbox_space_between_image_and_caption_phone" id="lightbox_space_between_image_and_caption_phone" value="'.$val.'"> <label for="lightbox_space_between_image_and_caption_phone">px</label>';
	}

	public function lightbox_space_below_setting(){
		$val = get_option( 'lightbox_space_below', '50' );
		echo '<input type="number" min="0" max="" step="1" name="lightbox_space_below" id="lightbox_space_below" value="'.$val.'"> <label for="lightbox_space_below">px</label>
        <br><p>If this space is too small your caption will be cut off.</p>';
	}

	public function lightbox_space_below_phone_setting(){
		$val = get_option( 'lightbox_space_below_phone', 30 );
		echo '<input type="number" min="0" max="" step="1" name="lightbox_space_below_phone" id="lightbox_space_below_phone" value="'.$val.'"> <label for="lightbox_space_below_phone">px</label>
        <br><p>If this space is too small your caption will be cut off.';
	}

	public function lightbox_space_around_setting(){
		$val = get_option( 'lightbox_space_around', '50' );
		echo '<input type="number" min="0" max="" step="1" name="lightbox_space_around" id="lightbox_space_around" value="'.$val.'"> <label for="lightbox_space_around">px</label>';
	}

	public function lightbox_space_around_phone_setting(){
		$val = get_option( 'lightbox_space_around_phone', 30 );
		echo '<input type="number" min="0" max="" step="1" name="lightbox_space_around_phone" id="lightbox_space_around_phone" value="'.$val.'"> <label for="lightbox_space_around_phone">px</label>';
	}

	public function lightbox_sliding_duration_setting(){
		$val = get_option( 'lightbox_sliding_duration', '1000' );
		echo '<input type="number" min="0" max="" step="1" name="lightbox_sliding_duration" id="lightbox_sliding_duration" value="'.$val.'"> <label for="lightbox_sliding_duration">milliseconds</label>';
	}

	public function lightbox_background_opacity_setting(){
		$val = get_option( 'lightbox_background_opacity', '90' );
		echo '<input type="number" min="0" max="100" name="lightbox_background_opacity" id="lightbox_background_opacity" value="'.$val.'"> <label for="lightbox_background_opacity">%</label>';
	}

	public function lightbox_background_color_setting(){
		$val = get_option( 'lightbox_background_color', '#ffffff' );
		echo '<input type="text" name="lightbox_background_color" id="lightbox_background_color" value="'.$val.'" class="laylightbox-color-picker">';
	}

	public function lightbox_use_custom_close_button_setting(){
		$val = get_option( 'lightbox_use_custom_close_button', "" );
		$checked = "";
		if($val == "on"){
			$checked = "checked";
		}
		echo
		'<input type="checkbox" name="lightbox_use_custom_close_button" id="lightbox_use_custom_close_button" '.$checked.'>';
	}

    public function lightbox_scroll_background_setting(){
        $val = get_option( 'lightbox_scroll_background', "on" );
        $checked = "";
        if($val == "on"){
            $checked = "checked";
        }
        echo
        '<input type="checkbox" name="lightbox_scroll_background" id="lightbox_scroll_background" '.$checked.'>';
    }

	public static function lightbox_arrow_setting($option_name){
		// https://gist.github.com/hlashbrooke/9267467#file-class-php-L324
		$image_thumb_id = get_option( $option_name, '' );
		$image_thumb = "";
		$noimage_image = $image_thumb;
		$hideRemoveButtonCSS = '';
		if($image_thumb_id != ''){
			$image_thumb = wp_get_attachment_image_src( $image_thumb_id, 'full' );
			$image_thumb = $image_thumb[0];
		}
		else{
			$hideRemoveButtonCSS = 'style="display:none;"';
		}
		echo 
		'<img id="'.$option_name.'_preview" style="max-width: 200px;" class="image_preview" data-noimagesrc="'.$noimage_image.'" src="'.$image_thumb.'">
		<p style="margin-bottom: 10px;">Max. size: 128 * 128 px</p>
		<input id="'.$option_name.'_button" style="margin-bottom: 5px;" type="button" class="image_upload_button button" value="Set image" /><br>
		<input id="'.$option_name.'_delete" '.$hideRemoveButtonCSS.' type="button" class="image_delete_button button" value="Remove image" /><br>
		<input id="'.$option_name.'" class="image_data_field" type="hidden" name="'.$option_name.'" value="'.$image_thumb_id.'"/>';
	}

	public function lightbox_arrowright_setting(){
		LayThemeLightbox::lightbox_arrow_setting('lightbox_arrowright');
	}

	public function lightbox_arrowleft_setting(){
		LayThemeLightbox::lightbox_arrow_setting('lightbox_arrowleft');
	}

	public function lightbox_close_icon_setting(){
		LayThemeLightbox::laylightbox_button_setting('lightbox_close_icon');
	}

	public static function laylightbox_button_setting($option_name){
		// https://gist.github.com/hlashbrooke/9267467#file-class-php-L324
		$image_thumb_id = get_option( $option_name, '' );
		$image_thumb = "";
		$noimage_image = $image_thumb;
		$hideRemoveButtonCSS = '';
		if($image_thumb_id != ''){
			$image_thumb = wp_get_attachment_image_src( $image_thumb_id, 'full' );
			$image_thumb = $image_thumb[0];
		}
		else{
			$hideRemoveButtonCSS = 'style="display:none;"';
		}
		echo 
		'<img id="'.$option_name.'_preview" style="max-width: 200px;" class="image_preview" data-noimagesrc="'.$noimage_image.'" src="'.$image_thumb.'"><br>
		<input id="'.$option_name.'_button" style="margin-bottom: 5px;" type="button" class="image_upload_button button" value="Set image" /><br>
		<input id="'.$option_name.'_delete" '.$hideRemoveButtonCSS.' type="button" class="image_delete_button button" value="Remove image" /><br>
		<input id="'.$option_name.'" class="image_data_field" type="hidden" name="'.$option_name.'" value="'.$image_thumb_id.'"/>';
	}

	public function lightbox_close_button_color_setting(){
		$val = get_option( 'lightbox_close_button_color', '#000000' );
		echo '<input type="text" name="lightbox_close_button_color" id="lightbox_close_button_color" value="'.$val.'" class="laylightbox-color-picker">';
	}

    public function lb_default_caption_setting(){
		$val = get_option( 'lightbox_default_caption', "none" );
		$checked1 = "";
		$checked2 = "";
		$checked3 = "";
        $checked4 = "";

		if($val == "none"){
			$checked1 = "checked";
		}
		else if($val == "caption"){
			$checked2 = "checked";
		}
		else if($val == "title"){
			$checked3 = "checked";
		}
        else if($val == "elementcaption"){
			$checked4 = "checked";
		}

		echo
		'<input type="radio" name="lightbox_default_caption" value="none" '.$checked1.' id="caption-none"><label for="caption-none">None</label><br>
		<input type="radio" name="lightbox_default_caption" value="caption" '.$checked2.' id="caption-caption"><label for="caption-caption">Image File Caption</label><br>
		<input type="radio" name="lightbox_default_caption" value="title" '.$checked3.' id="caption-title"><label for="caption-title">Image File Title</label><br>
        <input type="radio" name="lightbox_default_caption" value="elementcaption" '.$checked4.' id="caption-elementcaption"><label for="caption-elementcaption">Gridder Image Element Caption (Setting below "Textformat for Default Caption" will be ignored)</label>';
    }

	public function lb_active_in_categories_setting(){
		$val = get_option( 'lightbox_active_in_categories', "" );
		$checkedAll = "";
		$checkedIndividual = "";
		$checkedOff = "";

		if($val == "on" || $val == "all"){
			$checkedAll = "checked";
		}
		else if($val == "individual"){
			$checkedIndividual = "checked";
		}
		else if($val == "off" || $val == ""){
			$checkedOff = "checked";
		}

		echo
		'<input type="radio" name="lightbox_active_in_categories" value="all" '.$checkedAll.' id="all-categories"><label for="all-categories">Active for all Categories</label><br>
		<input type="radio" name="lightbox_active_in_categories" value="individual" '.$checkedIndividual.' id="individual-categories"><label for="individual-categories">Show a checkbox in the Category Edit Screen to activate Lightbox for individual Categories</label><br>
		<input type="radio" name="lightbox_active_in_categories" value="off" '.$checkedOff.' id="off-categories"><label for="off-categories">Off for all Categories</label>';
	}

	public function lb_active_in_projects_setting(){
		$val = get_option( 'lightbox_active_in_projects', "all" );
		$checkedAll = "";
		$checkedIndividual = "";
		$checkedOff = "";

		if($val == "on" || $val == "all"){
			$checkedAll = "checked";
		}
		else if($val == "individual"){
			$checkedIndividual = "checked";
		}
		else if($val == "off" || $val == ""){
			$checkedOff = "checked";
		}

		echo
		'<input type="radio" name="lightbox_active_in_projects" value="all" '.$checkedAll.' id="all-projects"><label for="all-projects">Active for all Projects</label><br>
		<input type="radio" name="lightbox_active_in_projects" value="individual" '.$checkedIndividual.' id="individual-projects"><label for="individual-projects">Show a checkbox in the Project Edit Screen to turn on the Lightbox for individual Projects</label><br>
		<input type="radio" name="lightbox_active_in_projects" value="off" '.$checkedOff.' id="off-projects"><label for="off-projects">Off for all Projects</label>';
	}

    public function lb_active_in_news_setting(){
		$val = get_option( 'lightbox_active_in_news', "" );
		$checkedAll = "";
		$checkedIndividual = "";
		$checkedOff = "";

		if($val == "on" || $val == "all"){
			$checkedAll = "checked";
		}
		else if($val == "individual"){
			$checkedIndividual = "checked";
		}
		else if($val == "off" || $val == ""){
			$checkedOff = "checked";
		}

		echo
		'<input type="radio" name="lightbox_active_in_news" value="all" '.$checkedAll.' id="all-news"><label for="all-news">Active for all News</label><br>
		<input type="radio" name="lightbox_active_in_news" value="individual" '.$checkedIndividual.' id="individual-news"><label for="individual-news">Show a checkbox in the News Edit Screen to turn on the Lightbox for individual News</label><br>
		<input type="radio" name="lightbox_active_in_news" value="off" '.$checkedOff.' id="off-news"><label for="off-news">Off for all News</label>';
    }

	public function lb_active_in_pages_setting(){
		$val = get_option( 'lightbox_active_in_pages', "" );
		$checkedAll = "";
		$checkedIndividual = "";
		$checkedOff = "";

		if($val == "on" || $val == "all"){
			$checkedAll = "checked";
		}
		else if($val == "individual"){
			$checkedIndividual = "checked";
		}
		else if($val == "off" || $val == ""){
			$checkedOff = "checked";
		}

		echo
		'<input type="radio" name="lightbox_active_in_pages" value="all" '.$checkedAll.' id="all-pages"><label for="all-pages">Active for all Pages</label><br>
		<input type="radio" name="lightbox_active_in_pages" value="individual" '.$checkedIndividual.' id="individual-pages"><label for="individual-pages">Show a checkbox in the Page Edit Screen to turn on the Lightbox for individual Pages</label><br>
		<input type="radio" name="lightbox_active_in_pages" value="off" '.$checkedOff.' id="off-pages"><label for="off-pages">Off for all Pages</label>';
	}

	public function lightbox_only_zoom_setting(){
		$val = get_option( 'lightbox_only_zoom', "" );
		$checked = "";
		if($val == "on"){
			$checked = "checked";
		}
		echo
		'<input type="checkbox" name="lightbox_only_zoom" id="lightbox_only_zoom" '.$checked.'>
		<label for="lightbox_only_zoom">There will be no navigation to next or previous images when clicking. Clicking anywhere on the lightbox will close it.</label>';
	}

	public function lightbox_only_drag_setting(){
		$val = get_option( 'lightbox_only_drag', "" );
		$checked = "";
		if($val == "on"){
			$checked = "checked";
		}
		echo
		'<input type="checkbox" name="lightbox_only_drag" id="lightbox_only_drag" '.$checked.'>
		<label for="lightbox_only_drag">The lightbox will have a "hand" cursor and you can only drag to navigate. Clicking anywhere will close the lightbox.<br>"Only Open & Close" needs to be deactivated for this to work.</label>';
	}


	public function lightbox_activate_on_phone_setting(){
		$val = get_option( 'lightbox_activate_on_phone', "" );
		$checked = "";
		if($val == "on"){
			$checked = "checked";
		}
		echo
		'<input type="checkbox" name="lightbox_activate_on_phone" id="lightbox_activate_on_phone" '.$checked.'>
		<label for="lightbox_activate_on_phone">Activate for phone version</label>';
	}

	public function lightbox_minimal_style_setting(){
		$isMinimal = get_option( 'lightbox_minimal_style', "" );
		$checked = "";
		if($isMinimal == "on"){
			$checked = "checked";
		}
		echo
		'<input type="checkbox" name="lightbox_minimal_style" id="lightbox_minimal_style" '.$checked.'>
		<label for="lightbox_minimal_style">With Minimal Style activated there is no "X" Close-Icon. Also you navigate by clicking on the image and a click on the background closes the lightbox.</label>';
	}

	public function lightbox_animation_style_setting(){
		$val = get_option('lightbox_animation_style', 'zoom');

		$selected1 = $val == 'zoom' ? 'selected' : '';
		$selected2 = $val == 'fade' ? 'selected' : '';
		$selected3 = $val == 'none' ? 'selected' : '';

		echo 
		'<select name="lightbox_animation_style">
			<option value="zoom" '.$selected1.'>Zoom</option> 
			<option value="fade" '.$selected2.'>Fade</option>
			<option value="none" '.$selected3.'>None</option>
	  	</select>';	
	}

	public function lb_setup_menu(){
		add_submenu_page( 'manage-layoptions', 'Lightbox Addon', 'Lightbox Addon', 'manage_options', 'manage-laylightbox', array($this, 'lb_options_markup') );
	}

	public function lb_options_markup(){
		require_once( LayThemeLightbox::$dir.'/backend/lightbox_options_markup.php' );
	}
}
new LayThemeLightbox();

require_once( plugin_dir_path( __FILE__ ).'/backend/individual_metaboxes.php' );

require 'plugin_update_check.php';
$MyUpdateChecker = new PluginUpdateChecker_2_0 (
    'https://kernl.us/api/v1/updates/56b3219b681396fc05d44cf8/',
    __FILE__,
    'laytheme-lightbox',
    1
);
